@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: main.cpp,v 2.3 1998/06/16 11:39:31 warwick Exp $
+** $Id: qt/examples/aclock/main.cpp   2.1.1   edited 2000-03-28 $
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -16,8 +16,12 @@ int main( int argc, char **argv )
 {
     QApplication a( argc, argv );
     AnalogClock *clock = new AnalogClock;
+    if ( argc == 2 && strcmp( argv[1], "-transparent" ) == 0 )
+	clock->setAutoMask( TRUE );
     clock->resize( 100, 100 );
     a.setMainWidget( clock );
     clock->show();
-    return a.exec();
+    int result = a.exec();
+    delete clock;
+    return result;
 }
