@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qnp.cpp,v 1.23.2.1 1998/09/22 10:43:59 hanord Exp $
+** $Id: qt/extensions/nsplugin/src/qnp.cpp   2.1.1   edited 2000-04-26 $
 **
 ** Implementation of Qt extension classes for Netscape Plugin support.
 **
 ** Created : 970601
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -56,12 +58,9 @@
 #include <qobjectlist.h>
 
 #include <qprinter.h>
-#ifdef _WS_X11_
-#include <qpsprinter.h>
-#endif
 #include <qfile.h>
 #include <qpainter.h>
-#include <qpaintdevicedefs.h>
+#include <q1xcompatibility.h>
 
 #include "qnp.h"
 
@@ -178,7 +177,7 @@ typedef void (*ForeignEventProc)(XEvent*);
 extern XtEventDispatchProc
  qt_np_cascade_event_handler[LASTEvent];      // defined in qnpsupport.cpp
 void            qt_reset_color_avail();       // defined in qcolor_x11.cpp
-void            qt_activate_timers();         // defined in qapplication_x11.cpp
+int             qt_activate_timers();         // defined in qapplication_x11.cpp
 timeval        *qt_wait_timer();              // defined in qapplication_x11.cpp
 void		qt_x11SendPostedEvents();     // defined in qapplication_x11.cpp
 Boolean  qt_event_handler( XEvent* event );   // defined in qnpsupport.cpp
@@ -301,7 +300,7 @@ public:
 	POINT curPos;
 	if ( GetCursorPos( &curPos ) ) {
 	    QPoint p(curPos.x, curPos.y);
-	
+
 	    QNPWidget *newFocussedWidget = 0;
 	    for ( QNPWidget* npw = npwidgets.first();
 		npw; npw = npwidgets.next() )
@@ -315,20 +314,20 @@ public:
 	    }
 	    if (newFocussedWidget != focussedWidget && focussedWidget)
 		focussedWidget->leaveInstance();
-	
+
 	    if (newFocussedWidget) {
 		if (newFocussedWidget != focussedWidget)
 		    newFocussedWidget->enterInstance();
 	    }
-	
+
 	    focussedWidget = newFocussedWidget;
 	}
     }
 
     bool notify( QObject* obj, QEvent* event )
     {
-	if ( event->type() == Event_Enter ||
-	     event->type() == Event_Leave )
+	if ( event->type() == QEvent::Enter ||
+	     event->type() == QEvent::Leave )
 	{
 	    checkFocussedWidget();
 	}
@@ -719,15 +718,10 @@ NPP_SetWindow(NPP instance, NPWindow* window)
     } else if (This->widget) {
 	// ### Maybe need a geometry setter that bypasses some Qt code?
 	// ### position is always (0,0), so we get by by ignoring it.
-	//This->widget->setGeometry(window->x,window->y, window->width, window->height);
 	if ( This->widget->width() != (int)window->width
 	  || This->widget->height() != (int)window->height )
 	{
-//#ifdef _WS_WIN_
-	    //This->widget->setGeometry(window->x, window->y, window->width, window->height);
-//#else
-	    This->widget->resize(window->width, window->height);
-//#endif
+	    This->widget->setGeometry(window->x, window->y, window->width, window->height);
 	} else {
 	    This->widget->update();
 	}
@@ -875,13 +869,13 @@ public:
 	file.open(IO_WriteOnly, fp);
 	QPDevCmdParam param;
 	param.device = &file;
-	cmd(PDC_SETDEV, 0, &param);
+	cmd(PdcSetdev, 0, &param);
     }
     void end()
     {
 	QPDevCmdParam param;
 	param.device = 0;
-	cmd(PDC_SETDEV, 0, &param);
+	cmd(PdcSetdev, 0, &param);
     }
 };
 #endif
@@ -927,11 +921,31 @@ NPP_Print(NPP instance, NPPrint* printInfo)
 }
 
 extern "C" void
-NPP_URLNotify(NPP /*instance*/,
-    const char* /*url*/,
-    NPReason /*reason*/,
-    void* /*notifyData*/)
+NPP_URLNotify(NPP instance,
+              const char* url,
+              NPReason reason,
+              void* notifyData)
 {
+    debug("NPP_URLNotify(%s,%d)",url,reason);
+    if (instance != NULL) {
+        QNPInstance::Reason r;
+        switch (reason) {
+        case NPRES_DONE:
+            r = QNPInstance::ReasonDone;
+            break;
+        case NPRES_USER_BREAK:
+            r = QNPInstance::ReasonBreak;
+            break;
+        case NPRES_NETWORK_ERR:
+            r = QNPInstance::ReasonError;
+            break;
+        default:
+            r = QNPInstance::ReasonUnknown;
+            break;
+        }
+        _NPInstance* This = (_NPInstance*) instance->pdata;
+        This->instance->notifyURL(url, r, notifyData);
+    }
 }
 
 
@@ -1079,13 +1093,13 @@ BOOL   WINAPI   DllMain (HANDLE hInst,
     switch ( ul_reason_for_call ) {
 	case DLL_PROCESS_ATTACH:
 	case DLL_THREAD_ATTACH:
-	    WinMain( hInst, 0, "", SW_SHOW );
+	    WinMain( (HINSTANCE)hInst, 0, "", SW_SHOW );
 	    break;
 	case DLL_PROCESS_DETACH:
 	case DLL_THREAD_DETACH:
 	    break;
     }
-	
+
     return TRUE;
 }
 
@@ -1101,7 +1115,7 @@ int main(int argc, char** argv)
 /*!
   \class QNPWidget qnp.h
   \brief A QWidget that is a Web-browser plugin window
-  
+
   \extension NSPlugin
 
   Derive from QNPWidget to create a widget that can be used as a
@@ -1140,6 +1154,17 @@ public:
   The default implementation is an empty window.
 */
 
+/*! \enum QNPWidget::InstanceMode
+
+  This enum type provides C++-friendly names for three #defines in
+  npapi.h: <ul>
+
+  <li> \c Embed - corresponds to NP_EMBED
+  <li> \c Full - corresponds to NP_FULL
+  <li> \c Background - corresponds to NP_BACKGROUND
+
+  </ul>
+*/
 /*!
   Creates a QNPWidget.
 */
@@ -1147,7 +1172,7 @@ QNPWidget::QNPWidget() :
     pi(next_pi)
 {
     if (!next_pi) {
-	fatal("QNPWidget must only be created within call to newWindow");
+	qFatal("QNPWidget must only be created within call to newWindow");
     }
     next_pi->widget = this;
     next_pi = 0;
@@ -1168,7 +1193,7 @@ QNPWidget::QNPWidget() :
   when the window is no longer required.  The Web-browser will delete windows
   when they leave the page.  The bindings will change the QWidget::winId()
   of the window when the window is resized, but this should not affect
-  normal widget behaviour.
+  normal widget behavior.
 */
 QNPWidget::~QNPWidget()
 {
@@ -1204,7 +1229,7 @@ public:
 	QRect g = geometry();
 	QColor bg = backgroundColor();
 	bool mt = hasMouseTracking();
-	bool hascurs = testWFlags( WCursorSet );
+	bool hascurs = testWFlags( WState_OwnCursor );
 	QCursor curs = cursor();
 	clearWFlags( WState_Created );
 	clearWFlags( WState_Visible );
@@ -1273,11 +1298,7 @@ void QNPWidget::setWindow(bool delold)
 
     createNewWindowsForAllChildren(this);
 
-//#ifdef _WS_WIN_
-    //setGeometry( pi->x, pi->y, pi->width, pi->height );
-//#else
-    resize( pi->width, pi->height );
-//#endif
+    setGeometry( pi->x, pi->y, pi->width, pi->height );
 }
 
 /*!
@@ -1342,7 +1363,7 @@ QNPInstance::QNPInstance() :
     pi(next_pi)
 {
     if (!next_pi) {
-	fatal("QNPInstance must only be created within call to newInstance");
+	qFatal("QNPInstance must only be created within call to newInstance");
     }
     next_pi->instance = this;
     next_pi = 0;
@@ -1388,11 +1409,8 @@ QNPWidget* QNPInstance::widget()
   by the most recent call to writeReady().
 
   Note that the AsFileOnly method is not supported by Netscape 2.0
-  and MS-Explorer 3.0.  A future version of Qt will hide this problem.
+  and MS-Explorer 3.0.
 
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppnewstream>
-  Netscape: NPP_NewStream method</a>
 */
 bool QNPInstance::newStreamCreated(QNPStream*, StreamMode&)
 {
@@ -1402,15 +1420,11 @@ bool QNPInstance::newStreamCreated(QNPStream*, StreamMode&)
 /*!
   Called when a stream is delivered as a single file rather than
   as chunks.  This may be simpler for a plugin to deal with, but
-  precludes any incremental behaviour.
+  precludes any incremental behavior.
   \sa newStreamCreated(), newStream()
 
   Note that the AsFileOnly method is not supported by Netscape 2.0
   and MS-Explorer 3.0.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppstreamasfile>
-  Netscape: NPP_StreamAsFile method</a>
 */
 void QNPInstance::streamAsFile(QNPStream*, const char*)
 {
@@ -1421,10 +1435,6 @@ void QNPInstance::streamAsFile(QNPStream*, const char*)
   be complete() and okay().  If it is not okay(), then an error has
   occurred.  If it is okay(), but not complete(), then the user has
   cancelled the transmission - do not give an error message in this case.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppdestroystream>
-  Netscape: NPP_DestroyStream method</a>
 */
 void QNPInstance::streamDestroyed(QNPStream*)
 {
@@ -1435,10 +1445,6 @@ void QNPInstance::streamDestroyed(QNPStream*)
   willing to receive from the given stream.
 
   The default returns a very large value.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppwriteready>
-  Netscape: NPP_WriteReady method</a>
 */
 int QNPInstance::writeReady(QNPStream*)
 {
@@ -1458,10 +1464,6 @@ int QNPInstance::writeReady(QNPStream*)
   in prior calls.
 
   This function should return the amount of data actually consumed.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#nppwrite>
-  Netscape: NPP_NewStream method</a>
 */
 int QNPInstance::write(QNPStream*, int, int len, void*)
 {
@@ -1473,10 +1475,6 @@ int QNPInstance::write(QNPStream*, int, int len, void*)
   Requests that the given URL be retrieved and sent to the named
   window.  See Netscape's JavaScript documentation for an explanation
   of window names.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgur.htm#npngeturl>
-  Netscape: NPN_GetURL method</a>
 */
 void QNPInstance::getURL(const char* url, const char* window)
 {
@@ -1486,10 +1484,6 @@ void QNPInstance::getURL(const char* url, const char* window)
 /*!
   This function is not tested.  It is an interface to the NPN_PostURL
   function of the Netscape Plugin API.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgur.htm#npnposturl>
-  Netscape: NPN_PostURL method</a>
 */
 void QNPInstance::postURL(const char* url, const char* window,
 	     uint len, const char* buf, bool file)
@@ -1500,14 +1494,24 @@ void QNPInstance::postURL(const char* url, const char* window,
 /*!
   Print the instance full-page.  By default, this returns FALSE, causing the
   browser to call the (embedded) print() function instead.
+  Requests that the given URL be retrieved and sent to the named
+  window.  See Netscape's JavaScript documentation for an explanation
+  of window names.
+
+  See also:
+  <a href=http://developer.netscape.com/docs/manuals/communicator/plugin/refpgur.htm#npngeturlnotify>
+  Netscape: NPN_GetURLNotify method</a>
+*/
+void QNPInstance::getURLNotify(const char* url, const char* window, void*data)
+{
+    NPN_GetURLNotify( pi->npp, url, window, data );
+}
+
+/*!
 
   This function is not tested.
   It is an encapsulation of the NPP_Print
   function of the Netscape Plugin API.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgdr.htm#nppprint>
-  Netscape: NPP_Print method</a>
 */
 bool QNPInstance::printFullPage()
 {
@@ -1520,10 +1524,6 @@ bool QNPInstance::printFullPage()
   This function is not tested.
   It is an encapsulation of the NPP_Print
   function of the Netscape Plugin API.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgdr.htm#nppprint>
-  Netscape: NPP_Print method</a>
 */
 void QNPInstance::print(QPainter*)
 {
@@ -1551,6 +1551,21 @@ const char* QNPInstance::argn(int i) const
     return pi->argn[i];
 }
 
+/*!
+  Called whenever an url is notified after call to NPN_GetURLNotify
+
+  This function is not tested.
+  It is an encapsulation of the NPP_URLNotify
+  function of the Netscape Plugin API.
+
+  See also:
+  <a href=http://developer.netscape.com/docs/manuals/communicator/plugin/refpgur.htm#nppurlnotify>
+  Netscape: NPP_URLNotify method</a>
+*/
+void QNPInstance::notifyURL(const char*, Reason, void*)
+{
+}
+
 /*!
   Returns the value of the <em>i</em>th argument.  See notes of argc().
 */
@@ -1595,10 +1610,6 @@ const char* QNPInstance::arg(const char* name) const
 
 /*!
   Returns the user agent (browser name) containing this instance.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgve.htm#npnuseragent>
-  Netscape: NPN_UserAgent method</a>
 */
 const char* QNPInstance::userAgent() const
 {
@@ -1610,10 +1621,6 @@ const char* QNPInstance::userAgent() const
   This function is not tested.
   It is an interface to the NPN_NewStream
   function of the Netscape Plugin API.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#npnnewstream>
-  Netscape: NPN_NewStream method</a>
 */
 QNPStream* QNPInstance::newStream(const char* mimetype, const char* window,
     bool as_file)
@@ -1626,10 +1633,6 @@ QNPStream* QNPInstance::newStream(const char* mimetype, const char* window,
 
 /*!
   Sets the status message in the browser containing this instance.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgve.htm#npnstatus>
-  Netscape: NPN_Status method</a>
 */
 void QNPInstance::status(const char* msg)
 {
@@ -1647,9 +1650,7 @@ void QNPInstance::status(const char* msg)
   The return value is actually a <tt>jref</tt> we use <tt>void*</tt> so
   as to avoid burdening plugins which do not require Java.
 
-  \sa QNPlugin::getJavaClass(), QNPlugin::getJavaEnv(), getJavaPeer(),
-      <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgja.htm#npngetjavapeer>
-      Netscape: NPN_GetJavaPeer</a>
+  \sa QNPlugin::getJavaClass(), QNPlugin::getJavaEnv(), getJavaPeer()
 */
 void* QNPInstance::getJavaPeer() const
 {
@@ -1776,10 +1777,6 @@ bool QNPStream::complete() const
 /*!
   Requests the given section of the stream be sent to the
   QNPInstance::write() function of the instance() of this stream.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#npnrequestread>
-  Netscape: NPN_RequestRead method</a>
 */
 void QNPStream::requestRead(int offset, uint length)
 {
@@ -1792,10 +1789,6 @@ void QNPStream::requestRead(int offset, uint length)
 
 /*!
   Writes data \e to the stream.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgst.htm#npnwrite>
-  Netscape: NPN_Write method</a>
 */
 int QNPStream::write( int len, void* buffer )
 {
@@ -1870,10 +1863,6 @@ QNPlugin::~QNPlugin()
 /*!
   Returns the version information - the version of the plugin API, and
   the version of the browser.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgve.htm#npnversion>
-  Netscape: NPN_Version method</a>
 */
 void QNPlugin::getVersionInfo(int& plugin_major, int& plugin_minor,
 	     int& browser_major, int& browser_minor)
@@ -1914,10 +1903,6 @@ void QNPlugin::getVersionInfo(int& plugin_major, int& plugin_minor,
   \fn const char* QNPlugin::getPluginNameString() const
 
   Returns the plain-text name of the plugin.
-
-  See also:
-  <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgdr.htm#nppgetvalue>
-  Netscape: NPP_GetValue method</a>
 */
 
 /*!
@@ -1936,9 +1921,7 @@ void QNPlugin::getVersionInfo(int& plugin_major, int& plugin_minor,
   The return value is actually a <tt>jref</tt> we use <tt>void*</tt> so
   as to avoid burdening plugins which do not require Java.
 
-  \sa getJavaEnv(), QNPInstance::getJavaPeer(),
-      <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgja.htm#nppgetjavaclass>
-      Netscape: NPP_GetJavaClass</a>
+  \sa getJavaEnv(), QNPInstance::getJavaPeer()
 */
 void* QNPlugin::getJavaClass()
 {
@@ -1949,13 +1932,10 @@ void* QNPlugin::getJavaClass()
   This function is called when the plugin is shutting down,
   with \a jc set to the value returned earlier by getJavaClass().
   The function should \e unuse the Java class and return 0.
-
-  \sa <a href=http://home.netscape.com/eng/mozilla/3.0/handbook/plugins/wr3.htm#Assoc>
-    Netscape: Associating a Class with your Plug-in</a>
 */
 void QNPlugin::unuseJavaClass()
 {
-    fatal("QNPlugin::unuseJavaClass() must overridden along with getJavaClass()");
+    qFatal("QNPlugin::unuseJavaClass() must overridden along with getJavaClass()");
 }
 
 /*!
@@ -1965,9 +1945,7 @@ void QNPlugin::unuseJavaClass()
   The return value is actually a <tt>JRIEnv*</tt> we use <tt>void*</tt> so
   as to avoid burdening plugins which do not require Java.
 
-  \sa getJavaClass(), QNPInstance::getJavaPeer(),
-      <a href=http://developer.netscape.com/library/documentation/communicator/plugin/refpgja.htm#npngetjavaenv>
-      Netscape: NPN_GetJavaEnv</a>
+  \sa getJavaClass(), QNPInstance::getJavaPeer()
 */
 void* QNPlugin::getJavaEnv() const
 {
