@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: globjwin.h,v 1.1.2.1 1998/11/30 10:47:00 aavit Exp $
+** $Id: qt/extensions/opengl/examples/glpixmap/globjwin.h   2.1.1   edited 2000-02-23 $
 **
 ** Definition of GLObjectWindow widget class
 ** The GLObjectWindow contains a GLBox and three sliders connected to
@@ -25,9 +25,8 @@ public:
 protected slots:
 
     void		makePixmap();
-    void		makePixmapManually();
     void		makePixmapHidden();
-    void		makePixmapHiddenManually();
+    void		makePixmapForMenu();
     void		useFixedPixmapSize();
 
 private:
@@ -35,6 +34,7 @@ private:
     GLBox* c1;
     QLabel* lb;
     int fixMenuItemId;
+    int insertMenuItemId;
     QSize pmSz;
     QPopupMenu* file;
 };
