@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qmessagebox.cpp,v 2.51.2.2 1999/02/10 12:26:05 warwick Exp $
+** $Id: qt/src/dialogs/qmessagebox.cpp   2.1.1   edited 2000-04-19 $
 **
 ** Implementation of QMessageBox class
 **
 ** Created : 950503
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,7 +27,6 @@
 #include "qlabel.h"
 #include "qpushbutton.h"
 #include "qimage.h"
-#include "qkeycode.h"
 #include "qapplication.h"
 
 // Message box icons, from page 210 of the Windows style guide.
@@ -35,132 +36,191 @@
 // using but a lot of people still use, is left out.
 
 /* XPM */
-static const char* information_xpm[]={
+static const char * const information_xpm[]={
 "32 32 5 1",
 ". c None",
 "c c #000000",
-"# c #999999",
+"* c #999999",
 "a c #ffffff",
 "b c #0000ff",
-"...........########.............",
-"........###aaaaaaaa###..........",
-"......##aaaaaaaaaaaaaa##........",
-".....#aaaaaaaaaaaaaaaaaa#.......",
-"....#aaaaaaaabbbbaaaaaaaac......",
-"...#aaaaaaaabbbbbbaaaaaaaac.....",
-"..#aaaaaaaaabbbbbbaaaaaaaaac....",
-".#aaaaaaaaaaabbbbaaaaaaaaaaac...",
-".#aaaaaaaaaaaaaaaaaaaaaaaaaac#..",
-"#aaaaaaaaaaaaaaaaaaaaaaaaaaaac#.",
-"#aaaaaaaaaabbbbbbbaaaaaaaaaaac#.",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-".#aaaaaaaaaaabbbbbaaaaaaaaaac###",
-".#aaaaaaaaaaabbbbbaaaaaaaaaac###",
-"..#aaaaaaaaaabbbbbaaaaaaaaac###.",
-"...caaaaaaabbbbbbbbbaaaaaac####.",
-"....caaaaaaaaaaaaaaaaaaaac####..",
-".....caaaaaaaaaaaaaaaaaac####...",
-"......ccaaaaaaaaaaaaaacc####....",
-".......#cccaaaaaaaaccc#####.....",
-"........###cccaaaac#######......",
-"..........####caaac#####........",
-".............#caaac##...........",
-"...............caac##...........",
-"................cac##...........",
-".................cc##...........",
-"..................###...........",
-"...................##..........."};
+"...........********.............",
+"........***aaaaaaaa***..........",
+"......**aaaaaaaaaaaaaa**........",
+".....*aaaaaaaaaaaaaaaaaa*.......",
+"....*aaaaaaaabbbbaaaaaaaac......",
+"...*aaaaaaaabbbbbbaaaaaaaac.....",
+"..*aaaaaaaaabbbbbbaaaaaaaaac....",
+".*aaaaaaaaaaabbbbaaaaaaaaaaac...",
+".*aaaaaaaaaaaaaaaaaaaaaaaaaac*..",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaaac*.",
+"*aaaaaaaaaabbbbbbbaaaaaaaaaaac*.",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+".*aaaaaaaaaaabbbbbaaaaaaaaaac***",
+".*aaaaaaaaaaabbbbbaaaaaaaaaac***",
+"..*aaaaaaaaaabbbbbaaaaaaaaac***.",
+"...caaaaaaabbbbbbbbbaaaaaac****.",
+"....caaaaaaaaaaaaaaaaaaaac****..",
+".....caaaaaaaaaaaaaaaaaac****...",
+"......ccaaaaaaaaaaaaaacc****....",
+".......*cccaaaaaaaaccc*****.....",
+"........***cccaaaac*******......",
+"..........****caaac*****........",
+".............*caaac**...........",
+"...............caac**...........",
+"................cac**...........",
+".................cc**...........",
+"..................***...........",
+"...................**..........."};
 /* XPM */
-static const char* warning_xpm[]={
+static const char* const warning_xpm[]={
 "32 32 4 1",
 ". c None",
 "a c #ffff00",
-"# c #000000",
+"* c #000000",
 "b c #999999",
-".............###................",
-"............#aaa#...............",
-"...........#aaaaa#b.............",
-"...........#aaaaa#bb............",
-"..........#aaaaaaa#bb...........",
-"..........#aaaaaaa#bb...........",
-".........#aaaaaaaaa#bb..........",
-".........#aaaaaaaaa#bb..........",
-"........#aaaaaaaaaaa#bb.........",
-"........#aaaa###aaaa#bb.........",
-".......#aaaa#####aaaa#bb........",
-".......#aaaa#####aaaa#bb........",
-"......#aaaaa#####aaaaa#bb.......",
-"......#aaaaa#####aaaaa#bb.......",
-".....#aaaaaa#####aaaaaa#bb......",
-".....#aaaaaa#####aaaaaa#bb......",
-"....#aaaaaaaa###aaaaaaaa#bb.....",
-"....#aaaaaaaa###aaaaaaaa#bb.....",
-"...#aaaaaaaaa###aaaaaaaaa#bb....",
-"...#aaaaaaaaaa#aaaaaaaaaa#bb....",
-"..#aaaaaaaaaaa#aaaaaaaaaaa#bb...",
-"..#aaaaaaaaaaaaaaaaaaaaaaa#bb...",
-".#aaaaaaaaaaaa##aaaaaaaaaaa#bb..",
-".#aaaaaaaaaaa####aaaaaaaaaa#bb..",
-"#aaaaaaaaaaaa####aaaaaaaaaaa#bb.",
-"#aaaaaaaaaaaaa##aaaaaaaaaaaa#bb.",
-"#aaaaaaaaaaaaaaaaaaaaaaaaaaa#bbb",
-"#aaaaaaaaaaaaaaaaaaaaaaaaaaa#bbb",
-".#aaaaaaaaaaaaaaaaaaaaaaaaa#bbbb",
-"..#########################bbbbb",
+".............***................",
+"............*aaa*...............",
+"...........*aaaaa*b.............",
+"...........*aaaaa*bb............",
+"..........*aaaaaaa*bb...........",
+"..........*aaaaaaa*bb...........",
+".........*aaaaaaaaa*bb..........",
+".........*aaaaaaaaa*bb..........",
+"........*aaaaaaaaaaa*bb.........",
+"........*aaaa***aaaa*bb.........",
+".......*aaaa*****aaaa*bb........",
+".......*aaaa*****aaaa*bb........",
+"......*aaaaa*****aaaaa*bb.......",
+"......*aaaaa*****aaaaa*bb.......",
+".....*aaaaaa*****aaaaaa*bb......",
+".....*aaaaaa*****aaaaaa*bb......",
+"....*aaaaaaaa***aaaaaaaa*bb.....",
+"....*aaaaaaaa***aaaaaaaa*bb.....",
+"...*aaaaaaaaa***aaaaaaaaa*bb....",
+"...*aaaaaaaaaa*aaaaaaaaaa*bb....",
+"..*aaaaaaaaaaa*aaaaaaaaaaa*bb...",
+"..*aaaaaaaaaaaaaaaaaaaaaaa*bb...",
+".*aaaaaaaaaaaa**aaaaaaaaaaa*bb..",
+".*aaaaaaaaaaa****aaaaaaaaaa*bb..",
+"*aaaaaaaaaaaa****aaaaaaaaaaa*bb.",
+"*aaaaaaaaaaaaa**aaaaaaaaaaaa*bb.",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaa*bbb",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaa*bbb",
+".*aaaaaaaaaaaaaaaaaaaaaaaaa*bbbb",
+"..*************************bbbbb",
 "....bbbbbbbbbbbbbbbbbbbbbbbbbbb.",
 ".....bbbbbbbbbbbbbbbbbbbbbbbbb.."};
 /* XPM */
-static const char* critical_xpm[]={
+static const char* const critical_xpm[]={
 "32 32 4 1",
 ". c None",
 "a c #999999",
-"# c #ff0000",
+"* c #ff0000",
 "b c #ffffff",
-"...........########.............",
-".........############...........",
-".......################.........",
-"......##################........",
-".....####################a......",
-"....######################a.....",
-"...########################a....",
-"..#######b##########b#######a...",
-"..######bbb########bbb######a...",
-".######bbbbb######bbbbb######a..",
-".#######bbbbb####bbbbb#######a..",
-"#########bbbbb##bbbbb#########a.",
-"##########bbbbbbbbbb##########a.",
-"###########bbbbbbbb###########aa",
-"############bbbbbb############aa",
-"############bbbbbb############aa",
-"###########bbbbbbbb###########aa",
-"##########bbbbbbbbbb##########aa",
-"#########bbbbb##bbbbb#########aa",
-".#######bbbbb####bbbbb#######aa.",
-".######bbbbb######bbbbb######aa.",
-"..######bbb########bbb######aaa.",
-"..#######b##########b#######aa..",
-"...########################aaa..",
-"....######################aaa...",
-"....a####################aaa....",
-".....a##################aaa.....",
-"......a################aaa......",
-".......aa############aaaa.......",
-".........aa########aaaaa........",
+"...........********.............",
+".........************...........",
+".......****************.........",
+"......******************........",
+".....********************a......",
+"....**********************a.....",
+"...************************a....",
+"..*******b**********b*******a...",
+"..******bbb********bbb******a...",
+".******bbbbb******bbbbb******a..",
+".*******bbbbb****bbbbb*******a..",
+"*********bbbbb**bbbbb*********a.",
+"**********bbbbbbbbbb**********a.",
+"***********bbbbbbbb***********aa",
+"************bbbbbb************aa",
+"************bbbbbb************aa",
+"***********bbbbbbbb***********aa",
+"**********bbbbbbbbbb**********aa",
+"*********bbbbb**bbbbb*********aa",
+".*******bbbbb****bbbbb*******aa.",
+".******bbbbb******bbbbb******aa.",
+"..******bbb********bbb******aaa.",
+"..*******b**********b*******aa..",
+"...************************aaa..",
+"....**********************aaa...",
+"....a********************aaa....",
+".....a******************aaa.....",
+"......a****************aaa......",
+".......aa************aaaa.......",
+".........aa********aaaaa........",
 "...........aaaaaaaaaaa..........",
 ".............aaaaaaa............"};
+#define QT_END_TEXT qt_dialog_default_key
 
 
+// the Qt logo, for aboutQt
+/* XPM */
+static const char * const qtlogo_xpm[] = {
+/* width height ncolors chars_per_pixel */
+"54 34 13 1",
+/* colors */
+"  c #000000",
+". c #999999",
+"X c #333366",
+"o c #6666CC",
+"O c #333333",
+"@ c #666699",
+"# c #000066",
+"$ c #666666",
+"% c #3333CC",
+"& c #000033",
+"* c #9999CC",
+"= c #333399",
+"+ c None",
+/* pixels */
+"+++++++++++++++++++.$OOO$.++++++++++++++++++++++++++++",
+"+++++++++++++++++$         O.+++++++++++++++++++++++++",
+"+++++++++++++++.O            $++++++++++++++++++++++++",
+"++++++++++++++.    $.++.$     O+++++++++++++++++++++++",
+"+++++++++++++.   O.+++++++$    O++++++++++++++++++++++",
+"+++++++++++++O   ++++++++++$    $+++++++++++++++++++++",
+"++++++++++++$   .+++++++++++O    .++++++++++++++++++++",
+"+++++++++++.   O+++++++++++++    O++++++.+++++++++++++",
+"+++++++++++$   .+++++++++++++$    .+++.O +++++++++++++",
+"+++++++++++    +++++++++++++++    O+++.  +++++++++++++",
+"++++++++++.  &Xoooo*++++++++++$    +++.  +++++++++++++",
+"++++++++++@=%%%%%%%%%%*+++++++.    .++.  +++++++++++++",
+"+++++++**oooooo**++*o%%%%o+++++    $++O  +++++++++++++",
+"+++++*****$OOX@oooo*++*%%%%%*++O   $+.   OOO$++++++++*",
+"++..+.++....$O$+*ooooo*+*o%%%%%O   O+$   $$O.++++++**+",
+"***+*+++++$$....+++*oooo**+*o%%#   O++O  ++++++***o*++",
+"*+++++++++O  $.....++**oooo**+*X   &o*O  ++++*ooo*++++",
+"++++++++++$   O++.....++**oooo*X   &%%&  ..*o%%*++++++",
+"++++++++++$    ++++.....+++**ooO   $*o&  @oo*+++++++++",
+"++++++++++.    .++++++.....+++*O   Xo*O  .++++++++++++",
+"+++++++++++    O+++++++++......    .++O  +++++++++++++",
+"+++++++++++O    +++.$$$.++++++.   O+++O  +++++++++++++",
+"+++++++++++.    $$OO    O.++++O   .+++O  +++++++++++++",
+"++++++++++++O    .+++.O   $++.   O++++O  +++++++++++++",
+"++++++++++++.    O+++++O   $+O   +++++O  +++++++++++++",
+"+++++++++++++.    O+++++O   O   .+++++O  .++++++++++++",
+"++++++++++++++$     .++++O     .++++.+$  O+.$.++++++++",
+"+++++++++++++++.      O$$O    .+++++...      +++++++++",
+"+++++++++++++++++$            O+++++$$+.O O$.+++++++++",
+"+++++++++++++++++++$OO  O$.O   O.++. .++++++++++++++++",
+"+++++++++++++++++++++++++++.     OO  .++++++++++++++++",
+"++++++++++++++++++++++++++++.       O+++++++++++++++++",
+"+++++++++++++++++++++++++++++.      .+++++++++++++++++",
+"++++++++++++++++++++++++++++++.O  O.++++++++++++++++++"
+};
+
+
+
+
+// BEING REVISED: paul
 /*!
   \class QMessageBox qmessagebox.h
   \brief Displays a brief message, an icon, and some buttons.
   \ingroup dialogs
 
-  \define QMessageBox::Icon
-
   A message box is a modal dialog that displays an icon, a text and up to
   three push buttons.  It's used for simple messages and questions.
 
@@ -180,6 +240,16 @@ static const char* critical_xpm[]={
   applications) for a simple messages, or two or even three buttons
   for questions.
 
+  The text part of all message box messages can be either rich text or
+  plain text. If you specify a rich text formatted string, it will be
+  rendered using the default stylesheet. See
+  QStyleSheet::defaultSheet() for details. With certain strings that
+  contain XML meta characters , the auto-rich text detection may fail,
+  interpreting plain text falsely as rich text. In these rare cases,
+  use QStyleSheet::convertFromPlainText() to convert your plain text
+  string to a visually equivalent rich text string or set the text
+  format explicitly with setTextFormat().
+
   Here are some examples of how to use the static member functions.
   After these examples you will find an overview of the non-static
   member functions.
@@ -267,7 +337,7 @@ static const char* critical_xpm[]={
   \endcode
 
   The critical() function should be reserved for critical errors.  In
-  this example, errorDetails is a QString or const char *, and QString
+  this example, errorDetails is a QString or const char*, and QString
   is used to concatenate several strings:
 
   \code
@@ -355,7 +425,7 @@ static const char* critical_xpm[]={
 
   QMessageBox has no signals or slots.
 
-  <img src=qmsgbox-m.gif> <img src=qmsgbox-w.gif>
+  <img src=qmsgbox-m.png> <img src=qmsgbox-w.png>
 
   \sa QDialog, <a href="http://www.iarchitect.com/errormsg.htm">Isys on
   error messages,</a>
@@ -364,6 +434,19 @@ static const char* critical_xpm[]={
 */
 
 
+/*!
+  \enum QMessageBox::Icon
+
+  This type includes the following values:
+  <ul>
+  <li> \c QMessageBox::NoIcon
+  <li> \c QMessageBox::Information
+  <li> \c QMessageBox::Warning
+  <li> \c QMessageBox::Critical
+  </ul>
+*/
+
+
 struct QMBData {
     QMBData(QMessageBox* parent) :
 	iconLabel( parent, "icon" )
@@ -387,7 +470,15 @@ static const int LastButton = QMessageBox::Ignore;
 */
 
 static const char *mb_texts[] = {
-    0, "OK", "Cancel", "Yes", "No", "Abort", "Retry", "Ignore", 0
+    0,
+    QT_TRANSLATE_NOOP("QMessageBox","OK"),
+    QT_TRANSLATE_NOOP("QMessageBox","Cancel"),
+    QT_TRANSLATE_NOOP("QMessageBox","Yes"),
+    QT_TRANSLATE_NOOP("QMessageBox","No"),
+    QT_TRANSLATE_NOOP("QMessageBox","Abort"),
+    QT_TRANSLATE_NOOP("QMessageBox","Retry"),
+    QT_TRANSLATE_NOOP("QMessageBox","Ignore"),
+    0
 };
 
 
@@ -402,7 +493,7 @@ static const char *mb_texts[] = {
 */
 
 QMessageBox::QMessageBox( QWidget *parent, const char *name )
-    : QDialog( parent, name, TRUE )
+    : QDialog( parent, name, TRUE, WStyle_Customize | WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu )
 {
     init( Ok, 0, 0 );
 }
@@ -422,6 +513,7 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
 
   Each button can have one of the following values:
   <ul>
+  <li>\c QMessageBox::NoButton
   <li>\c QMessageBox::Ok
   <li>\c QMessageBox::Cancel
   <li>\c QMessageBox::Yes
@@ -431,6 +523,9 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
   <li>\c QMessageBox::Ignore
   </ul>
 
+  Use QMessageBox::NoButton for the later parameters to have less than
+  three buttons in your messagebox.
+
   One of the buttons can be combined with the \c QMessageBox::Default flag
   to make a default button.
 
@@ -462,11 +557,12 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
   \sa setCaption(), setText(), setIcon()
 */
 
-QMessageBox::QMessageBox( const char *caption, const char *text, Icon icon,
+QMessageBox::QMessageBox( const QString& caption,
+			  const QString &text, Icon icon,
 			  int button0, int button1, int button2,
 			  QWidget *parent, const char *name,
 			  bool modal, WFlags f )
-    : QDialog( parent, name, modal, f )
+    : QDialog( parent, name, modal, f | WStyle_Customize | WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu )
 {
     init( button0, button1, button2 );
     setCaption( caption );
@@ -476,7 +572,7 @@ QMessageBox::QMessageBox( const char *caption, const char *text, Icon icon,
 
 
 /*!
-  Destroys the message box.
+  Destructs the message box.
 */
 
 QMessageBox::~QMessageBox()
@@ -493,7 +589,7 @@ void QMessageBox::init( int button0, int button1, int button2 )
 
     if ( (button2 && !button1) || (button1 && !button0) ) {
 #if defined(CHECK_RANGE)
-	::warning( "QMessageBox: Inconsistent button parameters" );
+	qWarning( "QMessageBox: Inconsistent button parameters" );
 #endif
 	button0 = button1 = button2 = 0;
     }
@@ -511,7 +607,7 @@ void QMessageBox::init( int button0, int button1, int button2 )
 	if ( (b & Default) ) {
 	    if ( mbd->defButton >= 0 ) {
 #if defined(CHECK_RANGE)
-		::warning( "QMessageBox: There can be at most one "
+		qWarning( "QMessageBox: There can be at most one "
 			   "default button" );
 #endif
 	    } else {
@@ -521,7 +617,7 @@ void QMessageBox::init( int button0, int button1, int button2 )
 	if ( (b & Escape) ) {
 	    if ( mbd->escButton >= 0 ) {
 #if defined(CHECK_RANGE)
-		::warning( "QMessageBox: There can be at most one "
+		qWarning( "QMessageBox: There can be at most one "
 			   "escape button" );
 #endif
 	    } else {
@@ -534,13 +630,13 @@ void QMessageBox::init( int button0, int button1, int button2 )
 		b = Ok;
 	} else if ( b < 0 || b > LastButton ) {
 #if defined(CHECK_RANGE)
-	    ::warning( "QMessageBox: Invalid button specifier" );
+	    qWarning( "QMessageBox: Invalid button specifier" );
 #endif
 	    b = Ok;
 	} else {
 	    if ( i > 0 && mbd->button[i-1] == 0 ) {
 #if defined(CHECK_RANGE)
-		::warning( "QMessageBox: Inconsistent button parameters; "
+		qWarning( "QMessageBox: Inconsistent button parameters; "
 			   "button %d defined but not button %d",
 			   i+1, i );
 #endif
@@ -555,10 +651,11 @@ void QMessageBox::init( int button0, int button1, int button2 )
 	if ( i >= mbd->numButtons ) {
 	    mbd->pb[i] = 0;
 	} else {
-	    QString buttonName;
+	    QCString buttonName;
 	    buttonName.sprintf( "button%d", i+1 );
-	    mbd->pb[i] = new QPushButton( mb_texts[mbd->button[i]],
-					  this, buttonName );
+	    mbd->pb[i] = new QPushButton(
+		tr(mb_texts[mbd->button[i]]),
+	        this, buttonName );
 	    if ( mbd->defButton == i ) {
 		mbd->pb[i]->setDefault( TRUE );
 		mbd->pb[i]->setFocus();
@@ -604,37 +701,36 @@ void QMessageBox::resizeButtons()
 
 
 /*!
-  Returns the message box text currently set, or null if no text has been set.
-  \sa setText()
+  Returns the message box text currently set, or a
+  \link QString::operator!() null string\endlink
+  if no text has been set.
+  \sa setText(), textFormat()
 */
 
-const char *QMessageBox::text() const
+QString QMessageBox::text() const
 {
     return label->text();
 }
 
 /*!
   Sets the message box text to be displayed.
-  \sa text()
+
+  \a text will be interpreted either as a plain text or as a rich
+  text, depending on the text format setting; see setTextFormat(). The
+  default setting is \c AutoText, i.e. the message box will try to
+  auto-detect the format of \a text.
+
+  \sa text(), setTextFormat()
 */
 
-void QMessageBox::setText( const char *text )
+void QMessageBox::setText( const QString &text )
 {
     label->setText( text );
 }
 
-
 /*!
   Returns the icon of the message box.
 
-  The return value is one of the following:
-  <ul>
-  <li> \c QMessageBox::NoIcon
-  <li> \c QMessageBox::Information
-  <li> \c QMessageBox::Warning
-  <li> \c QMessageBox::Critical
-  </ul>
-
   \sa setIcon(), iconPixmap()
 */
 
@@ -674,43 +770,43 @@ void QMessageBox::setIcon( Icon icon )
 
 QPixmap QMessageBox::standardIcon( Icon icon, GUIStyle style )
 {
-    const char **xpm_data;
+    const char * const * xpm_data;
     switch ( icon ) {
-	case Information:
-	    xpm_data = information_xpm;
-	    break;
-	case Warning:
-	    xpm_data = warning_xpm;
-	    break;
-	case Critical:
-	    xpm_data = critical_xpm;
-	    break;
-	default:
-	    xpm_data = 0;	
+    case Information:
+	xpm_data = information_xpm;
+	break;
+    case Warning:
+	xpm_data = warning_xpm;
+	break;
+    case Critical:
+	xpm_data = critical_xpm;
+	break;
+    default:
+	xpm_data = 0;
     }
     QPixmap pm;
     if ( xpm_data ) {
-	QImage image(xpm_data);
+	QImage image( (const char **) xpm_data);
 	if ( style == MotifStyle ) {
 	    // All that colour looks ugly in Motif
-	    QColorGroup g = QApplication::palette()->normal();
+	    QColorGroup g = QApplication::palette().normal();
 	    switch ( icon ) {
-	      case Information:
+	    case Information:
 		image.setColor( 2, 0xff000000 | g.dark().rgb() );
 		image.setColor( 3, 0xff000000 | g.base().rgb() );
 		image.setColor( 4, 0xff000000 | g.text().rgb() );
 		break;
-	      case Warning:
+	    case Warning:
 		image.setColor( 1, 0xff000000 | g.base().rgb() );
 		image.setColor( 2, 0xff000000 | g.text().rgb() );
 		image.setColor( 3, 0xff000000 | g.dark().rgb() );
 		break;
-	      case Critical:
+	    case Critical:
 		image.setColor( 1, 0xff000000 | g.dark().rgb() );
 		image.setColor( 2, 0xff000000 | g.text().rgb() );
 		image.setColor( 3, 0xff000000 | g.base().rgb() );
 	        break;
-	      default:
+	    default:
 		; // Can't happen
 	    }
 	}
@@ -753,43 +849,19 @@ void QMessageBox::setIconPixmap( const QPixmap &pixmap )
 }
 
 
-/*!
-  This function will be removed in a future version of Qt.
-*/
-
-const char *QMessageBox::buttonText() const
-{
-    return buttonText( Ok );
-}
-
-/*!
-  This function will be removed in a future version of Qt.
-*/
-
-void QMessageBox::setButtonText( const char *text )
-{
-    setButtonText( Ok, text ? text : "OK" );
-}
-
-
 /*!
   Returns the text of the message box button \a button, or null if the
   message box does not contain the button.
 
-  Example:
-  \code
-    QMessageBox mb( QMessageBox::Ok, QMessageBox::Cancel, 0 );
-    mb.buttonText( QMessageBox::Cancel );  // returns "Cancel"
-    mb.buttonText( QMessageBox::Ignore );  // returns 0
-  \endcode
-
   \sa setButtonText()
 */
 
-const char *QMessageBox::buttonText( int button ) const
+QString QMessageBox::buttonText( int button ) const
 {
     int index = indexOf(button);
-    return index >= 0 && mbd->pb[index] ? mbd->pb[index]->text() : 0;
+    return index >= 0 && mbd->pb[index]
+	    ? mbd->pb[index]->text()
+	    : QString::null;
 }
 
 
@@ -798,17 +870,10 @@ const char *QMessageBox::buttonText( int button ) const
   Setting the text of a button that is not in the message box is quietly
   ignored.
 
-  Example:
-  \code
-    QMessageBox mb( QMessageBox::Ok, QMessageBox::Cancel, 0 );
-    mb.setButtonText( QMessageBox::Ok, "All Right" );
-    mb.setButtonText( QMessageBox::Yes, "Yo" );	  // ignored
-  \endcode
-
   \sa buttonText()
 */
 
-void QMessageBox::setButtonText( int button, const char *text )
+void QMessageBox::setButtonText( int button, const QString &text )
 {
     int index = indexOf(button);
     if ( index >= 0 && mbd->pb[index] ) {
@@ -841,81 +906,89 @@ void QMessageBox::buttonClicked()
   This function will not be called if the message box has been explicitly
   resized before showing it.
 */
-
 void QMessageBox::adjustSize()
 {
-    int i;
-    QSize smax = mbd->buttonSize;
-    int border = smax.height()/2;
-    if ( border == 0 )
-	border = 10;
-    else if ( style() == MotifStyle )
-	border += 6;
-    for ( i=0; i<mbd->numButtons; i++ )
-	mbd->pb[i]->resize( smax );
+    if ( !testWState(WState_Polished) )
+	polish();
+    resizeButtons();
     label->adjustSize();
-    int bw = mbd->numButtons * smax.width() + (mbd->numButtons-1)*border;
-    int w = QMAX( bw, label->width() ) + 2*border;
-    int h = smax.height();
-    if ( label->height() )
-	h += label->height() + 3*border;
+    QSize labelSize( label->size() );
+    int n  = mbd->numButtons;
+    int bw = mbd->buttonSize.width();
+    int bh = mbd->buttonSize.height();
+    int border = bh/2 - style().buttonDefaultIndicatorWidth();
+    if ( border <= 0 )
+	border = 10;
+    int btn_spacing = 7;
+    if ( style() == MotifStyle )
+	btn_spacing = border;
+    int buttons = mbd->numButtons * bw + (n-1) * btn_spacing;
+    int h = bh;
+    if ( labelSize.height() )
+	h += labelSize.height() + 3*border;
     else
 	h += 2*border;
+    int lmargin = 0;
     if ( mbd->iconLabel.pixmap() && mbd->iconLabel.pixmap()->width() )  {
 	mbd->iconLabel.adjustSize();
-	w += mbd->iconLabel.pixmap()->width() + border;
-	if ( h < mbd->iconLabel.pixmap()->height() + 3*border + smax.height() )
-	    h = mbd->iconLabel.pixmap()->height() + 3*border + smax.height();
+	lmargin += mbd->iconLabel.width() + border;
+	if ( h < mbd->iconLabel.height() + 3*border + bh )
+	    h = mbd->iconLabel.height() + 3*border + bh;
     }
-    resize( w, h );	
+    int w = QMAX( buttons, labelSize.width() + lmargin ) + 2*border;
+    resize( w, h );
+    setMinimumSize( size() );
 }
 
 
-/*!
-  Handles resize events for the message box.
+/*!\reimp
 */
-
 void QMessageBox::resizeEvent( QResizeEvent * )
 {
     int i;
     int n  = mbd->numButtons;
     int bw = mbd->buttonSize.width();
     int bh = mbd->buttonSize.height();
-    int border = bh/2;
-    if ( border == 0 )
+    int border = bh/2 - style().buttonDefaultIndicatorWidth();
+    if ( border <= 0 )
 	border = 10;
-    else if ( style() == MotifStyle )
-	border += 6;
+    int btn_spacing = 7;
+    if ( style() == MotifStyle )
+	btn_spacing = border;
     int lmargin = 0;
     mbd->iconLabel.adjustSize();
     mbd->iconLabel.move( border, border );
     if ( mbd->iconLabel.pixmap() && mbd->iconLabel.pixmap()->width() )
-	lmargin += mbd->iconLabel.pixmap()->width() + border;
-    label->move( (width() + lmargin)/2 - label->width()/2,
-		 (height() - border - bh - label->height()) / 2 );
-    int space = (width() - bw*n)/(n+1);
-    for ( i=0; i<n; i++ ) {
-	mbd->pb[i]->move( space*(i+1)+bw*i,
-			  height() - border - bh );
-    }
+	lmargin += mbd->iconLabel.width() + border;
+    label->setGeometry( lmargin+border,
+			border,
+			width() - lmargin -2*border,
+			height() - 3*border - bh );
+    int extra_space = (width() - bw*n - 2*border - (n-1)*btn_spacing);
+    if ( style() == MotifStyle )
+	for ( i=0; i<n; i++ )
+	    mbd->pb[i]->move( border + i*bw + i*btn_spacing + extra_space*(i+1)/(n+1),
+			      height() - border - bh );
+    else
+	for ( i=0; i<n; i++ )
+	    mbd->pb[i]->move( border + i*bw + extra_space/2 + i*btn_spacing,
+			      height() - border - bh );
 }
 
 
-/*!
-  Handles key press events for the message box.
+/*!\reimp
 */
-
 void QMessageBox::keyPressEvent( QKeyEvent *e )
 {
     if ( e->key() == Key_Escape ) {
 	if ( mbd->escButton >= 0 ) {
 	    QPushButton *pb = mbd->pb[mbd->escButton];
 	    pb->animateClick();
+	    e->accept();
+	    return;
 	}
-	e->accept();
-    } else {
-	QDialog::keyPressEvent( e );
     }
+    QDialog::keyPressEvent( e );
 }
 
 
@@ -931,13 +1004,15 @@ void QMessageBox::keyPressEvent( QKeyEvent *e )
   information(), warning() or critical() instead.
 */
 
-int QMessageBox::message( const char *caption,
-			  const char *text,
-			  const char *buttonText,
+int QMessageBox::message( const QString &caption,
+			  const QString& text,
+			  const QString& buttonText,
 			  QWidget    *parent,
 			  const char * )
 {
-    return QMessageBox::information( parent, caption, text, buttonText?buttonText:"Ok" ) == 0;
+    return QMessageBox::information( parent, caption, text,
+				     buttonText.isEmpty()
+				     ? tr("OK") : buttonText ) == 0;
 }
 
 
@@ -950,14 +1025,16 @@ int QMessageBox::message( const char *caption,
   information(), warning() or critical() instead.
 */
 
-bool QMessageBox::query( const char *caption,
-			 const char *text,
-			 const char *yesButtonText,
-			 const char *noButtonText,
+bool QMessageBox::query( const QString &caption,
+			 const QString& text,
+			 const QString& yesButtonText,
+			 const QString& noButtonText,
 			 QWidget *parent, const char * )
 {
     return QMessageBox::information( parent, caption, text,
-				     yesButtonText?yesButtonText:"Ok", noButtonText ) == 0;
+				     yesButtonText.isEmpty()
+				     ? tr("OK") : yesButtonText,
+				     noButtonText ) == 0;
 }
 
 
@@ -973,7 +1050,7 @@ bool QMessageBox::query( const char *caption,
 */
 
 int QMessageBox::information( QWidget *parent,
-			      const char *caption, const char *text,
+			      const QString& caption, const QString& text,
 			      int button0, int button1, int button2 )
 {
     QMessageBox *mb = new QMessageBox( caption, text, Information,
@@ -998,7 +1075,7 @@ int QMessageBox::information( QWidget *parent,
 */
 
 int QMessageBox::warning( QWidget *parent,
-			  const char *caption, const char *text,
+			  const QString& caption, const QString& text,
 			  int button0, int button1, int button2 )
 {
     QMessageBox *mb = new QMessageBox( caption, text, Warning,
@@ -1023,7 +1100,7 @@ int QMessageBox::warning( QWidget *parent,
 */
 
 int QMessageBox::critical( QWidget *parent,
-			   const char *caption, const char *text,
+			   const QString& caption, const QString& text,
 			   int button0, int button1, int button2 )
 {
     QMessageBox *mb = new QMessageBox( caption, text, Critical,
@@ -1052,8 +1129,8 @@ int QMessageBox::critical( QWidget *parent,
   \sa QWidget::icon() QApplication::mainWidget()
 */
 
-void QMessageBox::about( QWidget *parent, const char *caption,
-			 const char *text )
+void QMessageBox::about( QWidget *parent, const QString &caption,
+			 const QString& text )
 {
     QMessageBox *mb = new QMessageBox( caption, text,
 				       Information,
@@ -1076,13 +1153,11 @@ void QMessageBox::about( QWidget *parent, const char *caption,
 }
 
 
-/*!
-  Reimplemented for implementational reasons.
+/*! \reimp
 */
 
-void QMessageBox::setStyle( GUIStyle s )
+void QMessageBox::styleChanged( QStyle& )
 {
-    QWidget::setStyle( s );
     if ( mbd->icon != NoIcon ) {
 	// Reload icon for new style
 	setIcon( mbd->icon );
@@ -1091,17 +1166,17 @@ void QMessageBox::setStyle( GUIStyle s )
 
 
 static int textBox( QWidget *parent, QMessageBox::Icon severity,
-		    const char *caption, const char *text,
-		    const char *button0Text,
-		    const char *button1Text,
-		    const char *button2Text,
+		    const QString& caption, const QString& text,
+		    const QString& button0Text,
+		    const QString& button1Text,
+		    const QString& button2Text,
 		    int defaultButtonNumber,
 		    int escapeButtonNumber )
 {
     int b[3];
-    b[0] = (button0Text && *button0Text) ? 1 : 0;
-    b[1] = (button1Text && *button1Text) ? 2 : 0;
-    b[2] = (button2Text && *button2Text) ? 3 : 0;
+    b[0] = 1;
+    b[1] = button1Text.isEmpty() ? 0 : 2;
+    b[2] = button2Text.isEmpty() ? 0 : 3;
 
     int i;
     for( i=0; i<3; i++ ) {
@@ -1115,7 +1190,9 @@ static int textBox( QWidget *parent, QMessageBox::Icon severity,
 				       b[0], b[1], b[2],
 				       parent, "information" );
     CHECK_PTR( mb );
-    if ( b[0] )
+    if ( button0Text.isEmpty() )
+	mb->setButtonText( 1, mb_texts[ QMessageBox::Ok ] );
+    else
 	mb->setButtonText( 1, button0Text );
     if ( b[1] )
 	mb->setButtonText( 2, button1Text );
@@ -1132,8 +1209,9 @@ static int textBox( QWidget *parent, QMessageBox::Icon severity,
   1-3 buttons.  Returns the number of the button that was clicked
   (0, 1 or 2).
 
-  \a button0Text is the text of the first button and must be present,
-  \a button1Text is the text of the second button and is optional, and
+  \a button0Text is the text of the first button and is optional.  If
+  \a button0Text is not supplied, "OK" (translated) will be used.
+  \a button1Text is the text of the second button and is optional.
   \a button2Text is the text of the third button and is optional.  \a
   defaultbuttonNumber (0-2) is the index of the default button;
   pressing Return or Enter is the same as clicking the default button.
@@ -1150,11 +1228,11 @@ static int textBox( QWidget *parent, QMessageBox::Icon severity,
   \sa warning(), critical()
 */
 
-int QMessageBox::information( QWidget *parent, const char *caption,
-			      const char *text,
-			      const char *button0Text,
-			      const char *button1Text,
-			      const char *button2Text,
+int QMessageBox::information( QWidget *parent, const QString &caption,
+			      const QString& text,
+			      const QString& button0Text,
+			      const QString& button1Text,
+			      const QString& button2Text,
 			      int defaultButtonNumber,
 			      int escapeButtonNumber )
 {
@@ -1169,7 +1247,8 @@ int QMessageBox::information( QWidget *parent, const char *caption,
   1-3 buttons.  Returns the number of the button that was clicked
   (0, 1 or 2).
 
-  \a button0Text is the text of the first button and must be present,
+  \a button0Text is the text of the first button and is optional.  If
+  \a button0Text is not supplied, "OK" (translated) will be used.
   \a button1Text is the text of the second button and is optional, and
   \a button2Text is the text of the third button and is optional.  \a
   defaultbuttonNumber (0-2) is the index of the default button;
@@ -1187,11 +1266,11 @@ int QMessageBox::information( QWidget *parent, const char *caption,
   \sa information(), critical()
 */
 
-int QMessageBox::warning( QWidget *parent, const char *caption,
-				 const char *text,
-				 const char *button0Text,
-				 const char *button1Text,
-				 const char *button2Text,
+int QMessageBox::warning( QWidget *parent, const QString &caption,
+				 const QString& text,
+				 const QString& button0Text,
+				 const QString& button1Text,
+				 const QString& button2Text,
 				 int defaultButtonNumber,
 				 int escapeButtonNumber )
 {
@@ -1206,7 +1285,8 @@ int QMessageBox::warning( QWidget *parent, const char *caption,
   1-3 buttons.  Returns the number of the button that was clicked
   (0, 1 or 2).
 
-  \a button0Text is the text of the first button and must be present,
+  \a button0Text is the text of the first button and is optional.  If
+  \a button0Text is not supplied, "OK" (translated) will be used.
   \a button1Text is the text of the second button and is optional, and
   \a button2Text is the text of the third button and is optional.  \a
   defaultbuttonNumber (0-2) is the index of the default button;
@@ -1224,11 +1304,11 @@ int QMessageBox::warning( QWidget *parent, const char *caption,
   \sa information() warning()
 */
 
-int QMessageBox::critical( QWidget *parent, const char *caption,
-				  const char *text,
-				  const char *button0Text,
-				  const char *button1Text,
-				  const char *button2Text,
+int QMessageBox::critical( QWidget *parent, const QString &caption,
+				  const QString& text,
+				  const QString& button0Text,
+				  const QString& button1Text,
+				  const QString& button2Text,
 				  int defaultButtonNumber,
 				  int escapeButtonNumber )
 {
@@ -1239,27 +1319,99 @@ int QMessageBox::critical( QWidget *parent, const char *caption,
 
 
 static const char *textAboutQt =
-"This program is developed with Qt, the multi-platform C++ GUI toolkit.\n\n"
-"Qt is a product of Troll Tech AS (http://www.troll.no).\n"
-"Qt is available under two different licenses:\n"
-"- The Free Edition, which may be used free of charge to develop\n"
-"  Free Software on the X Window System.\n"
-"- The Professional Edition, which may be used to develop commercial\n"
-"  software on both X and Microsoft Windows.\n\n"
-"Please contact sales@troll.no for information and pricing.";
+"<h3>About Qt</h3><p>This program uses Qt version " QT_VERSION_STR ", a "
+"multiplatform C++ GUI toolkit from Troll Tech. "
+"Qt provides single-source portability across Windows 95/98/NT/2000, "
+"Linux, Solaris, HP-UX and many other versions of Unix with X11.</p>"
+"<p>See <tt>http://www.trolltech.com/qt/</tt> for more information.</p>";
+void QT_END_TEXT() { QMessageBox::aboutQt(0,"E" "g" "g"); }
 
 
 /*!
   Displays a simple message box about Qt, with window caption \a
-  caption and optionally centered over \a parent.
+  caption and optionally centered over \a parent.  The message includes
+  the version number of Qt being used by the application.
 
   This is neat for inclusion into the Help menu.  See the menu.cpp
   example.
 */
 
-void QMessageBox::aboutQt( QWidget *parent, const char *caption )
+void QMessageBox::aboutQt( QWidget *parent, const QString &caption )
 {
-    if ( !caption )
-	caption = "About Qt";
-    information( parent, caption, textAboutQt );
+    QString c = caption;
+    if ( c.isNull() )
+	c = "About Qt";
+    QMessageBox *mb = new QMessageBox( parent, "about qt" );
+    mb->setCaption( caption.isNull()?QString::fromLatin1("About Qt"):caption );
+    mb->setText( qApp->translate( "QMessageBox", textAboutQt ) );
+    QPixmap pm;
+    QImage logo( (const char **)qtlogo_xpm);
+    if ( qGray(mb->palette().active().text().rgb()) >
+         qGray(mb->palette().active().base().rgb()) )
+    {
+	// light on dark, adjust some colors
+	logo.setColor( 0,0xFFffffff);
+	logo.setColor( 1,0xFF666666);
+	logo.setColor( 2,0xFFcccc66);
+	logo.setColor( 4,0xFFcccccc);
+	logo.setColor( 6,0xFFffff66);
+	logo.setColor( 7,0xFF999999);
+	logo.setColor( 8,0xFF3333FF);
+	logo.setColor( 9,0xFFffff33);
+	logo.setColor(11,0xFFcccc99);
+    }
+    if ( pm.convertFromImage( logo ) )
+	mb->setIconPixmap( pm );
+    mb->setButtonText( 0, tr("OK") );
+    if ( mb->mbd && mb->mbd->pb[0] ) {
+	mb->mbd->pb[0]->setAutoDefault( TRUE );
+	mb->mbd->pb[0]->setFocusPolicy( QWidget::StrongFocus );
+	mb->mbd->pb[0]->setDefault( TRUE );
+	mb->mbd->pb[0]->setFocus();
+    }
+    mb->exec();
 }
+
+/*!
+  \reimp
+*/
+
+void QMessageBox::setIcon( const QPixmap &pix )
+{
+    //reimplemented to avoid compiler warning.
+    QDialog::setIcon( pix );
+}
+
+
+/*!
+  Returns the current text format.
+
+  \sa setTextFormat()
+*/
+
+Qt::TextFormat QMessageBox::textFormat() const
+{
+    return label->textFormat();
+}
+
+/*!
+  Sets the text format to \a format. See the Qt::TextFormat enum for
+  an explanation of the possible options.
+
+  The default format is \c AutoText.
+
+  \sa textFormat(), setText()
+*/
+
+void QMessageBox::setTextFormat( Qt::TextFormat format )
+{
+    label->setTextFormat( format );
+}
+
+
+// My own personal favorite minimalist error message popped up whilst
+// testing Freehand 8 last month.  I took a screen shot.  I believe I
+// was trying to convert a file from one format to another.
+// Apparently, I...
+//
+// http://www.people.cornell.edu/pages/mlj8/cant.gif
