@@ -1,33 +1,37 @@
 /****************************************************************************
-** $Id: qasyncimageio.cpp,v 1.38.2.6 1999/02/13 14:57:55 hanord Exp $
+** $Id: qt/src/kernel/qasyncimageio.cpp   2.1.1   edited 2000-03-24 $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qasyncimageio.h"
 #include "qpainter.h"
 #include "qlist.h"
-#include "qt_gif.h"
+#include "qgif.h"
 #include <stdlib.h>
 
 
+
+// NOT REVISED
 /*!
   \class QImageConsumer qasyncimageio.h
   \brief An abstraction used by QImageDecoder.
@@ -57,8 +61,28 @@
 /*!
   \fn void QImageConsumer::frameDone()
 
-  Called when a frame of an animated image has ended and been revealed
-  as changed().  The decoder will not make
+  One of the two frameDone() functions will be called
+  when a frame of an animated image has ended and been revealed
+  as changed().
+
+  When this funciton is called, the current image should be displayed.
+
+  The decoder will not make
+  any further changes to the image until the next call to
+  QImageFormat::decode().
+*/
+
+/*!
+  \fn void QImageConsumer::frameDone( const QPoint& offset, const QRect& rect )
+
+  One of the two frameDone() functions will be called
+  when a frame of an animated image has ended and been revealed
+  as changed().
+
+  When this funciton is called, the area \a rect in the current image
+  should be moved by \a offset and displayed.
+
+  The decoder will not make
   any further changes to the image until the next call to
   QImageFormat::decode().
 */
@@ -92,7 +116,7 @@
   is called.  Of course, the image may not have been decoded by then, in
   which case the frame should not be displayed until it is complete.
   A value of -1 (the assumed default) indicates that the image should
-  be diplayed even while it is only partially loaded.
+  be displayed even while it is only partially loaded.
 */
 
 /*!
@@ -113,6 +137,14 @@
   QImageFormatType, and the QMovie class can be used for using
   all installed incremental image formats; QImageDecoder is only
   useful for creating new ways of feeding data to an QImageConsumer.
+  
+  \mustquote
+
+  Qt supports GIF reading, if it is configured that way during
+  installation (see qgif.h). If it is, we are required to state that
+  "The Graphics Interchange Format(c) is the Copyright property of
+  CompuServe Incorporated. GIF(sm) is a Service Mark property of
+  CompuServe Incorporated."
 */
 
 static const int max_header = 32;
@@ -121,7 +153,7 @@ static const int max_header = 32;
 
 
 
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 class Q_EXPORT QGIFFormat : public QImageFormat {
 public:
@@ -215,8 +247,8 @@ struct QImageDecoderPrivate {
     static void ensureFactories()
     {
 	if ( !factories ) {
-	    factories = new QListT<QImageFormatType>;
-// See qt_gif.h for important information regarding this option
+	    factories = new QList<QImageFormatType>;
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 	    gif_decoder_factory = new QGIFFormatType;
 #endif
@@ -226,7 +258,7 @@ struct QImageDecoderPrivate {
 
     static QList<QImageFormatType> * factories;
 
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
     static QGIFFormatType * gif_decoder_factory;
 #endif
@@ -236,7 +268,7 @@ struct QImageDecoderPrivate {
 };
 
 QList<QImageFormatType> * QImageDecoderPrivate::factories = 0;
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 QGIFFormatType * QImageDecoderPrivate::gif_decoder_factory = 0;
 #endif
@@ -246,7 +278,7 @@ void QImageDecoderPrivate::cleanup()
 {
     delete factories;
     factories = 0;
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
     delete gif_decoder_factory;
     gif_decoder_factory = 0;
@@ -290,7 +322,7 @@ QImageDecoder::~QImageDecoder()
   returns FALSE.
 
   Returns the number of bytes consumed, 0 if consumption is complete,
-  and -1 if decoding fails dur to invalid data.
+  and -1 if decoding fails due to invalid data.
 */
 int QImageDecoder::decode(const uchar* buffer, int length)
 {
@@ -301,7 +333,7 @@ int QImageDecoder::decode(const uchar* buffer, int length)
 	while (consumed < length && d->count < max_header) {
 	    d->header[d->count++] = buffer[consumed++];
 	}
-	
+
 	QImageDecoderPrivate::ensureFactories();
 
 	for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
@@ -333,6 +365,26 @@ int QImageDecoder::decode(const uchar* buffer, int length)
     }
 }
 
+/*!
+  Returns a QImageFormatType by name. This might be used in cases where
+  the user needs to force data to be interpretted as being in a certain
+  format.  \a name is one of the formats listed by
+  QImageDecoder::inputFormats(). Note that you will still need to supply
+  decodable data to result->decoderFor() before you can begin decoding
+  the data.
+*/
+QImageFormatType* QImageDecoder::format( const char* name )
+{
+    for (QImageFormatType* f = QImageDecoderPrivate::factories->first();
+	f;
+	f = QImageDecoderPrivate::factories->next())
+    {
+	if ( stricmp(name,f->formatName())==0 )
+	    return f;
+    }
+    return 0;
+}
+
 /*!
   Call this function to find the name of the format of the given header.
   The returned string is statically allocated.
@@ -413,7 +465,7 @@ void QImageDecoder::unregisterDecoderFactory(QImageFormatType* f)
   be sources for a QMovie, or for the first frame of the image stream
   to be loaded as a QImage or QPixmap.
 
-  Your new subclass must override the decode() function in order to
+  Your new subclass must reimplement the decode() function in order to
   process your new format.
 
   New QImageFormat objects are generated by new QImageFormatType factories.
@@ -433,7 +485,7 @@ QImageFormat::~QImageFormat()
   \fn int QImageFormat::decode(QImage& img, QImageConsumer* consumer,
 	    const uchar* buffer, int length)
 
-  New subclasses must override this method.
+  New subclasses must reimplement this method.
 
   It should decode some or all of the bytes from \a buffer into
   \a img, calling the methods of \a consumer as the decoding proceeds to
@@ -479,7 +531,7 @@ QImageFormat::~QImageFormat()
 	    uchar* buffer, int length)
 
   Returns a decoder for decoding an image which starts with the give bytes.
-  This function should only return a decoder if it is definate that the
+  This function should only return a decoder if it is definite that the
   decoder applies to data with the given header.  Returns 0 if there is
   insufficient data in the header to make a positive identification,
   or if the data is not recognized.
@@ -493,7 +545,7 @@ QImageFormat::~QImageFormat()
 */
 
 /*!
-  Creates a factory.  It automatically registers itself with QImageDecoder.
+  Constructs a factory.  It automatically registers itself with QImageDecoder.
 */
 QImageFormatType::QImageFormatType()
 {
@@ -509,6 +561,10 @@ QImageFormatType::~QImageFormatType()
 }
 
 
+/*!
+  Returns TRUE if Qt was compiled with built-in GIF reading support,
+  otherwise FALSE.
+*/
 bool qt_builtin_gif_reader()
 {
 #if defined(QT_BUILTIN_GIF_READER)
@@ -518,7 +574,7 @@ bool qt_builtin_gif_reader()
 #endif
 }
 
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 
 /* -- NOTDOC
@@ -551,7 +607,7 @@ QGIFFormat::QGIFFormat()
 */
 QGIFFormat::~QGIFFormat()
 {
-    delete[] globalcmap;
+    if (globalcmap) delete[] globalcmap;
 }
 
 
@@ -577,7 +633,7 @@ QImageFormat* QGIFFormatType::decoderFor(
      && buffer[3]=='8'
      && (buffer[4]=='9' || buffer[4]=='7')
      && buffer[5]=='a')
-        return new QGIFFormat;
+	return new QGIFFormat;
     return 0;
 }
 
@@ -912,8 +968,10 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		} else {
 		    if (needfirst) {
 			firstcode=oldcode=code;
-			if (!out_of_bounds && !(preserve_trans && firstcode==trans))
-			    line[y][x] = firstcode;
+			if (!out_of_bounds) {
+			    if (!(preserve_trans && firstcode==trans))
+				line[y][x] = firstcode;
+			}
 			x++;
 			if (x>=swidth) out_of_bounds = TRUE;
 			needfirst=FALSE;
@@ -957,8 +1015,10 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			oldcode=incode;
 			while (sp>stack) {
 			    --sp;
-			    if (!out_of_bounds && !(preserve_trans && *sp==trans))
-				line[y][x] = *sp;
+			    if (!out_of_bounds) {
+				if (!(preserve_trans && *sp==trans))
+				    line[y][x] = *sp;
+			    }
 			    x++;
 			    if (x>=swidth) out_of_bounds = TRUE;
 			    if (x>right) {
@@ -1121,32 +1181,36 @@ void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 	    int i;
 	    my = QMIN(7, bottom-y);
 	    for (i=1; i<=my; i++)
-	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
+		memcpy(img.scanLine(y+i)+left, img.scanLine(y)+left,
+		    right-left+1);
 	    if (consumer && !out_of_bounds)
-	        consumer->changed(QRect(left, y, right-left+1, my+1));
+		consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
-	    if (y>bottom) { interlace++; y=4; }
+	    if (y>bottom) { interlace++; y=top+4; }
 	} break;
       case 2:
 	{
 	    int i;
 	    my = QMIN(3, bottom-y);
 	    for (i=1; i<=my; i++)
-	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
-    	    if (consumer && !out_of_bounds)
-	        consumer->changed(QRect(left, y, right-left+1, my+1));
+		memcpy(img.scanLine(y+i)+left, img.scanLine(y)+left,
+		    right-left+1);
+	    if (consumer && !out_of_bounds)
+		consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
-	    if (y>bottom) { interlace++; y=2; }
+	    if (y>bottom) { interlace++; y=top+2; }
 	} break;
       case 3:
 	{
+	    int i;
 	    my = QMIN(1, bottom-y);
-	    for (int i=1; i<=my; i++)
-	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
+	    for (i=1; i<=my; i++)
+		memcpy(img.scanLine(y+i)+left, img.scanLine(y)+left,
+		    right-left+1);
 	    if (consumer && !out_of_bounds)
-	        consumer->changed(QRect(left, y, right-left+1, my+1));
+		consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=4;
-	    if (y>bottom) { interlace++; y=1; }
+	    if (y>bottom) { interlace++; y=top+1; }
 	} break;
       case 4:
 	if (consumer && !out_of_bounds)
