@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qasyncio.cpp,v 1.10.2.1 1998/08/12 16:55:06 agulbra Exp $
+** $Id: qt/src/kernel/qasyncio.cpp   2.1.1   edited 2000-05-06 $
 **
 ** Implementation of asynchronous I/O classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,21 +28,21 @@
 #include "qtimer.h"
 #include <stdlib.h>
 
+// NOT REVISED
 /*!
   \class QAsyncIO qasyncio.h
   \brief Encapsulates I/O asynchronicity.
 
   The Qt classes for asynchronous input/output provide a simple
-  mechanism to allow large files or slow datasources to be processed
+  mechanism to allow large files or slow data sources to be processed
   without using large amounts of memory, or blocking the user interface.
 
-  This facility is used in Qt to drive animated images.  See QImageConsumer
-  and QImageSource.
+  This facility is used in Qt to drive animated images.  See QImageConsumer.
 */
 
 
 /*!
-  Destroys the async IO object.
+  Destructs the async IO object.
 */
 QAsyncIO::~QAsyncIO()
 {
@@ -49,7 +51,7 @@ QAsyncIO::~QAsyncIO()
 /*!
   Ensures only one object can respond to changes in readiness.
 */
-void QAsyncIO::connect(QObject* obj, const char* member)
+void QAsyncIO::connect(QObject* obj, const char *member)
 {
     signal.disconnect(0, 0);
     signal.connect(obj, member);
@@ -148,7 +150,7 @@ void QDataSource::maybeReady()
   it to the given data sink.  The count will be no more than the amount
   indicated by the most recent call to readyToSend().  The source must
   use all the provided data, and the sink will be prepared to accept at
-  least this much data.  
+  least this much data.
 */
 
 /*!
@@ -166,7 +168,7 @@ bool QDataSource::rewindable() const
   is TRUE, then the data source must take measures to allow the rewind()
   function to subsequently operate as described.  If rewindable() is FALSE,
   the function should call QDataSource::enableRewind(), which aborts with
-  a fatal() error.
+  a qFatal() error.
 
   For example, a network connection may choose to utilize a disk cache
   of input only if rewinding is enabled before the first buffer-full of
@@ -175,7 +177,7 @@ bool QDataSource::rewindable() const
 */
 void QDataSource::enableRewind(bool)
 {
-    fatal("Attempted to make unrewindable QDataSource rewindable");
+    qFatal("Attempted to make unrewindable QDataSource rewindable");
 }
 
 /*!
@@ -184,7 +186,7 @@ void QDataSource::enableRewind(bool)
 */
 void QDataSource::rewind()
 {
-    fatal("Attempted to rewind unrewindable QDataSource");
+    qFatal("Attempted to rewind unrewindable QDataSource");
 }
 
 /*!
@@ -211,7 +213,7 @@ QIODeviceSource::QIODeviceSource(QIODevice* device, int buffer_size) :
 }
 
 /*!
-  Destroys the QIODeviceSource, deleting the QIODevice from which it was
+  Destructs the QIODeviceSource, deleting the QIODevice from which it was
   constructed.
 */
 QIODeviceSource::~QIODeviceSource()
@@ -257,7 +259,7 @@ void QIODeviceSource::enableRewind(bool on)
     rew = on;
 }
 
-/*
+/*!
   Calls reset() on the QIODevice.
 */
 void QIODeviceSource::rewind()
@@ -288,7 +290,7 @@ void QIODeviceSource::rewind()
 */
 
 /*!
-  Creates a QDataPump to move data from a given \a data_source
+  Constructs a QDataPump to move data from a given \a data_source
   to a given \a data_sink.
 */
 QDataPump::QDataPump(QDataSource* data_source, QDataSink* data_sink) :
@@ -311,23 +313,21 @@ void QDataPump::kickStart()
 void QDataPump::tryToPump()
 {
     int supply, demand;
-    do {
-	supply = source->readyToSend();
-	demand = sink->readyToReceive();
-	if (demand <= 0) {
-	    return;
-	}
-	interval = 0;
-	if (supply < 0) {
-	    // All done (until source signals change in readiness) 
-	    sink->eof();
-	    return;
-	}
-	if (!supply)
-	    return;
-	source->sendTo(sink, QMIN(supply, demand));
-    } while (1);
 
-    // ### DOESN'T GET HERE ANYMORE
+    supply = source->readyToSend();
+    demand = sink->readyToReceive();
+    if (demand <= 0) {
+	return;
+    }
+    interval = 0;
+    if (supply < 0) {
+	// All done (until source signals change in readiness)
+	sink->eof();
+	return;
+    }
+    if (!supply)
+	return;
+    source->sendTo(sink, QMIN(supply, demand));
+
     timer.start(0, TRUE);
 }
