@@ -1,38 +1,43 @@
 /****************************************************************************
-** $Id: qcolor.cpp,v 2.16.2.2 1998/11/01 20:31:32 agulbra Exp $
+** $Id: qt/src/kernel/qcolor.cpp   2.1.1   edited 2000-05-22 $
 **
 ** Implementation of QColor class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qcolor.h"
+#include "qnamespace.h"
 #include "qdatastream.h"
 #include <stdlib.h>
 #include <ctype.h>
 
 
+// NOT REVISED
 /*!
   \class QColor qcolor.h
   \brief The QColor class provides colors based on RGB.
 
   \ingroup color
   \ingroup drawing
+  \ingroup appearance
 
   A color is normally specified in terms of RGB (red,green and blue)
   components, but it is also possible to specify HSV (hue,saturation
@@ -43,18 +48,18 @@
   value is used by the underlying window system to refer to a color.  It
   can be thought of as an index into the display hardware's color table.
 
-  There are 19 predefined global QColor objects:
-  \c black, \c white, \c darkGray, \c gray, \c lightGray, \c red, \c green,
-  \c blue, \c cyan, \c magenta, \c yellow, \c darkRed, \c darkGreen,
-  \c darkBlue, \c darkCyan, \c darkMagenta, \c darkYellow, \c color0 and
-  \c color1.
+  There are 19 predefined QColor objects: \c black, \c white, \c
+  darkGray, \c gray, \c lightGray, \c red, \c green, \c blue, \c cyan,
+  \c magenta, \c yellow, \c darkRed, \c darkGreen, \c darkBlue, \c
+  darkCyan, \c darkMagenta, \c darkYellow, \c color0 and \c color1.
 
-  The colors \c color0 (zero pixel value) and \c color1 (non-zero pixel value)
-  are special colors for drawing in \link QBitmap bitmaps\endlink.
+  The colors \c color0 (zero pixel value) and \c color1 (non-zero
+  pixel value) are special colors for drawing in \link QBitmap
+  bitmaps\endlink.
 
-  The QColor class has an efficient, dynamic color allocation strategy.
-  A color is normally allocated the first time it is used (lazy allocation),
-  that is, whenever the pixel() function is called:
+  The QColor class has an efficient, dynamic color allocation
+  strategy.  A color is normally allocated the first time it is used
+  (lazy allocation), that is, whenever the pixel() function is called:
 
   <ol>
   <li>Is the pixel value valid? If it is, just return it, otherwise,
@@ -92,7 +97,8 @@
   -10), S=255, V=180.  A grayish light red could have H about 0 (say
   350-359 or 0-10), S about 50-100, and S=255.
 
-  \sa QPalette, QColorGroup, QApplication::setColorSpec()
+  \sa QPalette, QColorGroup, QApplication::setColorSpec(),
+  <a href="http://www.inforamp.net/~poynton/Poynton-color.html">Color FAQ.</a>
 */
 
 /*****************************************************************************
@@ -107,25 +113,27 @@
 #define COLOR1_PIX 1
 #endif
 
-const QColor color0	( 0x00ffffff, COLOR0_PIX );
-const QColor color1	( 0x00000000, COLOR1_PIX );
-const QColor black	(   0,	 0,   0 );
-const QColor white	( 255, 255, 255 );
-const QColor darkGray	( 128, 128, 128 );
-const QColor gray	( 160, 160, 164 );
-const QColor lightGray	( 192, 192, 192 );
-const QColor red	( 255,	 0,   0 );
-const QColor green	(   0, 255,   0 );
-const QColor blue	(   0,	 0, 255 );
-const QColor cyan	(   0, 255, 255 );
-const QColor magenta	( 255,	 0, 255 );
-const QColor yellow	( 255, 255,   0 );
-const QColor darkRed	( 128,	 0,   0 );
-const QColor darkGreen	(   0, 128,   0 );
-const QColor darkBlue	(   0,	 0, 128 );
-const QColor darkCyan	(   0, 128, 128 );
-const QColor darkMagenta( 128,	 0, 128 );
-const QColor darkYellow ( 128, 128,   0 );
+static QColor stdcol[19];
+
+QT_STATIC_CONST_IMPL QColor & Qt::color0 = stdcol[0];
+QT_STATIC_CONST_IMPL QColor & Qt::color1  = stdcol[1];
+QT_STATIC_CONST_IMPL QColor & Qt::black  = stdcol[2];
+QT_STATIC_CONST_IMPL QColor & Qt::white = stdcol[3];
+QT_STATIC_CONST_IMPL QColor & Qt::darkGray = stdcol[4];
+QT_STATIC_CONST_IMPL QColor & Qt::gray = stdcol[5];
+QT_STATIC_CONST_IMPL QColor & Qt::lightGray = stdcol[6];
+QT_STATIC_CONST_IMPL QColor & Qt::red = stdcol[7];
+QT_STATIC_CONST_IMPL QColor & Qt::green = stdcol[8];
+QT_STATIC_CONST_IMPL QColor & Qt::blue = stdcol[9];
+QT_STATIC_CONST_IMPL QColor & Qt::cyan = stdcol[10];
+QT_STATIC_CONST_IMPL QColor & Qt::magenta = stdcol[11];
+QT_STATIC_CONST_IMPL QColor & Qt::yellow = stdcol[12];
+QT_STATIC_CONST_IMPL QColor & Qt::darkRed = stdcol[13];
+QT_STATIC_CONST_IMPL QColor & Qt::darkGreen = stdcol[14];
+QT_STATIC_CONST_IMPL QColor & Qt::darkBlue = stdcol[15];
+QT_STATIC_CONST_IMPL QColor & Qt::darkCyan = stdcol[16];
+QT_STATIC_CONST_IMPL QColor & Qt::darkMagenta = stdcol[17];
+QT_STATIC_CONST_IMPL QColor & Qt::darkYellow = stdcol[18];
 
 
 /*****************************************************************************
@@ -134,11 +142,14 @@ const QColor darkYellow ( 128, 128,   0 );
 
 bool QColor::color_init   = FALSE;		// color system not initialized
 bool QColor::globals_init = FALSE;		// global color not initialized
-bool QColor::lalloc = TRUE;			// lazy color allocation
+bool QColor::lazy_alloc = TRUE;			// lazy color allocation
+
+
+QColor* QColor::globalColors()
+{
+    return stdcol;
+}
 
-#if QT_VERSION == 200
-#error "Rename lalloc to lazy_alloc"
-#endif
 
 /*!
   Initializes the global colors.  This function is called if a global
@@ -159,27 +170,27 @@ bool QColor::lalloc = TRUE;			// lazy color allocation
 void QColor::initGlobalColors()
 {
     globals_init = TRUE;
-    ((QColor*)(&::color0))->pix = COLOR0_PIX;
-    ((QColor*)(&::color1))->pix = COLOR1_PIX;
-    ((QColor*)(&::color0))->rgbVal = 0x00ffffff;
-    ((QColor*)(&::color1))->rgbVal = 0;
-    ((QColor*)(&::black))	->setRgb(   0,	 0,   0 );
-    ((QColor*)(&::white))	->setRgb( 255, 255, 255 );
-    ((QColor*)(&::darkGray))	->setRgb( 128, 128, 128 );
-    ((QColor*)(&::gray))	->setRgb( 160, 160, 164 );
-    ((QColor*)(&::lightGray))	->setRgb( 192, 192, 192 );
-    ((QColor*)(&::red))		->setRgb( 255,	 0,   0 );
-    ((QColor*)(&::green))	->setRgb(   0, 255,   0 );
-    ((QColor*)(&::blue))	->setRgb(   0,	0,  255 );
-    ((QColor*)(&::cyan))	->setRgb(   0, 255, 255 );
-    ((QColor*)(&::magenta))	->setRgb( 255,	0,  255 );
-    ((QColor*)(&::yellow))	->setRgb( 255, 255,   0 );
-    ((QColor*)(&::darkRed))	->setRgb( 128,	0,    0 );
-    ((QColor*)(&::darkGreen))	->setRgb(   0, 128,   0 );
-    ((QColor*)(&::darkBlue))	->setRgb(   0,	0,  128 );
-    ((QColor*)(&::darkCyan))	->setRgb(   0, 128, 128 );
-    ((QColor*)(&::darkMagenta)) ->setRgb( 128,	0,  128 );
-    ((QColor*)(&::darkYellow))	->setRgb( 128, 128,   0 );
+    stdcol[ 0].pix = COLOR0_PIX;
+    stdcol[ 1].pix = COLOR1_PIX;
+    stdcol[ 0].rgbVal = 0x00ffffff;
+    stdcol[ 1].rgbVal = 0;
+    stdcol[ 2].setRgb(   0,   0,   0 );
+    stdcol[ 3].setRgb( 255, 255, 255 );
+    stdcol[ 4].setRgb( 128, 128, 128 );
+    stdcol[ 5].setRgb( 160, 160, 164 );
+    stdcol[ 6].setRgb( 192, 192, 192 );
+    stdcol[ 7].setRgb( 255,   0,   0 );
+    stdcol[ 8].setRgb(   0, 255,   0 );
+    stdcol[ 9].setRgb(   0,   0, 255 );
+    stdcol[10].setRgb(   0, 255, 255 );
+    stdcol[11].setRgb( 255,   0, 255 );
+    stdcol[12].setRgb( 255, 255,   0 );
+    stdcol[13].setRgb( 128,   0,   0 );
+    stdcol[14].setRgb(   0, 128,   0 );
+    stdcol[15].setRgb(   0,   0, 128 );
+    stdcol[16].setRgb(   0, 128, 128 );
+    stdcol[17].setRgb( 128,   0, 128 );
+    stdcol[18].setRgb( 128, 128,   0 );
 }
 
 
@@ -216,7 +227,7 @@ QColor::QColor( QRgb rgb, uint pixel )
     if ( pixel == 0xffffffff ) {
 	setRgb( rgb );
     } else {
-	rgbVal = rgb | RGB_DIRECT;
+	rgbVal = (rgb & RGB_MASK) | RGB_DIRECT;
 	pix    = pixel;
     }
 }
@@ -247,16 +258,28 @@ QColor::QColor( int x, int y, int z, Spec colorSpec )
 
 
 /*!
-  Constructs a named color, i.e. loads the color from the color database.
+  Constructs a named color in the same way as setNamedColor().
+  \sa setNamedColor()
+*/
+
+QColor::QColor( const QString& name )
+{
+    setNamedColor( name );
+}
+
+
+/*!
+  Constructs a named color in the same way as setNamedColor().
   \sa setNamedColor()
 */
 
 QColor::QColor( const char *name )
 {
-    setNamedColor( name );
+    setNamedColor( QString(name) );
 }
 
 
+
 /*!
   Constructs a color that is a copy of \a c.
 */
@@ -270,56 +293,6 @@ QColor::QColor( const QColor &c )
 }
 
 
-inline static int hex2int( char hexchar )
-{
-    int v;
-    if ( isdigit(hexchar) )
-	v = hexchar - '0';
-    else if ( isalpha(hexchar) )
-	v = toupper(hexchar) - 'A' + 10;
-    else
-	v = 0;
-    return v;
-}
-
-
-/*!
-  Sets the RGB value to that of the named color.
-
-  The "#RRGGBB" color format is supported on all platforms, where
-  RR, GG, and BB and hex digits.
-
-  A color name may also be a color in the X color database (rgb.txt) ,
-  e.g.  "steelblue" or "gainsboro".  The X color names also work under
-  Qt for Windows.
-*/
-
-void QColor::setNamedColor( const char *name )
-{
-    if ( name == 0 || *name == '\0' ) {
-	setRgb( 0 );
-    } else if ( name[0] == '#' ) {
-	const char *p = &name[1];
-	int len = strlen(p);
-	int r, g, b;
-	if ( len == 6 ) {
-	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
-	    g = (hex2int(p[2]) << 4) + hex2int(p[3]);
-	    b = (hex2int(p[4]) << 4) + hex2int(p[5]);
-	} else if ( len == 3 ) {
-	    r = hex2int(p[0]);
-	    g = hex2int(p[1]);
-	    b = hex2int(p[2]);
-	} else {
-	    r = g = b = 0;
-	}
-	setRgb( r, g, b );
-    } else {
-	setSystemNamedColor( name );
-    }
-}
-
-
 /*!
   Assigns a copy of the color \c and returns a reference to this color.
 */
@@ -336,7 +309,7 @@ QColor &QColor::operator=( const QColor &c )
 
 /*!
   \fn bool QColor::isValid() const
-  Returns TRUE if the color is invalid, i.e. it was constructed using the
+  Returns FALSE if the color is invalid, i.e. it was constructed using the
   default constructor.
 */
 
@@ -348,9 +321,94 @@ QColor &QColor::operator=( const QColor &c )
 */
 
 
+/*!
+  Returns the name of the color in the format #RRGGBB.
+
+  \sa setNamedColor()
+*/
+
+QString QColor::name() const
+{
+    QString s;
+    s.sprintf( "#%02x%02x%02x", red(), green(), blue() );
+    return s;
+}
+
+
+inline static int hex2int( QChar hexchar )
+{
+    int v;
+    if ( hexchar.isDigit() )
+	v = hexchar.digitValue();
+    else if ( hexchar >= 'A' && hexchar <= 'F' )
+	v = hexchar.cell() - 'A' + 10;
+    else if ( hexchar >= 'a' && hexchar <= 'f' )
+	v = hexchar.cell() - 'a' + 10;
+    else
+	v = 0;
+    return v;
+}
+
+
+/*!
+  Sets the RGB value to that of the named color.
+
+  The color name may be in one of five formats: <ul>
+  <li> #RGB (each of R, G and B is a single hex digit)
+  <li> #RRGGBB
+  <li> #RRRGGGBBB
+  <li> #RRRRGGGGBBBB
+  <li> A named from the X color database (rgb.txt),  e.g.
+  "steelblue" or "gainsboro".  These color names also work
+  under Qt for Windows.
+  </ul>
+*/
+
+void QColor::setNamedColor( const QString& name )
+{
+    if ( name.isEmpty() ) {
+	setRgb( 0 );
+    } else if ( name[0] == '#' ) {
+	const QChar *p = name.unicode()+1;
+	int len = name.length()-1;
+	int r, g, b;
+	if ( len == 12 ) {
+	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
+	    g = (hex2int(p[4]) << 4) + hex2int(p[5]);
+	    b = (hex2int(p[8]) << 4) + hex2int(p[9]);
+	} else if ( len == 9 ) {
+	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
+	    g = (hex2int(p[3]) << 4) + hex2int(p[4]);
+	    b = (hex2int(p[6]) << 4) + hex2int(p[7]);
+	} else if ( len == 6 ) {
+	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
+	    g = (hex2int(p[2]) << 4) + hex2int(p[3]);
+	    b = (hex2int(p[4]) << 4) + hex2int(p[5]);
+	} else if ( len == 3 ) {
+	    r = hex2int(p[0]) << 4;
+	    g = hex2int(p[1]) << 4;
+	    b = hex2int(p[2]) << 4;
+	} else {
+	    r = g = b = 0;
+	}
+	setRgb( r, g, b );
+    } else {
+	setSystemNamedColor( name );
+    }
+}
+
+
 #undef max
 #undef min
 
+/*! \obsolete */
+
+void QColor::getHsv( int &h, int &s, int &v ) const
+{
+    hsv( &h, &s, &v );
+}
+
+
 /*!
   Returns the current RGB value as HSV.
 
@@ -367,9 +425,9 @@ QColor &QColor::operator=( const QColor &c )
 
 void QColor::hsv( int *h, int *s, int *v ) const
 {
-    int r = (int)(rgbVal & 0xff);
-    int g = (int)((rgbVal >> 8) & 0xff);
-    int b = (int)((rgbVal >> 16) & 0xff);
+    int r = qRed(rgbVal);
+    int g = qGreen(rgbVal);
+    int b = qBlue(rgbVal);
     uint max = r;				// maximum RGB component
     int whatmax = 0;				// r=>0, g=>1, b=>2
     if ( (uint)g > max ) {
@@ -427,7 +485,7 @@ void QColor::setHsv( int h, int s, int v )
 {
 #if defined(CHECK_RANGE)
     if ( h < -1 || (uint)s > 255 || (uint)v > 255 ) {
-	warning( "QColor::setHsv: HSV parameters out of range" );
+	qWarning( "QColor::setHsv: HSV parameters out of range" );
 	return;
     }
 #endif
@@ -465,11 +523,9 @@ void QColor::setHsv( int h, int s, int v )
   \fn QRgb QColor::rgb() const
   Returns the RGB value.
 
-  Bits 0-7 = red, bits 8-15 = green, bits 16-23 = blue.
-
   The return type \e QRgb is equivalent to \c unsigned \c int.
 
-  \sa setRgb(), hsv()
+  \sa setRgb(), hsv(), qRed(), qBlue(), qGreen()
 */
 
 /*!
@@ -480,9 +536,9 @@ void QColor::setHsv( int h, int s, int v )
 
 void QColor::rgb( int *r, int *g, int *b ) const
 {
-    *r = (int)(rgbVal & 0xff);
-    *g = (int)((rgbVal >> 8) & 0xff);
-    *b = (int)((rgbVal >> 16) & 0xff);
+    *r = qRed(rgbVal);
+    *g = qGreen(rgbVal);
+    *b = qBlue(rgbVal);
 }
 
 
@@ -496,10 +552,10 @@ void QColor::setRgb( int r, int g, int b )
 {
 #if defined(CHECK_RANGE)
     if ( (uint)r > 255 || (uint)g > 255 || (uint)b > 255 )
-	warning( "QColor::setRgb: RGB parameter(s) out of range" );
+	qWarning( "QColor::setRgb: RGB parameter(s) out of range" );
 #endif
-    rgbVal = qRgb(r,g,b);
-    if ( lalloc || !color_init ) {
+    rgbVal = ((r & 0xff) << 16) | ((g & 0xff) << 8) | (b & 0xff);
+    if ( lazy_alloc || !color_init ) {
 	rgbVal |= RGB_DIRTY;			// alloc later
 	pix = 0;
     } else {
@@ -511,11 +567,6 @@ void QColor::setRgb( int r, int g, int b )
 /*!
   Sets the RGB value to \a rgb.
 
-  Bits 0-7 = red, bits 8-15 = green, bits 16-23 = blue.
-
-  \warning The bit encoding may change in a future version of Qt.
-  Please use the qRgb() function to compose RGB triplets.
-
   The type \e QRgb is equivalent to \c unsigned \c int.
 
   \sa rgb(), setHsv()
@@ -523,29 +574,29 @@ void QColor::setRgb( int r, int g, int b )
 
 void QColor::setRgb( QRgb rgb )
 {
-    if ( lalloc || !color_init ) {
-	rgbVal = (rgb & 0x00ffffff) | RGB_DIRTY;// alloc later
+    if ( lazy_alloc || !color_init ) {
+	rgbVal = (rgb & RGB_MASK) | RGB_DIRTY;	// alloc later
 	pix = 0;
     } else {
-	rgbVal = (rgb & 0x00ffffff);
+	rgbVal = (rgb & RGB_MASK);
 	alloc();				// alloc now
     }
 }
 
-
 /*!
   \fn int QColor::red() const
-  Returns the red component of the RGB value.
+  Returns the R (red) component of the RGB value.
 */
 
+
 /*!
   \fn int QColor::green() const
-  Returns the green component of the RGB value.
+  Returns the G (green) component of the RGB value.
 */
 
 /*!
   \fn int QColor::blue() const
-  Returns the blue component of the RGB value.
+  Returns the B (blue) component of the RGB value.
 */
 
 
@@ -652,7 +703,7 @@ QColor QColor::dark( int factor ) const
 
 void QColor::setLazyAlloc( bool enable )
 {
-    lalloc = enable;
+    lazy_alloc = enable;
 }
 
 
@@ -661,7 +712,8 @@ void QColor::setLazyAlloc( bool enable )
   Returns the pixel value.
 
   This value is used by the underlying window system to refer to a color.
-  It can be thought of as an index into the display hardware's color table.
+  It can be thought of as an index into the display hardware's color table,
+  but the value is an arbitrary 32-bit value.
 
   \sa setLazyAlloc(), alloc()
 */
@@ -675,24 +727,31 @@ void QColor::setLazyAlloc( bool enable )
   \relates QColor
   Writes a color object to the stream.
 
-  Serialization format: RGB value serialized as an UINT32.
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QColor &c )
 {
-    return s << (UINT32)c.rgb();
+    Q_UINT32 p = (Q_UINT32)c.rgb();
+    if ( s.version() == 1 )			// Swap red and blue
+	p = ((p << 16) & 0xff0000) | ((p >> 16) & 0xff) | (p & 0xff00ff00);
+    return s << p;
 }
 
 /*!
   \relates QColor
   Reads a color object from the stream.
+
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QColor &c )
 {
-    UINT32 rgb;
-    s >> rgb;
-    c.setRgb( rgb );
+    Q_UINT32 p;
+    s >> p;
+    if ( s.version() == 1 )			// Swap red and blue
+	p = ((p << 16) & 0xff0000) | ((p >> 16) & 0xff) | (p & 0xff00ff00);
+    c.setRgb( p );
     return s;
 }
 
@@ -727,13 +786,19 @@ QDataStream &operator>>( QDataStream &s, QColor &c )
   \relates QColor
   Returns the RGB triplet \a (r,g,b).
 
-  Bits 0-7 = \e r (red), bits 8-15 = \e g (green), bits 16-23 = \e b (blue).
+  The return type \e QRgb is equivalent to \c unsigned \c int.
 
-  \warning The bit encoding may change in a future version of Qt.
+  \sa qRgba(), qRed(), qGreen(), qBlue()
+*/
+
+/*!
+  \fn QRgb qRgba( int r, int g, int b, int a )
+  \relates QColor
+  Returns the RGBA triplet \a (r,g,b,a).
 
   The return type \e QRgb is equivalent to \c unsigned \c int.
 
-  \sa qRed(), qGreen(), qBlue()
+  \sa qRgb(), qRed(), qGreen(), qBlue()
 */
 
 /*!
