@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qcolor_x11.cpp,v 2.32.2.1 1999/01/25 20:22:23 warwick Exp $
+** $Id: qt/src/kernel/qcolor_x11.cpp   2.1.1   edited 2000-04-06 $
 **
 ** Implementation of QColor class for X11
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,10 +27,9 @@
 #include "string.h"
 #include "qpaintdevice.h"
 #include "qapplication.h"
-#define	 GC GC_QQQ
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xos.h>
+#include "qt_x11.h"
+
+// NOT REVISED
 
 /*****************************************************************************
   The color dictionary speeds up color allocation significantly for X11.
@@ -47,18 +48,18 @@ struct QColorData {
     int	 context;				// allocation context
 };
 
-typedef Q_DECLARE(QIntDictM,QColorData)		QColorDict;
-typedef Q_DECLARE(QIntDictIteratorM,QColorData) QColorDictIt;
+typedef QIntDict<QColorData> QColorDict;
+typedef QIntDictIterator<QColorData> QColorDictIt;
 static QColorDict *colorDict  = 0;		// dict of allocated colors
 
 static bool	colors_avail  = TRUE;		// X colors available
 static int	current_alloc_context = 0;	// current color alloc context
+static bool	g_truecolor;			// truecolor visual
 static Visual  *g_vis	= 0;			// visual
 static XColor  *g_carr	= 0;			// color array
 static bool	g_carr_fetch = TRUE;		// perform XQueryColors?
 static int	g_cells = 0;			// number of entries in g_carr
 static bool    *g_our_alloc = 0;		// our allocated colors
-static bool	g_truecolor;
 static uint	red_mask , green_mask , blue_mask;
 static int	red_shift, green_shift, blue_shift;
 static const uint col_std_dict = 419;
@@ -69,9 +70,7 @@ static int	col_div_r;
 static int	col_div_g;
 static int	col_div_b;
 
-extern int	qt_ncols_option;		// defined in qapplication_x11.cpp
-extern int	qt_visual_option;
-extern bool	qt_cmap_option;
+extern int	qt_ncols_option;		// qapplication_x11.cpp
 
 
 /*
@@ -116,46 +115,6 @@ static int find_nearest_color( int r, int g, int b, int* mindist_out )
 }
 
 
-/*
-  Returns a truecolor visual (if there is one). 8-bit TrueColor visuals
-  are ignored, unless the user has explicitly requested -visual TrueColor.
-  The SGI X server usually has an 8 bit default visual, but the application
-  can also ask for a truecolor visual. This is what we do if
-  QApplication::colorSpec() is QApplication::ManyColor.
-*/
-
-static Visual *find_truecolor_visual( Display *dpy, int *depth, int *ncols )
-{
-    XVisualInfo *vi, rvi;
-    int best=0, n, i;
-    int scr = DefaultScreen(dpy);
-    rvi.c_class = TrueColor;
-    rvi.screen  = scr;
-    vi = XGetVisualInfo( dpy, VisualClassMask | VisualScreenMask,
-			 &rvi, &n );
-    if ( vi ) {
-	for ( i=0; i<n; i++ ) {
-	    if ( vi[i].depth > vi[best].depth )
-		best = i;
-	}
-    }
-    Visual *v = DefaultVisual(dpy,scr);
-    if ( !vi || (vi[best].visualid == XVisualIDFromVisual(v)) ||
-	 (vi[best].depth <= 8 && qt_visual_option != TrueColor) )
-    {
-	*depth = DefaultDepth(dpy,scr);
-	*ncols = DisplayCells(dpy,scr);	
-    } else {
-	v = vi[best].visual;
-	*depth = vi[best].depth;
-	*ncols = vi[best].colormap_size;
-    }
-    if ( vi )
-	XFree( (char *)vi );
-    return v;
-}
-
-
 /*****************************************************************************
   QColor misc internal functions
  *****************************************************************************/
@@ -181,7 +140,7 @@ static int highest_bit( uint v )
 
 int QColor::maxColors()
 {
-    return QPaintDevice::x11Cells();
+    return QPaintDevice::x11AppCells();
 }
 
 /*!
@@ -194,7 +153,7 @@ int QColor::maxColors()
 
 int QColor::numBitPlanes()
 {
-    return QPaintDevice::x11Depth();
+    return QPaintDevice::x11AppDepth();
 }
 
 
@@ -206,48 +165,20 @@ int QColor::numBitPlanes()
 
 void QColor::initialize()
 {
+    static const int blackIdx = 2;
+    static const int whiteIdx = 3;
+
     if ( color_init )				// already initialized
 	return;
     color_init = TRUE;
 
-    Display *dpy  = qt_xdisplay();
-    int      scr  = DefaultScreen(dpy);
+    Display *dpy  = QPaintDevice::x11AppDisplay();
+    int      scr  = QPaintDevice::x11AppScreen();
+    int	     ncols= QPaintDevice::x11AppCells();
     int	     spec = QApplication::colorSpec();
-    int	     depth, ncols;
-    Colormap cmap;
-    const int tc = TrueColor;
 
-#if QT_VERSION == 200
-#error "Remove old colorspecs"
-    if ( spec == (int)QApplication::ManyColor ||
-	 qt_visual_option == TrueColor ) {
-#else /* } bracket match */
-    if ( (spec & ((int)QApplication::ManyColor | 2)) ||
-	 (qt_visual_option == TrueColor) ) {
-#endif
-	g_vis = find_truecolor_visual( dpy, &depth, &ncols );
-    } else {
-	g_vis = DefaultVisual(dpy,scr);
-	depth = DefaultDepth(dpy,scr);
-	ncols = DisplayCells(dpy,scr);
-    }
-    g_truecolor = g_vis->c_class == tc;
-    bool defVis = (XVisualIDFromVisual(g_vis) ==
-		   XVisualIDFromVisual(DefaultVisual(dpy,scr)));
-    bool defCmap;
-    if ( g_truecolor )
-	defCmap = defVis;
-    else
-	defCmap = !qt_cmap_option;
-
-    if ( defCmap ) {
-	cmap = DefaultColormap(dpy,scr);
-    } else {
-#if QT_VERSION >= 149
-#error "Add qt_create_colormap similar to the code in qgl"
-#endif
-	cmap = XCreateColormap(dpy, RootWindow(dpy,scr), g_vis, AllocNone);
-    }
+    g_vis = (Visual *)QPaintDevice::x11AppVisual();
+    g_truecolor = g_vis->c_class == TrueColor;
 
     if ( !g_truecolor ) {
 	// Create the g_our_alloc array, which remembers which color pixels
@@ -267,15 +198,6 @@ void QColor::initialize()
 	}
     }
 
-    QPaintDevice::x_display   = dpy;
-    QPaintDevice::x_screen    = scr;
-    QPaintDevice::x_depth     = depth;
-    QPaintDevice::x_cells     = ncols;
-    QPaintDevice::x_colormap  = (uint)cmap;
-    QPaintDevice::x_defcmap   = defCmap;
-    QPaintDevice::x_visual    = g_vis;
-    QPaintDevice::x_defvisual = defVis;
-
     int dictsize;
     if ( g_truecolor ) {			// truecolor
 	dictsize    = 1;			// will not need color dict
@@ -293,22 +215,18 @@ void QColor::initialize()
 
   // Initialize global color objects
 
-    ((QColor*)(&black))->rgbVal = qRgb( 0, 0, 0 );
-    ((QColor*)(&white))->rgbVal = qRgb( 255, 255, 255 );
-    if ( defVis && defCmap ) {
-	((QColor*)(&black))->pix = (uint)BlackPixel( dpy, scr );
-	((QColor*)(&white))->pix = (uint)WhitePixel( dpy, scr );
+    globalColors()[blackIdx].rgbVal = qRgb( 0, 0, 0 ) & RGB_MASK;
+    globalColors()[whiteIdx].rgbVal = qRgb( 255, 255, 255 ) & RGB_MASK;
+    if ( QPaintDevice::x11AppDefaultVisual() &&
+	 QPaintDevice::x11AppDefaultColormap() ) {
+	globalColors()[blackIdx].pix = (uint)BlackPixel( dpy, scr );
+	globalColors()[whiteIdx].pix = (uint)WhitePixel( dpy, scr );
     } else {
-	((QColor*)(&black))->alloc();
-	((QColor*)(&white))->alloc();
+	globalColors()[blackIdx].alloc();
+	globalColors()[whiteIdx].alloc();
     }
 
-#if QT_VERSION == 200
-#error "Remove old colorspecs"
     if ( spec == (int)QApplication::ManyColor ) {
-#else /* } bracket match */
-    if ( spec & ((int)QApplication::ManyColor | 2) ) {
-#endif
 	color_reduce = TRUE;
 
 	switch ( qt_ncols_option ) {
@@ -372,12 +290,14 @@ void QColor::cleanup()
     if ( !color_init )
 	return;
     color_init = FALSE;
-    if ( g_carr )				// Avoid purify complaint
+    if ( g_carr ) {
 	delete [] g_carr;
-    if ( g_our_alloc )				// Avoid purify complaint
+	g_carr = 0;
+    }
+    if ( g_our_alloc ) {
 	delete [] g_our_alloc;
-    if ( !QPaintDevice::x_defcmap )
-	XFreeColormap( QPaintDevice::x__Display(), QPaintDevice::x_colormap );
+	g_our_alloc = 0;
+    }
     if ( colorDict ) {
 	colorDict->setAutoDelete( TRUE );
 	colorDict->clear();
@@ -394,27 +314,28 @@ void QColor::cleanup()
 /*!
   Allocates the RGB color and returns the pixel value.
 
-  Allocating a color means to obtain a pixel value from the RGB specification.
-  The pixel value is an index into the global color table.
+  Allocating a color means to obtain a pixel value from the RGB
+  specification.  The pixel value is an index into the global color
+  table, but should be considered an arbitrary platform-dependent value.
 
-  Calling the pixel() function will allocate automatically if
-  the color was not already allocated.
+  The pixel() function calls alloc() if necessary, so in general you
+  don't need to call this function.
 
   \sa setLazyAlloc(), enterAllocContext()
 */
 
 uint QColor::alloc()
 {
-    Display *dpy = QPaintDevice::x__Display();
-    int      scr = QPaintDevice::x11Screen();
+    Display *dpy = QPaintDevice::x11AppDisplay();
+    int      scr = QPaintDevice::x11AppScreen();
     if ( (rgbVal & RGB_INVALID) || !color_init ) {
 	rgbVal = 0;				// invalid color or state
 	pix = dpy ? (uint)BlackPixel(dpy, scr) : 0;
 	return pix;
     }
-    int r = (int)(rgbVal & 0xff);
-    int g = (int)((rgbVal >> 8) & 0xff);
-    int b = (int)((rgbVal >> 16) & 0xff);
+    int r = qRed(rgbVal);
+    int g = qGreen(rgbVal);
+    int b = qBlue(rgbVal);
     if ( g_truecolor ) {			// truecolor: map to pixel
 	r = red_shift	> 0 ? r << red_shift   : r >> -red_shift;
 	g = green_shift > 0 ? g << green_shift : g >> -green_shift;
@@ -445,12 +366,12 @@ uint QColor::alloc()
 
     do {
 	// This loop is run until we manage to either allocate or
-	// find an approximate color, it stops after 100 iterations.
+	// find an approximate color, it stops after a few iterations.
 
 	try_again = FALSE;
 
 	if ( try_alloc && colors_avail &&
-	     XAllocColor(dpy,QPaintDevice::x11Colormap(),&col) ) {
+	     XAllocColor(dpy,QPaintDevice::x11AppColormap(),&col) ) {
 
 	    // We could allocate the color
 	    pix = (uint)col.pixel;
@@ -460,12 +381,13 @@ uint QColor::alloc()
 		g_our_alloc[pix] = TRUE;	// reuse without XAllocColor
 
 	} else {
-	    // No available colors
+	    // No available colors, or we did not want to allocate one
 	    int i;
 	    colors_avail = FALSE;		// no more available colors
 	    if ( g_carr_fetch ) {		// refetch color array
 		g_carr_fetch = FALSE;
-		XQueryColors(dpy, QPaintDevice::x11Colormap(), g_carr,g_cells);
+		XQueryColors( dpy, QPaintDevice::x11AppColormap(), g_carr,
+			      g_cells );
 	    }
 	    int mindist;
 	    i = find_nearest_color( r, g, b, &mindist );
@@ -478,7 +400,7 @@ uint QColor::alloc()
 		int rx = rr - r;
 		int gx = rg - g;
 		int bx = rb - b;
-		int dist = rx*rx + gx*gx + bx*bx;		// calculate distance
+		int dist = rx*rx + gx*gx + bx*bx; // calculate distance
 		if ( dist < mindist ) {
 		    // reduced color is closer - try to alloc it
 		    r = rr;
@@ -490,7 +412,7 @@ uint QColor::alloc()
 		    try_alloc = TRUE;
 		    try_again = TRUE;
 		    colors_avail = TRUE;
-		    continue; // Try alloc reduced colour
+		    continue; // Try alloc reduced color
 		}
 	    }
 
@@ -500,11 +422,11 @@ uint QColor::alloc()
 		return pix;
 	    }
 	    if ( g_our_alloc[i] ) {		// we've already allocated it
-		i = (uint)g_carr[i].pixel;//#### not needed
+		; // i == g_carr[i].pixel
 	    } else {
 		// Try to allocate existing color
 		col = g_carr[i];
-		if ( XAllocColor(dpy,QPaintDevice::x11Colormap(), &col) ) {
+		if ( XAllocColor(dpy,QPaintDevice::x11AppColormap(), &col) ) {
 		    i = (uint)col.pixel;
 		    g_carr[i] = col;		// update color array
 		    if ( current_alloc_context == 0 )
@@ -523,7 +445,7 @@ uint QColor::alloc()
 	    }
 	}
 
-    } while ( try_again && try_count < 100 );
+    } while ( try_again && try_count < 2 );
 
     if ( try_again ) {				// no hope of allocating color
 	rgbVal |= RGB_INVALID;
@@ -532,8 +454,7 @@ uint QColor::alloc()
     }
     // All colors outside context 0 must go into the dictionary
     bool many = colorDict->count() >= colorDict->size() * 8;
-    if ( many && colorDict->size() == col_std_dict )
-    {
+    if ( many && colorDict->size() == col_std_dict ) {
 	colorDict->resize( col_large_dict );
     }
     if ( !many || current_alloc_context != 0 ) {
@@ -547,24 +468,25 @@ uint QColor::alloc()
 }
 
 
-void QColor::setSystemNamedColor( const char *name )
+void QColor::setSystemNamedColor( const QString& name )
 {
     if ( !color_init ) {
 #if defined(CHECK_STATE)
-	warning( "QColor::setSystemNamedColor: Cannot perform this operation "
+	qWarning( "QColor::setSystemNamedColor: Cannot perform this operation "
 		 "because QApplication does not exist" );
 #endif
 	alloc();				// makes the color black
 	return;
     }
     XColor col, hw_col;
-    if ( XLookupColor(QPaintDevice::x__Display(),
-		      QPaintDevice::x11Colormap(), name, &col, &hw_col) ) {
+    if ( XLookupColor(QPaintDevice::x11AppDisplay(),
+		      QPaintDevice::x11AppColormap(), name.latin1(),
+		      &col, &hw_col) ) {
 	setRgb( col.red>>8, col.green>>8, col.blue>>8 );
 	return;					// success
     }
     // The name lookup failed if we got here
-    if ( lalloc ) {
+    if ( lazy_alloc ) {
 	rgbVal = RGB_INVALID | RGB_DIRTY;
 	pix = 0;
     } else {
@@ -599,9 +521,9 @@ static void init_context_stack()
 
   Example:
   \code
-    QPixmap loadPixmap( const char *fileName )
+    QPixmap loadPixmap( QString fileName )
     {
-        static int alloc_context = 0;
+	static int alloc_context = 0;
 	if ( alloc_context )
 	    QColor::destroyAllocContext( alloc_context );
 	alloc_context = QColor::enterAllocContext();
@@ -651,7 +573,7 @@ int QColor::enterAllocContext()
     init_context_stack();
     if ( context_ptr+1 == MAX_CONTEXTS ) {
 #if defined(CHECK_STATE)
-	warning( "QColor::enterAllocContext: Context stack overflow" );
+	qWarning( "QColor::enterAllocContext: Context stack overflow" );
 #endif
 	return 0;
     }
@@ -673,7 +595,7 @@ void QColor::leaveAllocContext()
     init_context_stack();
     if ( context_ptr == 0 ) {
 #if defined(CHECK_STATE)
-	warning( "QColor::leaveAllocContext: Context stack underflow" );
+	qWarning( "QColor::leaveAllocContext: Context stack underflow" );
 #endif
 	return;
     }
@@ -739,6 +661,7 @@ void QColor::destroyAllocContext( int context )
 	}
     }
     if ( i )
-	XFreeColors( QPaintDevice::x__Display(), QPaintDevice::x11Colormap(),
+	XFreeColors( QPaintDevice::x11AppDisplay(),
+		     QPaintDevice::x11AppColormap(),
 		     pixels, i, 0 );
 }
