@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qdrawutil.cpp,v 2.19.2.1 1998/11/02 13:01:29 hanord Exp $
+** $Id: qt/src/kernel/qdrawutil.cpp   2.1.1   edited 2000-03-24 $
 **
 ** Implementation of draw utilities
 **
 ** Created : 950920
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,43 +27,54 @@
 #include "qbitmap.h"
 #include "qpixmapcache.h"
 
+// REVISED: paul
+
 /*!
   \relates QPainter
 
-  Draws a horizontal (\e y1 == \e y2) or vertical (\e x1 == \e x2) shaded
-  line using the painter \e p.
+  <code>#include &lt;qdrawutil.h&gt;</code>
 
-  Nothing is drawn if \e y1 != y2 and \e x1 != x2 (i.e. the line is neither
-  horizontal nor vertical).
+  Draws a horizontal (\a y1 == \a y2) or vertical (\a x1 == \a x2) shaded
+  line using the painter \a p.
 
-  The color group argument \e g specifies the shading colors
+  Nothing is drawn if \a y1 != \a y2 and \a x1 != \a x2 (i.e. the line
+  is neither horizontal nor vertical).
+
+  The color group argument \a g specifies the shading colors
   (\link QColorGroup::light() light\endlink,
   \link QColorGroup::dark() dark\endlink and
   \link QColorGroup::mid() middle\endlink colors).
 
-  The line appears sunken if \e sunken is TRUE, or raised if \e sunken is
+  The line appears sunken if \a sunken is TRUE, or raised if \a sunken is
   FALSE.
 
-  The \e lineWidth argument specifies the line width for each of the
+  The \a lineWidth argument specifies the line width for each of the
   lines. It is not the total line width.
 
-  The \e midLineWidth argument specifies the width of a middle line drawn
+  The \a midLineWidth argument specifies the width of a middle line drawn
   in the QColorGroup::mid() color.
 
   If you want to use a QFrame widget instead, you can make it display a
   shaded line, for example
   <code>QFrame::setFrameStyle( QFrame::HLine | QFrame::Sunken )</code>.
 
-  \sa qDrawShadeRect(), qDrawShadePanel()
+  \warning This function does not look at QWidget::style()
+  or QApplication::style(). Use the drawing functions in QStyle to make
+  widgets that follow the current GUI style.
+
+  \sa qDrawShadeRect(), qDrawShadePanel(), QStyle::drawSeparator()
 */
 
 void qDrawShadeLine( QPainter *p, int x1, int y1, int x2, int y2,
 		     const QColorGroup &g, bool sunken,
 		     int lineWidth, int midLineWidth )
 {
+    if (!( p && lineWidth >= 0 && midLineWidth >= 0 ) )	{
 #if defined(CHECK_RANGE)
-    ASSERT( p && lineWidth >= 0 && midLineWidth >= 0 );
+	qWarning( "qDrawShadeLine invalid parameters." );
 #endif
+	return;
+    }
     int tlw = lineWidth*2 + midLineWidth;	// total line width
     QPen oldPen = p->pen();			// save pen
     if ( sunken )
@@ -138,30 +151,38 @@ void qDrawShadeLine( QPainter *p, int x1, int y1, int x2, int y2,
 /*!
   \relates QPainter
 
-  Draws a shaded rectangle/box given by \e (x,y,w,h) using the painter \e p.
+  <code>#include &lt;qdrawutil.h&gt;</code>
 
-  The color group argument \e g specifies the shading colors
+  Draws a shaded rectangle/box given by (\a x, \a y, \a w, \a h)
+  using the painter \a p.
+
+  The color group argument \a g specifies the shading colors
   (\link QColorGroup::light() light\endlink,
   \link QColorGroup::dark() dark\endlink and
   \link QColorGroup::mid() middle\endlink colors).
 
-  The rectangle appears sunken if \e sunken is TRUE, or raised if \e
+  The rectangle appears sunken if \a sunken is TRUE, or raised if \a
   sunken is FALSE.
 
-  The \e lineWidth argument specifies the line width for each of the
+  The \a lineWidth argument specifies the line width for each of the
   lines. It is not the total line width.
 
-  The \e midLineWidth argument specifies the width of a middle line drawn
+  The \a midLineWidth argument specifies the width of a middle line drawn
   in the QColorGroup::mid() color.
 
-  The rectangle interior is filled with the \e *fill brush unless \e fill
+  The rectangle interior is filled with the \a fill brush unless \a fill
   is null.
 
   If you want to use a QFrame widget instead, you can make it display a
   shaded rectangle, for example
   <code>QFrame::setFrameStyle( QFrame::Box | QFrame::Raised )</code>.
 
-  \sa qDrawShadeLine(), qDrawShadePanel(), qDrawPlainRect()
+  \warning This function does not look at QWidget::style()
+  or QApplication::style(). Use the drawing functions in QStyle to make
+  widgets that follow the current GUI style.
+
+  \sa qDrawShadeLine(), qDrawShadePanel(), qDrawPlainRect(),
+  QStyle::drawRect(), QStyle::drawRectStrong()
 */
 
 void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
@@ -171,9 +192,12 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
 {
     if ( w == 0 || h == 0 )
 	return;
+    if ( ! ( w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0 ) ) {
 #if defined(CHECK_RANGE)
-    ASSERT( w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0 );
+	qWarning( "qDrawShadeRect() Invalid parameters." );
 #endif
+	return;
+    }
     QPen oldPen = p->pen();			// save pen
     if ( sunken )
 	p->setPen( g.dark() );
@@ -228,7 +252,7 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
     if ( fill ) {
 	QBrush oldBrush = p->brush();
 	int tlw = lineWidth + midLineWidth;
-	p->setPen( NoPen );
+	p->setPen( Qt::NoPen );
 	p->setBrush( *fill );
 	p->drawRect( x+tlw, y+tlw, w-2*tlw, h-2*tlw );
 	p->setBrush( oldBrush );
@@ -240,26 +264,34 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
 /*!
   \relates QPainter
 
-  Draws a shaded panel given by \e (x,y,w,h) using the painter \e p.
+  <code>#include &lt;qdrawutil.h&gt;</code>
 
-  The color group argument \e g specifies the shading colors
+  Draws a shaded panel given by (\a x, \a y, \a w, \a h)
+  using the painter \a p.
+
+  The color group argument \a g specifies the shading colors
   (\link QColorGroup::light() light\endlink,
   \link QColorGroup::dark() dark\endlink and
   \link QColorGroup::mid() middle\endlink colors).
 
-  The panel appears sunken if \e sunken is TRUE, or raised if \e sunken is
+  The panel appears sunken if \a sunken is TRUE, or raised if \a sunken is
   FALSE.
 
-  The \e lineWidth argument specifies the line width.
+  The \a lineWidth argument specifies the line width.
 
-  The panel interior is filled with the \e *fill brush unless \e fill is
+  The panel interior is filled with the \a fill brush unless \a fill is
   null.
 
   If you want to use a QFrame widget instead, you can make it display a
   shaded panel, for example
   <code>QFrame::setFrameStyle( QFrame::Panel | QFrame::Sunken )</code>.
 
-  \sa qDrawWinPanel(), qDrawShadeLine(), qDrawShadeRect()
+  \warning This function does not look at QWidget::style()
+  or QApplication::style(). Use the drawing functions in QStyle to make
+  widgets that follow the current GUI style.
+
+
+  \sa qDrawWinPanel(), qDrawShadeLine(), qDrawShadeRect(), QStyle::drawPanel()
 */
 
 void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
@@ -268,9 +300,11 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
 {
     if ( w == 0 || h == 0 )
 	return;
+    if ( !( w > 0 && h > 0 && lineWidth >= 0 ) ) {
 #if defined(CHECK_RANGE)
-    ASSERT( w > 0 && h > 0 && lineWidth >= 0 );
+    	qWarning( "qDrawShadePanel() Invalid parameters." );
 #endif
+    }
     QPen oldPen = p->pen();			// save pen
     QPointArray a( 4*lineWidth );
     if ( sunken )
@@ -316,7 +350,7 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
     p->drawLineSegments( a );
     if ( fill ) {				// fill with fill color
 	QBrush oldBrush = p->brush();
-	p->setPen( NoPen );
+	p->setPen( Qt::NoPen );
 	p->setBrush( *fill );
 	p->drawRect( x+lineWidth, y+lineWidth, w-lineWidth*2, h-lineWidth*2 );
 	p->setBrush( oldBrush );
@@ -346,7 +380,7 @@ static void qDrawWinShades( QPainter *p,
 			   const QColor &c3, const QColor &c4,
 			   const QBrush *fill )
 {
-    if ( w < 2 || h < 2 )			// nothing to draw
+    if ( w < 2 || h < 2 )			// can't do anything with that
 	return;
     QPen oldPen = p->pen();
     QPointArray a( 3 );
@@ -374,7 +408,7 @@ static void qDrawWinShades( QPainter *p,
 	if ( fill ) {
 	    QBrush oldBrush = p->brush();
 	    p->setBrush( *fill );
-	    p->setPen( NoPen );
+	    p->setPen( Qt::NoPen );
 	    p->drawRect( x+2, y+2, w-4, h-4 );
 	    p->setBrush( oldBrush );
 	}
@@ -386,22 +420,29 @@ static void qDrawWinShades( QPainter *p,
 /*!
   \relates QPainter
 
-  Draws a Windows-style button given by \e (x,y,w,h) using the painter \e p.
+  <code>#include &lt;qdrawutil.h&gt;</code>
 
-  The color group argument \e g specifies the shading colors
+  Draws a Windows-style button given by (\a x, \a y, \a w, \a h)
+  using the painter \a p.
+
+  The color group argument \a g specifies the shading colors
   (\link QColorGroup::light() light\endlink,
   \link QColorGroup::dark() dark\endlink and
   \link QColorGroup::mid() middle\endlink colors).
 
-  The button appears sunken if \e sunken is TRUE, or raised if \e sunken
+  The button appears sunken if \a sunken is TRUE, or raised if \a sunken
   is FALSE.
 
   The line width is 2 pixels.
 
-  The button interior is filled with the \e *fill brush unless \e fill is
+  The button interior is filled with the \a *fill brush unless \a fill is
   null.
 
-  \sa qDrawWinPanel()
+  \warning This function does not look at QWidget::style()
+  or QApplication::style(). Use the drawing functions in QStyle to make
+  widgets that follow the current GUI style.
+
+  \sa qDrawWinPanel(), QStyle::drawButton()
 */
 
 void qDrawWinButton( QPainter *p, int x, int y, int w, int h,
@@ -410,32 +451,39 @@ void qDrawWinButton( QPainter *p, int x, int y, int w, int h,
 {
     if ( sunken )
 	qDrawWinShades( p, x, y, w, h,
-		       black, g.light(), g.dark(), g.background(), fill );
+		       g.shadow(), g.light(), g.dark(), g.button(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.light(), black, g.midlight(), g.dark(), fill );
+		       g.light(), g.shadow(), g.midlight(), g.dark(), fill );
 }
 
 /*!
   \relates QPainter
 
-  Draws a Windows-style panel given by \e (x,y,w,h) using the painter \e p.
+  <code>#include &lt;qdrawutil.h&gt;</code>
 
-  The color group argument \e g specifies the shading colors.
+  Draws a Windows-style panel given by (\a x, \a y, \a w, \a h)
+  using the painter \a p.
 
-  The panel appears sunken if \e sunken is TRUE, or raised if \e sunken is
+  The color group argument \a g specifies the shading colors.
+
+  The panel appears sunken if \a sunken is TRUE, or raised if \a sunken is
   FALSE.
 
   The line width is 2 pixels.
 
-  The button interior is filled with the \e *fill brush unless \e fill is
+  The button interior is filled with the \a fill brush unless \a fill is
   null.
 
   If you want to use a QFrame widget instead, you can make it display a
   shaded panel, for example
   <code>QFrame::setFrameStyle( QFrame::WinPanel | QFrame::Raised )</code>.
 
-  \sa qDrawShadePanel(), qDrawWinButton()
+  \warning This function does not look at QWidget::style()
+  or QApplication::style(). Use the drawing functions in QStyle to make
+  widgets that follow the current GUI style.
+
+  \sa qDrawShadePanel(), qDrawWinButton(), QStyle::drawPanel()
 */
 
 void qDrawWinPanel( QPainter *p, int x, int y, int w, int h,
@@ -444,30 +492,37 @@ void qDrawWinPanel( QPainter *p, int x, int y, int w, int h,
 {
     if ( sunken )
 	qDrawWinShades( p, x, y, w, h,
-		       g.dark(), g.light(), black, g.midlight(), fill );
+			g.dark(), g.light(), g.shadow(), g.midlight(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.midlight(), black, g.light(), g.dark(), fill );
+			g.midlight(), g.shadow(), g.light(), g.dark(), fill );
 }
 
 
 /*!
   \relates QPainter
 
-  Draws a plain rectangle given by \e (x,y,w,h) using the painter \e p.
+  <code>#include &lt;qdrawutil.h&gt;</code>
 
-  The color argument \e c specifies the line color.
+  Draws a plain rectangle given by (\a x, \a y, \a w, \a h)
+  using the painter \a p.
 
-  The \e lineWidth argument specifies the line width.
+  The color argument \a c specifies the line color.
 
-  The rectangle interior is filled with the \e *fill brush unless \e fill
+  The \a lineWidth argument specifies the line width.
+
+  The rectangle interior is filled with the \a fill brush unless \a fill
   is null.
 
   If you want to use a QFrame widget instead, you can make it display a
-  shaded rectangle, for example
+  plain rectangle, for example
   <code>QFrame::setFrameStyle( QFrame::Box | QFrame::Plain )</code>.
 
-  \sa qDrawShadeRect()
+  \warning This function does not look at QWidget::style()
+  or QApplication::style(). Use the drawing functions in QStyle to make
+  widgets that follow the current GUI style.
+
+  \sa qDrawShadeRect(), QStyle::drawRect()
 */
 
 void qDrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
@@ -475,17 +530,19 @@ void qDrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
 {
     if ( w == 0 || h == 0 )
 	return;
+    if ( !( w > 0 && h > 0 && lineWidth >= 0 ) ) {
 #if defined(CHECK_RANGE)
-    ASSERT( w > 0 && h > 0 && lineWidth >= 0 );
+	qWarning( "qDrawPlainRect() Invalid parameters." );
 #endif
+    }
     QPen   oldPen   = p->pen();
     QBrush oldBrush = p->brush();
     p->setPen( c );
-    p->setBrush( NoBrush );
+    p->setBrush( Qt::NoBrush );
     for ( int i=0; i<lineWidth; i++ )
 	p->drawRect( x+i, y+i, w-i*2, h-i*2 );
     if ( fill ) {				// fill with fill color
-	p->setPen( NoPen );
+	p->setPen( Qt::NoPen );
 	p->setBrush( *fill );
 	p->drawRect( x+lineWidth, y+lineWidth, w-lineWidth*2, h-lineWidth*2 );
     }
@@ -494,28 +551,28 @@ void qDrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
 }
 
 
-QRect qItemRect( QPainter *p, GUIStyle gs,
+QRect qItemRect( QPainter *p, Qt::GUIStyle gs,
 		int x, int y, int w, int h,
-		int flags, 
+		int flags,
 		bool enabled,
 		const QPixmap *pixmap,
-		const char *text, int len )
+		const QString& text, int len )
 {
     QRect result;
 
     if ( pixmap ) {
-	if ( (flags & AlignVCenter) == AlignVCenter )
+	if ( (flags & Qt::AlignVCenter) == Qt::AlignVCenter )
 	    y += h/2 - pixmap->height()/2;
-	else if ( (flags & AlignBottom) == AlignBottom)
+	else if ( (flags & Qt::AlignBottom) == Qt::AlignBottom)
 	    y += h - pixmap->height();
-	if ( (flags & AlignRight) == AlignRight )
+	if ( (flags & Qt::AlignRight) == Qt::AlignRight )
 	    x += w - pixmap->width();
-	else if ( (flags & AlignHCenter) == AlignHCenter )
+	else if ( (flags & Qt::AlignHCenter) == Qt::AlignHCenter )
 	    x += w/2 - pixmap->width()/2;
 	result = QRect(x, y, pixmap->width(), pixmap->height());
-    } else if ( text && p ) {
+    } else if ( !text.isNull() && p ) {
 	result = p->boundingRect( x, y, w, h, flags, text, len );
-	if ( gs == WindowsStyle && !enabled ) {
+	if ( gs == Qt::WindowsStyle && !enabled ) {
 	    result.setWidth(result.width()+1);
 	    result.setHeight(result.height()+1);
 	}
@@ -527,30 +584,30 @@ QRect qItemRect( QPainter *p, GUIStyle gs,
 }
 
 
-void qDrawItem( QPainter *p, GUIStyle gs,
+void qDrawItem( QPainter *p, Qt::GUIStyle gs,
 		int x, int y, int w, int h,
-		int flags, 
+		int flags,
 		const QColorGroup &g, bool enabled,
 		const QPixmap *pixmap,
-		const char *text, int len )
+		const QString& text, int len , const QColor* penColor )
 {
-    p->setPen( g.text() );
+    p->setPen( penColor?*penColor:g.foreground() );
     if ( pixmap ) {
 	QPixmap  pm( *pixmap );
-	bool clip = (flags & DontClip) == 0;
+	bool clip = (flags & Qt::DontClip) == 0;
 	if ( clip ) {
 	    if ( pm.width() < w && pm.height() < h )
 		clip = FALSE;
 	    else
 		p->setClipRect( x, y, w, h );
 	}
-	if ( (flags & AlignVCenter) == AlignVCenter )
+	if ( (flags & Qt::AlignVCenter) == Qt::AlignVCenter )
 	    y += h/2 - pm.height()/2;
-	else if ( (flags & AlignBottom) == AlignBottom)
+	else if ( (flags & Qt::AlignBottom) == Qt::AlignBottom)
 	    y += h - pm.height();
-	if ( (flags & AlignRight) == AlignRight )
+	if ( (flags & Qt::AlignRight) == Qt::AlignRight )
 	    x += w - pm.width();
-	else if ( (flags & AlignHCenter) == AlignHCenter )
+	else if ( (flags & Qt::AlignHCenter) == Qt::AlignHCenter )
 	    x += w/2 - pm.width()/2;
 	if ( !enabled ) {
 	    if ( pm.mask() ) {			// pixmap with a mask
@@ -574,7 +631,7 @@ void qDrawItem( QPainter *p, GUIStyle gs,
 		pm = *mask;
 		if (del) delete mask;
 	    }
-	    if ( gs == WindowsStyle ) {
+	    if ( gs == Qt::WindowsStyle ) {
 		p->setPen( g.light() );
 		p->drawPixmap( x+1, y+1, pm );
 		p->setPen( g.text() );
@@ -583,8 +640,8 @@ void qDrawItem( QPainter *p, GUIStyle gs,
 	p->drawPixmap( x, y, pm );
 	if ( clip )
 	    p->setClipping( FALSE );
-    } else if ( text ) {
-	if ( gs == WindowsStyle && !enabled ) {
+    } else if ( !text.isNull() ) {
+	if ( gs == Qt::WindowsStyle && !enabled ) {
 	    p->setPen( g.light() );
 	    p->drawText( x+1, y+1, w, h, flags, text, len );
 	    p->setPen( g.text() );
@@ -669,22 +726,22 @@ void qDrawPlainRect( QPainter *p, const QRect &r, const QColor &c,
 }
 
 
-static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
+static void qDrawWinArrow( QPainter *p, Qt::ArrowType type, bool down,
 			   int x, int y, int w, int h,
 			   const QColorGroup &g, bool enabled )
 {
     QPointArray a;				// arrow polygon
     switch ( type ) {
-    case UpArrow:
+    case Qt::UpArrow:
 	a.setPoints( 7, -3,1, 3,1, -2,0, 2,0, -1,-1, 1,-1, 0,-2 );
 	break;
-    case DownArrow:
+    case Qt::DownArrow:
 	a.setPoints( 7, -3,-1, 3,-1, -2,0, 2,0, -1,1, 1,1, 0,2 );
 	break;
-    case LeftArrow:
+    case Qt::LeftArrow:
 	a.setPoints( 7, 1,-3, 1,3, 0,-2, 0,2, -1,-1, -1,1, -2,0 );
 	break;
-    case RightArrow:
+    case Qt::RightArrow:
 	a.setPoints( 7, -1,-3, -1,3, 0,-2, 0,2, 1,-1, 1,1, 2,0 );
 	break;
     }
@@ -697,7 +754,11 @@ static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
     }
 
     QPen savePen = p->pen();			// save current pen
-    p->fillRect( x, y, w, h, g.background() );
+    if (down)
+	p->setBrushOrigin(p->brushOrigin() + QPoint(1,1));
+    p->fillRect( x, y, w, h, g.brush( QColorGroup::Button ) );
+    if (down)
+	p->setBrushOrigin(p->brushOrigin() - QPoint(1,1));
     if ( enabled ) {
 	a.translate( x+w/2, y+h/2 );
 	p->setPen( g.foreground() );
@@ -724,16 +785,16 @@ static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
 
 // motif arrows look the same whether they are used or not
 // is this correct?
-static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
+static void qDrawMotifArrow( QPainter *p, Qt::ArrowType type, bool down,
 			     int x, int y, int w, int h,
-			     const QColorGroup &g, bool ) 
+			     const QColorGroup &g, bool )
 {
     QPointArray bFill;				// fill polygon
     QPointArray bTop;				// top shadow.
     QPointArray bBot;				// bottom shadow.
     QPointArray bLeft;				// left shadow.
     QWMatrix	matrix;				// xform matrix
-    bool vertical = type == UpArrow || type == DownArrow;
+    bool vertical = type == Qt::UpArrow || type == Qt::DownArrow;
     bool horizontal = !vertical;
     int	 dim = w < h ? w : h;
     int	 colspec = 0x0000;			// color specification array
@@ -780,7 +841,7 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 	}
     }
 
-    if ( type == UpArrow || type == LeftArrow ) {
+    if ( type == Qt::UpArrow || type == Qt::LeftArrow ) {
 	matrix.translate( x, y );
 	if ( vertical ) {
 	    matrix.translate( 0, h - 1 );
@@ -794,7 +855,7 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 	else
 	    colspec = horizontal ? 0x1443 : 0x1434;
     }
-    else if ( type == DownArrow || type == RightArrow ) {
+    else if ( type == Qt::DownArrow || type == Qt::RightArrow ) {
 	matrix.translate( x, y );
 	if ( vertical ) {
 	    matrix.translate( w-1, 0 );
@@ -808,7 +869,7 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 
     QColor *cols[5];
     cols[0] = 0;
-    cols[1] = (QColor *)&g.background();
+    cols[1] = (QColor *)&g.button();
     cols[2] = (QColor *)&g.mid();
     cols[3] = (QColor *)&g.light();
     cols[4] = (QColor *)&g.dark();
@@ -820,14 +881,14 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
     QPen     savePen   = p->pen();		// save current pen
     QBrush   saveBrush = p->brush();		// save current brush
     QWMatrix wxm = p->worldMatrix();
-    QPen     pen( NoPen );
-    QBrush   brush( CMID );
+    QPen     pen( Qt::NoPen );
+    QBrush brush = g.brush( QColorGroup::Button );
 
     p->setPen( pen );
     p->setBrush( brush );
     p->setWorldMatrix( matrix, TRUE );		// set transformation matrix
     p->drawPolygon( bFill );			// fill arrow
-    p->setBrush( NoBrush );			// don't fill
+    p->setBrush( Qt::NoBrush );			// don't fill
 
     p->setPen( CLEFT );
     p->drawLineSegments( bLeft );
@@ -847,20 +908,22 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 }
 
 
-void qDrawArrow( QPainter *p, ArrowType type, GUIStyle style, bool down,
+void qDrawArrow( QPainter *p, Qt::ArrowType type, Qt::GUIStyle style, bool down,
 		 int x, int y, int w, int h,
 		 const QColorGroup &g, bool enabled )
 {
     switch ( style ) {
-	case WindowsStyle:
+	case Qt::WindowsStyle:
 	    qDrawWinArrow( p, type, down, x, y, w, h, g, enabled );
 	    break;
-	case MotifStyle:
+	case Qt::MotifStyle:
 	    qDrawMotifArrow( p, type, down, x, y, w, h, g, enabled );
 	    break;
-#if defined(CHECK_RANGE)
 	default:
-	    warning( "qDrawArrow: Requested GUI style not supported" );
+#if defined(CHECK_RANGE)
+	    qWarning( "qDrawArrow: Requested GUI style not supported" );
+#else
+	    ;
 #endif
     }
 }
