@@ -1,26 +1,30 @@
 /****************************************************************************
-** $Id: qmovie.cpp,v 1.39.2.3 1998/11/13 09:37:03 ettrich Exp $
+** $Id: qt/src/kernel/qmovie.cpp   2.1.1   edited 2000-04-07 $
 **
 ** Implementation of movie classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
+// #define QT_SAVE_MOVIE_HACK
+
 #include "qtimer.h"
 #include "qpainter.h"
 #include "qlist.h"
@@ -35,11 +39,13 @@
 
 #include <stdlib.h>
 
+// NOT REVISED
 /*!
   \class QMovie qmovie.h
   \brief Incrementally loads an animation or image, signalling as it progresses.
 
   \ingroup images
+  \ingroup drawing
 
   A QMovie provides a QPixmap as the framePixmap(), and connections can
   be made via connectResize() and connectUpdate() to receive notification
@@ -68,8 +74,7 @@
   extension is obeyed.
 
   Archives of animated GIFs and tools for building them can be found
-  at <a href=http://www.yahoo.com/Arts/Visual_Arts/Animation/Computer_Animation/Animated_GIFs/>
-  Yahoo!</a>.
+  at <a href="http://dir.yahoo.com/Arts/Visual_Arts/Animation/Computer_Animation/Animated_GIFs/">Yahoo</a>.
 
   We are required to state: The Graphics Interchange Format(c) is the
   Copyright property of CompuServe Incorporated. GIF(sm) is a Service
@@ -77,33 +82,32 @@
 
   \warning Unisys has changed its position regarding GIF.  If you are
   in a country where Unisys holds a patent on LZW compression and/or
-  decompression, Unisys may require a license from you.  These
-  countries include Canada, Japan, the USA, France, Germany, Italy and
-  the UK.  There is more information on Unisys web site: <a
-  href="http://corp2.unisys.com/LeadStory/lzwfaq.html">Overview of
-  Unisys' position.</a> GIF support may be removed in a future version
-  of Qt.  We recommend using the PNG format, which is available in the
-  <a href="imageio.html">Qt Image IO Extension</a> package.
+  decompression and you want to use GIF, Unisys may require you to
+  license that technology.  These countries include Canada, Japan, the
+  USA, France, Germany, Italy and the UK.
 
-  <img src="qmovie.gif">
+  GIF support may be removed completely in a future version of Qt.  We
+  recommend using the PNG format.
+
+  <img src="qmovie.png">
 
   \sa QLabel::setMovie()
 */
 
-class QMoviePrivate : public QObject, public QShared,
+class QMovieFilePrivate : public QObject, public QShared,
 		      private QDataSink, private QImageConsumer
 {
     Q_OBJECT
 
 public: // for QMovie
 
-    // Creates a null QMoviePrivate
-    QMoviePrivate();
+    // Creates a null QMovieFilePrivate
+    QMovieFilePrivate();
 
-    // NOTE:  The ownership of the QDataSource is transferred to the QMoviePrivate
-    QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize);
+    // NOTE:  The ownership of the QDataSource is transferred to the QMovieFilePrivate
+    QMovieFilePrivate(QDataSource* src, QMovie* movie, int bufsize);
 
-    virtual ~QMoviePrivate();
+    virtual ~QMovieFilePrivate();
 
     bool isNull() const;
 
@@ -116,7 +120,9 @@ public: // for QMovie
     // This as QImageConsumer
     void changed(const QRect& rect);
     void end();
+    void preFrameDone(); //util func
     void frameDone();
+    void frameDone(const QPoint&, const QRect& rect);
     void restartTimer();
     void setLooping(int l);
     void setFramePeriod(int milliseconds);
@@ -165,9 +171,15 @@ public:
 
     int error;
     bool empty;
+
+#ifdef QT_SAVE_MOVIE_HACK
+  bool save_image;
+  int image_number;
+#endif
 };
 
-QMoviePrivate::QMoviePrivate()
+
+QMovieFilePrivate::QMovieFilePrivate()
 {
     buffer = 0;
     pump = 0;
@@ -176,8 +188,8 @@ QMoviePrivate::QMoviePrivate()
     init(FALSE);
 }
 
-// NOTE:  The ownership of the QDataSource is transferred to the QMoviePrivate
-QMoviePrivate::QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize) :
+// NOTE:  The ownership of the QDataSource is transferred to the QMovieFilePrivate
+QMovieFilePrivate::QMovieFilePrivate(QDataSource* src, QMovie* movie, int bufsize) :
     that(movie),
     buf_size(bufsize)
 {
@@ -191,7 +203,7 @@ QMoviePrivate::QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize) :
     init(TRUE);
 }
 
-QMoviePrivate::~QMoviePrivate()
+QMovieFilePrivate::~QMovieFilePrivate()
 {
     if ( buffer )				// Avoid purify complaint
 	delete [] buffer;
@@ -200,15 +212,20 @@ QMoviePrivate::~QMoviePrivate()
     delete source;
 }
 
-bool QMoviePrivate::isNull() const
+bool QMovieFilePrivate::isNull() const
 {
     return !pump;
 }
 
 // Initialize.  Only actually allocate any space if \a fully is TRUE,
-// otherwise, just enough to be a valid null QMoviePrivate.
-void QMoviePrivate::init(bool fully)
+// otherwise, just enough to be a valid null QMovieFilePrivate.
+void QMovieFilePrivate::init(bool fully)
 {
+#ifdef QT_SAVE_MOVIE_HACK
+    save_image = TRUE;
+    image_number = 0;
+#endif
+
     buf_usage = buf_r = buf_w = 0;
     if ( buffer )				// Avoid purify complaint
 	delete [] buffer;
@@ -235,7 +252,7 @@ void QMoviePrivate::init(bool fully)
     empty = TRUE;
 }
 
-void QMoviePrivate::flushBuffer()
+void QMovieFilePrivate::flushBuffer()
 {
     while (buf_usage && !waitingForFrameTick && stepping != 0 && !error) {
 	int used = decoder->decode(buffer + buf_r,
@@ -252,28 +269,21 @@ void QMoviePrivate::flushBuffer()
     maybeReady();
 }
 
-void QMoviePrivate::updatePixmapFromImage()
+void QMovieFilePrivate::updatePixmapFromImage()
 {
     if (changed_area.isEmpty()) return;
 
     // Create temporary QImage to hold the part we want
     const QImage& gimg = decoder->image();
-    QImage img(changed_area.size(), 8, gimg.numColors());
-    img.setAlphaBuffer(gimg.hasAlphaBuffer());
+    QImage img = gimg.copy(changed_area);
 
-    // Copy color map.
-    memcpy(img.colorTable(), gimg.colorTable(),
-	sizeof(QRgb)*gimg.numColors());
-
-    // Copy pixels.
-    const int t = changed_area.top();
-    const int b = changed_area.bottom();
-    const int l = changed_area.left();
-    const int w = changed_area.width();
-    const int h = changed_area.height();
-    for (int y=t; y<=b; y++) {
-	memcpy(img.scanLine(y-t), gimg.scanLine(y)+l, w);
+#ifdef QT_SAVE_MOVIE_HACK
+    if ( save_image ) {
+	QString name;
+	name.sprintf("movie%i.ppm",image_number++);
+	gimg.save( name, "PPM" );
     }
+#endif
 
     // Resize to size of image
     if (mypixmap.width() != gimg.width() || mypixmap.height() != gimg.height())
@@ -282,14 +292,14 @@ void QMoviePrivate::updatePixmapFromImage()
     if (bg.isValid()) {
 	QPainter p;
 	p.begin(&mypixmap);
-	p.fillRect(l, t, w, h, bg);
+	p.fillRect(changed_area, bg);
 	p.end();
     } else {
 	if (gimg.hasAlphaBuffer()) {
 	    // Resize to size of image
 	    if (mymask.isNull()) {
 		mymask.resize(gimg.width(), gimg.height());
-		mymask.fill(color1);
+		mymask.fill( Qt::color1 );
 	    }
 	}
 	mypixmap.setMask(QBitmap()); // Remove reference to my mask
@@ -298,15 +308,19 @@ void QMoviePrivate::updatePixmapFromImage()
     // Convert to pixmap and paste that onto myself
     QPixmap lines;
     lines.convertFromImage(img);
-    bitBlt(&mypixmap, l, t, &lines, 0, 0, w, h, CopyROP, !bg.isValid());
+    bitBlt(&mypixmap, changed_area.left(), changed_area.top(),
+	&lines, 0, 0, changed_area.width(), changed_area.height(),
+	CopyROP, !bg.isValid());
 
     if (!bg.isValid() && gimg.hasAlphaBuffer()) {
-	bitBlt(&mymask, l, t, lines.mask(), 0, 0, w, h, CopyROP, TRUE);
+	bitBlt(&mymask, changed_area.left(), changed_area.top(),
+	    lines.mask(), 0, 0, changed_area.width(), changed_area.height(),
+	    CopyROP, TRUE);
 	mypixmap.setMask(mymask);
     }
 }
 
-void QMoviePrivate::showChanges()
+void QMovieFilePrivate::showChanges()
 {
     if (changed_area.isValid()) {
 	updatePixmapFromImage();
@@ -318,18 +332,19 @@ void QMoviePrivate::showChanges()
     }
 }
 
-// QMoviePrivate as QImageConsumer
-void QMoviePrivate::changed(const QRect& rect)
+// QMovieFilePrivate as QImageConsumer
+void QMovieFilePrivate::changed(const QRect& rect)
 {
-    if (!frametimer->isActive()) frametimer->start(0);
+    if (!frametimer->isActive())
+	frametimer->start(0);
     changed_area = changed_area.unite(rect);
 }
 
-void QMoviePrivate::end()
+void QMovieFilePrivate::end()
 {
 }
 
-void QMoviePrivate::frameDone()
+void QMovieFilePrivate::preFrameDone()
 {
     if (stepping > 0) {
 	stepping--;
@@ -341,12 +356,32 @@ void QMoviePrivate::frameDone()
 	waitingForFrameTick = TRUE;
 	restartTimer();
     }
+}
+void QMovieFilePrivate::frameDone()
+{
+    preFrameDone();
     showChanges();
     emit dataStatus(QMovie::EndOfFrame);
     framenumber++;
 }
+void QMovieFilePrivate::frameDone(const QPoint& p,
+				const QRect& rect)
+{
+    preFrameDone();
+    const QImage& gimg = decoder->image();
+    if (framenumber==0) {
+	mypixmap.resize(gimg.width(), gimg.height());
+	emit sizeChanged(gimg.size());
+    }
+    bitBlt(&mypixmap, p.x(), p.y(), &gimg, rect.x(), rect.y(),
+		    rect.width(), rect.height());
+    // TODO: transparency
+    emit areaChanged(QRect(p,rect.size()));
+    emit dataStatus(QMovie::EndOfFrame);
+    framenumber++;
+}
 
-void QMoviePrivate::restartTimer()
+void QMovieFilePrivate::restartTimer()
 {
     if (speed > 0) {
 	int i = frameperiod >= 0 ? frameperiod * 100/speed : 0;
@@ -359,7 +394,7 @@ void QMoviePrivate::restartTimer()
     }
 }
 
-void QMoviePrivate::setLooping(int nloops)
+void QMovieFilePrivate::setLooping(int nloops)
 {
     if (loop == -1) { // Only if we don't already know how many loops!
 	if (source->rewindable()) {
@@ -372,14 +407,14 @@ void QMoviePrivate::setLooping(int nloops)
     }
 }
 
-void QMoviePrivate::setFramePeriod(int milliseconds)
+void QMovieFilePrivate::setFramePeriod(int milliseconds)
 {
     // Animation:  only show complete frame
     frameperiod = milliseconds;
     if (stepping<0 && frameperiod >= 0) restartTimer();
 }
 
-void QMoviePrivate::setSize(int w, int h)
+void QMovieFilePrivate::setSize(int w, int h)
 {
     if (mypixmap.width() != w || mypixmap.height() != h) {
 	mypixmap.resize(w, h);
@@ -388,9 +423,9 @@ void QMoviePrivate::setSize(int w, int h)
 }
 
 
-// QMoviePrivate as QDataSink
+// QMovieFilePrivate as QDataSink
 
-int QMoviePrivate::readyToReceive()
+int QMovieFilePrivate::readyToReceive()
 {
     // Could pre-fill buffer, but more efficient to just leave the
     // data back at the source.
@@ -398,7 +433,7 @@ int QMoviePrivate::readyToReceive()
 	? 0 : buf_size;
 }
 
-void QMoviePrivate::receive(const uchar* b, int bytecount)
+void QMovieFilePrivate::receive(const uchar* b, int bytecount)
 {
     if ( bytecount ) empty = FALSE;
 
@@ -421,11 +456,15 @@ void QMoviePrivate::receive(const uchar* b, int bytecount)
     }
 }
 
-void QMoviePrivate::eof()
+void QMovieFilePrivate::eof()
 {
     if ( empty )
 	emit dataStatus(QMovie::SourceEmpty);
 
+#ifdef QT_SAVE_MOVIE_HACK
+    save_image = FALSE;
+#endif
+
     emit dataStatus(QMovie::EndOfLoop);
 
     if (loop >= 0) {
@@ -451,7 +490,7 @@ void QMoviePrivate::eof()
     }
 }
 
-void QMoviePrivate::pause()
+void QMovieFilePrivate::pause()
 {
     if ( stepping ) {
 	stepping = 0;
@@ -460,7 +499,7 @@ void QMoviePrivate::pause()
     }
 }
 
-void QMoviePrivate::refresh()
+void QMovieFilePrivate::refresh()
 {
     if (!decoder) {
 	frametimer->stop();
@@ -480,25 +519,25 @@ void QMoviePrivate::refresh()
     flushBuffer();
 }
 
-///////////////// End of QMoviePrivate /////////////////
+///////////////// End of QMovieFilePrivate /////////////////
 
 
 
 
 
 /*!
-  Creates a null QMovie.  The only interesting thing to do to such
+  Constructs a null QMovie.  The only interesting thing to do to such
   a movie is to assign another movie to it.
 
   \sa isNull()
 */
 QMovie::QMovie()
 {
-    d = new QMoviePrivate();
+    d = new QMovieFilePrivate();
 }
 
 /*!
-  Creates a QMovie which reads an image sequence from the given
+  Constructs a QMovie which reads an image sequence from the given
   QDataSource.  The source must be allocated dynamically,
   as it becomes owned by the QMovie, and will be destroyed
   when the movie is destroyed.
@@ -511,29 +550,27 @@ QMovie::QMovie()
 */
 QMovie::QMovie(QDataSource* src, int bufsize)
 {
-    d = new QMoviePrivate(src, this, bufsize);
+    d = new QMovieFilePrivate(src, this, bufsize);
 }
 
 /*!
-  \overload
-  Creates a QMovie which reads an image sequence from the named file.
+  Constructs a QMovie which reads an image sequence from the named file.
 */
-QMovie::QMovie(const char* fileName, int bufsize)
+QMovie::QMovie(const QString &fileName, int bufsize)
 {
     QFile* file = new QFile(fileName);
     file->open(IO_ReadOnly);
-    d = new QMoviePrivate(new QIODeviceSource(file), this, bufsize);
+    d = new QMovieFilePrivate(new QIODeviceSource(file), this, bufsize);
 }
 
 /*!
-  \overload
-  Creates a QMovie which reads an image sequence from given data.
+  Constructs a QMovie which reads an image sequence from given data.
 */
 QMovie::QMovie(QByteArray data, int bufsize)
 {
     QBuffer* buffer = new QBuffer(data);
     buffer->open(IO_ReadOnly);
-    d = new QMoviePrivate(new QIODeviceSource(buffer), this, bufsize);
+    d = new QMovieFilePrivate(new QIODeviceSource(buffer), this, bufsize);
 }
 
 /*!
@@ -641,7 +678,7 @@ int QMovie::frameNumber() const
 */
 bool QMovie::paused() const
 {
-    return d->stepping==0;
+    return d->stepping == 0;
 }
 
 /*!
@@ -679,8 +716,9 @@ void QMovie::pause()
 */
 void QMovie::unpause()
 {
-    if ( d->stepping >= 0 ) {
-	if (d->isNull()) return;
+    if ( d->stepping >= 0 )	{
+	if (d->isNull())
+	    return;
 	d->stepping = -1;
 	d->restartTimer();
     }
@@ -691,8 +729,8 @@ void QMovie::unpause()
 */
 void QMovie::step(int steps)
 {
-    if (d->isNull()) return;
-
+    if (d->isNull())
+	return;
     d->stepping = steps;
     d->frametimer->start(0);
     d->waitingForFrameTick = FALSE; // Full speed ahead!
@@ -712,8 +750,8 @@ void QMovie::step()
 */
 void QMovie::restart()
 {
-    if (d->isNull()) return;
-
+    if (d->isNull())
+	return;
     if (d->source->rewindable()) {
 	d->source->enableRewind(TRUE);
 	d->source->rewind();
@@ -740,22 +778,19 @@ int QMovie::speed() const
 void QMovie::setSpeed(int percent)
 {
     int oldspeed = d->speed;
-
     if ( oldspeed != percent && percent >= 0 ) {
 	d->speed = percent;
-
 	// Restart timer only if really needed
 	if (d->stepping < 0) {
 	    if ( !percent || !oldspeed    // To or from zero
-	    || oldspeed*4 / percent > 4   // More than 20% slower
-	    || percent*4 / oldspeed > 4   // More than 20% faster
-	    )
+		 || oldspeed*4 / percent > 4   // More than 20% slower
+		 || percent*4 / oldspeed > 4   // More than 20% faster
+		 )
 		d->restartTimer();
 	}
     }
 }
 
-
 /*!
   Connects the given member, of type \code void member(const QSize&) \endcode
   such that it is signalled when the movie changes size.
@@ -764,7 +799,7 @@ void QMovie::setSpeed(int percent)
   persist until they are explicitly disconnected with disconnectResize(), or
   until \e every shared copy of the movie is deleted.
 */
-void QMovie::connectResize(QObject* receiver, const char* member)
+void QMovie::connectResize(QObject* receiver, const char *member)
 {
     QObject::connect(d, SIGNAL(sizeChanged(const QSize&)), receiver, member);
 }
@@ -774,7 +809,7 @@ void QMovie::connectResize(QObject* receiver, const char* member)
   previously connected by
   connectResize().
 */
-void QMovie::disconnectResize(QObject* receiver, const char* member)
+void QMovie::disconnectResize(QObject* receiver, const char *member)
 {
     QObject::disconnect(d, SIGNAL(sizeChanged(const QSize&)), receiver, member);
 }
@@ -788,7 +823,7 @@ void QMovie::disconnectResize(QObject* receiver, const char* member)
   persist until they are explicitly disconnected with disconnectUpdate(), or
   until \e every shared copy of the movie is deleted.
 */
-void QMovie::connectUpdate(QObject* receiver, const char* member)
+void QMovie::connectUpdate(QObject* receiver, const char *member)
 {
     QObject::connect(d, SIGNAL(areaChanged(const QRect&)), receiver, member);
 }
@@ -798,7 +833,7 @@ void QMovie::connectUpdate(QObject* receiver, const char* member)
   previously connected by
   connectUpdate().
 */
-void QMovie::disconnectUpdate(QObject* receiver, const char* member)
+void QMovie::disconnectUpdate(QObject* receiver, const char *member)
 {
     QObject::disconnect(d, SIGNAL(areaChanged(const QRect&)), receiver, member);
 }
@@ -828,7 +863,7 @@ void QMovie::disconnectUpdate(QObject* receiver, const char* member)
   persist until they are explicitly disconnected with disconnectStatus(), or
   until \e every shared copy of the movie is deleted.
 */
-void QMovie::connectStatus(QObject* receiver, const char* member)
+void QMovie::connectStatus(QObject* receiver, const char *member)
 {
     QObject::connect(d, SIGNAL(dataStatus(int)), receiver, member);
 }
@@ -838,153 +873,10 @@ void QMovie::connectStatus(QObject* receiver, const char* member)
   previously connected by
   connectStatus().
 */
-void QMovie::disconnectStatus(QObject* receiver, const char* member)
+void QMovie::disconnectStatus(QObject* receiver, const char *member)
 {
     QObject::disconnect(d, SIGNAL(dataStatus(int)), receiver, member);
 }
 
 
-/* tmake ignore Q_OBJECT */
-
-//       MANUALLY INCLUDED.  Regenerate in vi with:   !Gmoc %
-//       WARNING! All changes made below will be lost
-//
-/****************************************************************************
-** QMoviePrivate meta object code from reading C++ file 'qmovie.cpp'
-**
-** Created: Mon Aug 31 17:13:23 1998
-**      by: The Qt Meta Object Compiler ($Revision: 1.39.2.3 $)
-**
-** WARNING! All changes made in this file will be lost!
-*****************************************************************************/
-
-#if !defined(Q_MOC_OUTPUT_REVISION)
-#define Q_MOC_OUTPUT_REVISION 2
-#elif Q_MOC_OUTPUT_REVISION != 2
-#error "Moc format conflict - please regenerate all moc files"
-#endif
-
-#include <qmetaobject.h>
-
-
-const char *QMoviePrivate::className() const
-{
-    return "QMoviePrivate";
-}
-
-QMetaObject *QMoviePrivate::metaObj = 0;
-
-void QMoviePrivate::initMetaObject()
-{
-    if ( metaObj )
-	return;
-    if ( strcmp(QObject::className(), "QObject") != 0 )
-	badSuperclassWarning("QMoviePrivate","QObject");
-    if ( !QObject::metaObject() )
-	QObject::initMetaObject();
-    typedef void(QMoviePrivate::*m1_t0)();
-    m1_t0 v1_0 = &QMoviePrivate::refresh;
-    QMetaData *slot_tbl = new QMetaData[1];
-    slot_tbl[0].name = "refresh()";
-    slot_tbl[0].ptr = *((QMember*)&v1_0);
-    typedef void(QMoviePrivate::*m2_t0)(const QSize&);
-    typedef void(QMoviePrivate::*m2_t1)(const QRect&);
-    typedef void(QMoviePrivate::*m2_t2)(int);
-    m2_t0 v2_0 = &QMoviePrivate::sizeChanged;
-    m2_t1 v2_1 = &QMoviePrivate::areaChanged;
-    m2_t2 v2_2 = &QMoviePrivate::dataStatus;
-    QMetaData *signal_tbl = new QMetaData[3];
-    signal_tbl[0].name = "sizeChanged(const QSize&)";
-    signal_tbl[1].name = "areaChanged(const QRect&)";
-    signal_tbl[2].name = "dataStatus(int)";
-    signal_tbl[0].ptr = *((QMember*)&v2_0);
-    signal_tbl[1].ptr = *((QMember*)&v2_1);
-    signal_tbl[2].ptr = *((QMember*)&v2_2);
-    metaObj = new QMetaObject( "QMoviePrivate", "QObject",
-	slot_tbl, 1,
-	signal_tbl, 3 );
-}
-#include <qobjectdefs.h>
-
-#if QT_VERSION >= 141
-// newer implementation
-#include <qsignalslotimp.h>
-#else
-// for late-model 1.x header files
-#if !defined(Q_MOC_CONNECTIONLIST_DECLARED)
-#define Q_MOC_CONNECTIONLIST_DECLARED
-#include <qlist.h>
-Q_DECLARE(QListM,QConnection);
-Q_DECLARE(QListIteratorM,QConnection);
-#endif
-#endif
-
-// SIGNAL sizeChanged
-void QMoviePrivate::sizeChanged( const QSize& t0 )
-{
-    QConnectionList *clist = receivers("sizeChanged(const QSize&)");
-    if ( !clist || signalsBlocked() )
-	return;
-    typedef void (QObject::*RT0)();
-    typedef RT0 *PRT0;
-    typedef void (QObject::*RT1)(const QSize&);
-    typedef RT1 *PRT1;
-    RT0 r0;
-    RT1 r1;
-    QConnectionListIt it(*clist);
-    QConnection   *c;
-    QSenderObject *object;
-    while ( (c=it.current()) ) {
-	++it;
-	object = (QSenderObject*)c->object();
-	object->setSender( this );
-	switch ( c->numArgs() ) {
-	    case 0:
-		r0 = *((PRT0)(c->member()));
-		(object->*r0)();
-		break;
-	    case 1:
-		r1 = *((PRT1)(c->member()));
-		(object->*r1)(t0);
-		break;
-	}
-    }
-}
-
-// SIGNAL areaChanged
-void QMoviePrivate::areaChanged( const QRect& t0 )
-{
-    QConnectionList *clist = receivers("areaChanged(const QRect&)");
-    if ( !clist || signalsBlocked() )
-	return;
-    typedef void (QObject::*RT0)();
-    typedef RT0 *PRT0;
-    typedef void (QObject::*RT1)(const QRect&);
-    typedef RT1 *PRT1;
-    RT0 r0;
-    RT1 r1;
-    QConnectionListIt it(*clist);
-    QConnection   *c;
-    QSenderObject *object;
-    while ( (c=it.current()) ) {
-	++it;
-	object = (QSenderObject*)c->object();
-	object->setSender( this );
-	switch ( c->numArgs() ) {
-	    case 0:
-		r0 = *((PRT0)(c->member()));
-		(object->*r0)();
-		break;
-	    case 1:
-		r1 = *((PRT1)(c->member()));
-		(object->*r1)(t0);
-		break;
-	}
-    }
-}
-
-// SIGNAL dataStatus
-void QMoviePrivate::dataStatus( int t0 )
-{
-    activate_signal( "dataStatus(int)", t0 );
-}
+#include "qmovie.moc"
