@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qnpsupport.cpp,v 2.12.2.1 1998/08/12 16:55:08 agulbra Exp $
+** $Id: qt/src/kernel/qnpsupport.cpp   2.1.1   edited 2000-03-24 $
 **
 ** Low-level support for Netscape Plugins under X11.
 **
 ** Created : 970601
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -31,7 +33,6 @@
 void		qt_reset_color_avail();	      // in qcolor_x11.cpp
 int		qt_activate_timers();	      // in qapplication_x11.cpp
 timeval	       *qt_wait_timer();	      // in qapplication_x11.cpp
-void		qt_x11SendPostedEvents();     // in qapplication_x11.cpp
 
 typedef int (*SameAsXtEventDispatchProc)(XEvent*);
 SameAsXtEventDispatchProc qt_np_cascade_event_handler[LASTEvent];
@@ -117,24 +118,24 @@ void qt_np_remove_timeoutcb( SameAsXtTimerCallbackProc cb )
 
 int qt_event_handler( XEvent* event )
 {
-    qt_x11SendPostedEvents();
+    QApplication::sendPostedEvents();
     if ( qApp->x11ProcessEvent( event ) == -1
-	&& !QApplication::activePopupWidget()
-	&& !QApplication::activeModalWidget()
-    ) {
-        // Qt did not recognize the event
+	 && !QApplication::activePopupWidget()
+	 && !QApplication::activeModalWidget()
+	 ) {
+	// Qt did not recognize the event
 	return qt_np_cascade_event_handler[event->type]( event );
     } else {
-        // Qt recognized the event (it may not have actually used it
-        // in a widget, but that is irrelevant here).
+	// Qt recognized the event (it may not have actually used it
+	// in a widget, but that is irrelevant here).
 	if ( event->type == LeaveNotify && qt_np_leave_cb
-	  && !QApplication::activePopupWidget()
-          && !QApplication::activeModalWidget())
-	{
-	    XLeaveWindowEvent* e = (XLeaveWindowEvent*)event;
-	    qt_np_leave_cb(e);
-	}
-        if ( islist ) {
+	     && !QApplication::activePopupWidget()
+	     && !QApplication::activeModalWidget())
+	    {
+		XLeaveWindowEvent* e = (XLeaveWindowEvent*)event;
+		qt_np_leave_cb(e);
+	    }
+	if ( islist ) {
 	    qt_activate_timers();
 	    timeval *tm = qt_wait_timer();
 	    if (tm) {
@@ -142,9 +143,12 @@ int qt_event_handler( XEvent* event )
 		qt_np_set_timer(interval);
 	    }
 	}
-        qt_reset_color_avail();
-	qt_x11SendPostedEvents();
-        return True;
+	qt_reset_color_avail();
+	QApplication::sendPostedEvents();
+	// send the event to Xt in any case (think about XGrabPointer with ownerEvents TRUE
+	if ( event->type != 7 && event->type != 8 )
+	    (void) qt_np_cascade_event_handler[event->type]( event );
+	return True;
     }
 }
 
