@@ -1,45 +1,52 @@
 /****************************************************************************
-** $Id: qpixmap.cpp,v 2.31 1998/07/06 01:57:39 agulbra Exp $
+** $Id: qt/src/kernel/qpixmap.cpp   2.1.1   edited 2000-05-22 $
 **
 ** Implementation of QPixmap class
 **
 ** Created : 950301
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
+#include "qpixmap.h"
+
 #include "qbitmap.h"
 #include "qimage.h"
 #include "qwidget.h"
 #include "qpainter.h"
 #include "qdatastream.h"
 #include "qbuffer.h"
+#include "qobjectlist.h"
+#include "qapplication.h"
 
+// NOT REVISED
 /*!
   \class QPixmap qpixmap.h
   \brief The QPixmap class is an off-screen pixel-based paint device.
 
   \ingroup drawing
   \ingroup shared
-  
+
   It is one of the two classes Qt provides for dealing with images,
   the other being QImage.  QPixmap is designed and optimized for
   drawing; QImage is designed and optimized for I/O and for direct
   pixel access/manipulation.  There are (slow) functions to convert
-  between QImage and QPixmp; convertToImage() and convertFromImage().
+  between QImage and QPixmap; convertToImage() and convertFromImage().
 
   One common use of the QPixmap class is to enable smooth updating of
   widgets.  Whenever something complex needs to be drawn, you can use
@@ -51,27 +58,103 @@
   <li> Paint the pixmap.
   <li> bitBlt() the pixmap contents onto the widget.
   </ol>
-  
+
   Pixel data in a pixmap is internal and managed by the underlying
   window system.  Pixels can only be accessed through QPainter
   functions, through bitBlt(), and by converting the QPixmap to a
   QImage.
 
   You can display a QPixmap on the screen easily using
-  e.g. QLabel::setPixmap(), and all the \link QButton button classes
-  \endlink support pixmap use.
-  
-  There are also convenience functions to get and set single pixels
-  and to load and save the entire pixmap; these work by converting the
-  pixmap to a QImage internally.
+  e.g. QLabel::setPixmap(), and all the QButton subclasses support
+  pixmap use.
 
-  The QPixmap class is optimized by the use of \link shclass.html implicit
-  sharing\endlink, so it is very efficient to pass QPixmap objects as
-  arguments.
+  The QPixmap class uses lazy copying, so it is practical to pass pass
+  QPixmap objects as arguments.
+
+  Note about Windows 95 and 98: On Windows 9x, the system crashes if
+  you create more than approximately 1000 pixmaps, independent of the
+  size of the pixmaps or installed RAM.  Windows NT does not have this
+  limitation.
+
+  Qt tries to work around the resource limitation.  If you set the
+  pixmap optimization to \c QPixmap::MemoryOptim and the width of your
+  pixmap is less than or equal to 128 pixels, Qt stores the pixmap in
+  a way which is very memory-efficient when there are many pixmaps.
+
+  If your application uses dozens or hundreds of pixmaps, e.g. on tool
+  bar buttons, in popup menus, and you plan to run it on Windows 95 or
+  Windows 98, then we recommend using code like this:
+
+  \code
+    QPixmap::setDefaultOptimization( QPixmap::MemoryOptim );
+    while ( ... ) {
+      // load tool bar pixmaps etc.
+      QPixmap *pixmap = new QPixmap(fileName);
+    }
+    QPixmap::setDefaultOptimization( QPixmap::NormalOptim );
+  \endcode
 
   \sa QBitmap, QImage, QImageIO, \link shclass.html Shared Classes\endlink
 */
 
+/*! \enum QPixmap::ColorMode
+
+  This enum type defines the color modes that exist for converting
+  QImage objects to QPixmap.  The current values are: <ul>
+
+  <li> \c Auto - select \c Color or \c Mono on a case-by-case basis.
+  <li> \c Color - always create colored pixmaps.
+  <li> \c Mono - always create bitmaps.
+  </ul>
+*/
+
+/*! \enum QPixmap::Optimization
+
+  QPixmap has the choice of optimizing for speed or memory in a few
+  places, and the best choice varies from pixmap to pixmap, but can
+  generally be derived heuristically.  This enum type defines a number
+  of optimization modes you can set for any pixmap, to tweak the
+  speed/memory tradeoffs:
+
+  <ul>
+
+  <li> \c DefaultOptim - whatever QPixmap::defaultOptimization()
+  returns.  A pixmap with this optimization mode set always has the
+  default optimization type, even if the default is changed with
+  setDefaultOptimization().
+
+  <li> \c NoOptim - no optimization (currently the same as \c MemoryOptim).
+
+  <li> \c MemoryOptim - optimize for minimal memory use.
+
+  <li> \c NormalOptim - optimize for typical usage.  Often uses more
+  memory than \c MemoryOptim, and often faster.
+
+  <li> \c BestOptim - optimize for pixmaps that are drawn very often
+  and where performance is critical.  Generally uses more memory than
+  \c NormalOptim and may provide a little better speed.
+
+  </ul>
+
+  We recommend sticking with \c DefaultOptim
+*/
+
+
+QPixmap::Optimization QPixmap::defOptim = QPixmap::NormalOptim;
+
+
+/*!
+  \internal
+  Private constructor which takes the bitmap flag and the optimization.
+*/
+
+QPixmap::QPixmap( int w, int h, int depth, bool bitmap,
+		  Optimization optimization )
+    : QPaintDevice( QInternal::Pixmap )
+{
+    init( w, h, depth, bitmap, optimization );
+}
+
 
 /*!
   Constructs a null pixmap.
@@ -79,9 +162,9 @@
 */
 
 QPixmap::QPixmap()
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
 }
 
 /*!
@@ -99,22 +182,23 @@ QPixmap::QPixmap()
   \sa isNull()
 */
 
-QPixmap::QPixmap( int w, int h, int depth )
-    : QPaintDevice( PDT_PIXMAP )
+QPixmap::QPixmap( int w, int h, int depth, Optimization optimization )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( w, h, depth );
+    init( w, h, depth, FALSE, optimization );
 }
 
 /*!
-  \overload QPixmap::QPixmap( const QSize &size, int depth )
+  \overload QPixmap::QPixmap( const QSize &size, int depth, Optimization optimization )
 */
 
-QPixmap::QPixmap( const QSize &size, int depth )
-    : QPaintDevice( PDT_PIXMAP )
+QPixmap::QPixmap( const QSize &size, int depth, Optimization optimization )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( size.width(), size.height(), depth );
+    init( size.width(), size.height(), depth, FALSE, optimization );
 }
 
+
 /*!
   Constructs a pixmap from the file \e fileName. If the file does not
   exist, or is of an unknown format, the pixmap becomes a null pixmap.
@@ -124,11 +208,11 @@ QPixmap::QPixmap( const QSize &size, int depth )
   \sa isNull(), load(), loadFromData(), save(), imageFormat()
 */
 
-QPixmap::QPixmap( const char *fileName, const char *format,
+QPixmap::QPixmap( const QString& fileName, const char *format,
 	int conversion_flags )
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     load( fileName, format, conversion_flags );
 }
 
@@ -141,38 +225,94 @@ QPixmap::QPixmap( const char *fileName, const char *format,
   \sa isNull(), load(), loadFromData(), save(), imageFormat()
 */
 
-QPixmap::QPixmap( const char *fileName, const char *format, ColorMode mode )
-    : QPaintDevice( PDT_PIXMAP )
+QPixmap::QPixmap( const QString& fileName, const char *format, ColorMode mode )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     load( fileName, format, mode );
 }
 
 /*!
   Constructs a pixmap from \a xpm, which must be a valid XPM image.
+
+  Error are silently ignored.
+
+  Note that it's possible to squeeze the XPM variable a little bit by
+  using an unusual declaration:
+
+  \example
+    static const char * const start_xpm[]={
+        "16 15 8 1",
+        "a c #cec6bd",
+    ....
+  \endcode
+
+  The extra \c const makes the entire definition read-only, which is
+  slightly more efficient e.g. when the code is in a shared library,
+  and ROMable when the application is to be stored in ROM.
+
+  In order to use that sort of declaration, you must cast the variable
+  back to <nobr><code>const char **</code></nobr> when you create the
+  QPixmap.
 */
 
 QPixmap::QPixmap( const char *xpm[] )
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     QImage image( xpm );
     if ( !image.isNull() )
 	convertFromImage( image );
 }
 
-/*
-  Constructs a pixmaps by loading from \a data.
+/*!
+  Constructs a pixmaps by loading from \a img_data.
+  The data can be in any image format supported by Qt.
 
   \sa loadFromData()
 */
-/* Not until Qt 2.0 - adding conversions break sourcecode.
-QPixmap::QPixmap( const QByteArray & data )
+
+QPixmap::QPixmap( const QByteArray & img_data )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    loadFromData(data);
+    init( 0, 0, 0, FALSE, defOptim );
+    loadFromData( img_data );
 }
+
+
+/*!
+  Constructs a pixmap which is a copy of \e pixmap.
 */
 
+QPixmap::QPixmap( const QPixmap &pixmap )
+    : QPaintDevice( QInternal::Pixmap )
+{
+    if ( pixmap.paintingActive() ) {		// make a deep copy
+	data = 0;
+	operator=( pixmap.copy() );
+    } else {
+	data = pixmap.data;
+	data->ref();
+	devFlags = pixmap.devFlags;		// copy QPaintDevice flags
+#if defined(_WS_WIN_)
+	hdc = pixmap.hdc;			// copy Windows device context
+#elif defined(_WS_X11_)
+	hd = pixmap.hd;				// copy X11 drawable
+	copyX11Data( &pixmap );			// copy x11Data
+#endif
+    }
+}
+
+
+/*!
+  Destructs the pixmap.
+*/
+
+QPixmap::~QPixmap()
+{
+    deref();
+}
+
 
 /*!
   Returns a
@@ -181,18 +321,59 @@ QPixmap::QPixmap( const QByteArray & data )
   \sa operator=()
 */
 
-QPixmap QPixmap::copy() const
+QPixmap QPixmap::copy( bool ignoreMask ) const
 {
-    QPixmap tmp( data->w, data->h, data->d );
-    tmp.data->optim  = data->optim;		// copy optimization flag
-    tmp.data->bitmap = data->bitmap;		// copy bitmap flag
-    tmp.data->opt    = data->opt;		// copy optimization setting
-    if ( !tmp.isNull() ) {			// copy the bitmap
-	bitBlt( &tmp, 0,0, this, 0,0, data->w, data->h, CopyROP, TRUE );
-	if ( data->mask )			// copy the mask
-	    tmp.setMask( *data->mask );
+    QPixmap pm( data->w, data->h, data->d, data->bitmap, data->optim );
+    if ( !pm.isNull() ) {			// copy the bitmap
+#if defined(_WS_X11_)
+	pm.copyX11Data( this );
+#endif
+	bitBlt( &pm, 0,0, this, 0,0, data->w, data->h, CopyROP, TRUE );
+	if ( !ignoreMask && data->mask )		// copy the mask
+	    pm.setMask( data->selfmask ? *((QBitmap*)&pm) : *data->mask );
     }
-    return tmp;
+    return pm;
+}
+
+
+/*!
+  Assigns the pixmap \e pixmap to this pixmap and returns a reference to
+  this pixmap.
+*/
+
+QPixmap &QPixmap::operator=( const QPixmap &pixmap )
+{
+    if ( paintingActive() ) {
+#if defined(CHECK_STATE)
+	qWarning("QPixmap::operator=: Cannot assign to pixmap during painting");
+#endif
+	return *this;
+    }
+    pixmap.data->ref();				// avoid 'x = x'
+    deref();
+    if ( pixmap.paintingActive() ) {		// make a deep copy
+	init( pixmap.width(), pixmap.height(), pixmap.depth(),
+	      pixmap.data->bitmap, pixmap.data->optim );
+	data->uninit = FALSE;
+	if ( !isNull() ) {
+	    bitBlt( this, 0, 0, &pixmap, pixmap.width(), pixmap.height(),
+		    CopyROP, TRUE );
+	    if ( pixmap.mask() )
+		setMask( pixmap.data->selfmask ? *((QBitmap*)(this))
+					       : *pixmap.mask() );
+	}
+	pixmap.data->deref();
+    } else {
+	data = pixmap.data;
+	devFlags = pixmap.devFlags;		// copy QPaintDevice flags
+#if defined(_WS_WIN_)
+	hdc = pixmap.hdc;
+#elif defined(_WS_X11_)
+	hd = pixmap.hd;				// copy QPaintDevice drawable
+	copyX11Data( &pixmap );			// copy x11Data
+#endif
+    }
+    return *this;
 }
 
 
@@ -264,6 +445,7 @@ QPixmap &QPixmap::operator=( const QImage &image )
 
 /*!
   \fn void QPixmap::fill( const QWidget *widget, const QPoint &ofs )
+
   Fills the pixmap with the widget's background color or pixmap.
   If the background is empty, nothing is done.
 
@@ -287,7 +469,7 @@ QPixmap &QPixmap::operator=( const QImage &image )
 
     QPainter p( &pix );
     p.translate( -ur.x(), -ur.y() );	// use widget coordinate system
-                                        // when drawing on pixmap
+					// when drawing on pixmap
     //    ... draw on pixmap ...
 
     p.end();
@@ -309,7 +491,7 @@ void QPixmap::fill( const QWidget *widget, int xofs, int yofs )
 	    QPainter p;
 	    p.begin( this );
 	    p.setPen( NoPen );
-	    p.setBrush( QBrush( black,*widget->backgroundPixmap() ) );
+	    p.setBrush( QBrush( Qt::black,*widget->backgroundPixmap() ) );
 	    p.setBrushOrigin( -xofs, -yofs );
 	    p.drawRect( 0, 0, width(), height() );
 	    p.end();
@@ -335,31 +517,29 @@ void QPixmap::fill( const QWidget *widget, int xofs, int yofs )
 void QPixmap::resize( int w, int h )
 {
     if ( w < 1 || h < 1 ) {			// becomes null
-	QPixmap pm;
-	pm.data->optim	= data->optim;		// keep optimization flag
-	pm.data->bitmap = data->bitmap;		// keep is-a flag
-	pm.data->opt	= data->opt;		// keep optimization setting
+	QPixmap pm( 0, 0, 0, data->bitmap, data->optim );
 	*this = pm;
 	return;
     }
-
     int d;
     if ( depth() > 0 )
 	d = depth();
     else
 	d = isQBitmap() ? 1 : -1;
-    QPixmap pm( w, h, d );			// create new pixmap
+    // Create new pixmap
+    QPixmap pm( w, h, d, data->bitmap, data->optim );
     if ( !data->uninit && !isNull() )		// has existing pixmap
 	bitBlt( &pm, 0, 0, this, 0, 0,		// copy old pixmap
 		QMIN(width(), w),
 		QMIN(height(),h), CopyROP, TRUE );
-    pm.data->optim  = data->optim;		// keep optimization flag
-    pm.data->bitmap = data->bitmap;		// keep bitmap flag
-    pm.data->opt    = data->opt;		// keep optimization setting
     if ( data->mask ) {				// resize mask as well
-	QBitmap m = *data->mask;
-	m.resize( w, h );
-	pm.setMask( m );
+	if ( data->selfmask ) {			// preserve self-mask
+	    pm.setMask( *((QBitmap*)&pm) );
+	} else {				// independent mask
+	    QBitmap m = *data->mask;
+	    m.resize( w, h );
+	    pm.setMask( m );
+	}
     }
     *this = pm;
 }
@@ -385,38 +565,43 @@ void QPixmap::resize( int w, int h )
   \sa mask(), createHeuristicMask(), QBitmap
 */
 
-void QPixmap::setMask( const QBitmap &mask )
+void QPixmap::setMask( const QBitmap &newmask )
 {
-    if ( mask.handle() && mask.handle() == handle() ) {
-	const QPixmap *tmp = &mask;		// dec cxx bug
-	QPixmap m = tmp->copy();
+    const QPixmap *tmp = &newmask;		// dec cxx bug
+    if ( (data == tmp->data) ||
+	 ( newmask.handle() && newmask.handle() == handle() ) ) {
+	QPixmap m = tmp->copy( TRUE );
 	setMask( *((QBitmap*)&m) );
 	data->selfmask = TRUE;			// mask == pixmap
 	return;
     }
     detach();
     data->selfmask = FALSE;
-    if ( mask.isNull() ) {			// reset the mask
+    if ( newmask.isNull() ) {			// reset the mask
 	delete data->mask;
 	data->mask = 0;
 	return;
     }
-    if ( mask.width() != width() || mask.height() != height() ) {
+    if ( newmask.width() != width() || newmask.height() != height() ) {
 #if defined(CHECK_RANGE)
-	warning( "QPixmap::setMask: The pixmap and the mask must have "
+	qWarning( "QPixmap::setMask: The pixmap and the mask must have "
 		 "the same size" );
 #endif
 	return;
     }
     delete data->mask;
-    data->mask = new QBitmap( mask );
+    QBitmap* newmaskcopy;
+    if ( newmask.mask() )
+	newmaskcopy = (QBitmap*)new QPixmap( tmp->copy( TRUE ) );
+    else
+	newmaskcopy = new QBitmap( newmask );
+    data->mask = newmaskcopy;
 }
 
 
 /*!
   \fn bool QPixmap::selfMask() const
-  Returns TRUE if the pixmap's mask is identical to the pixmap
-  itself.
+  Returns TRUE if the pixmap's mask is identical to the pixmap itself.
   \sa mask()
 */
 
@@ -455,29 +640,12 @@ QBitmap QPixmap::createHeuristicMask( bool clipTight ) const
   \sa load(), save()
 */
 
-const char *QPixmap::imageFormat( const char *fileName )
+const char* QPixmap::imageFormat( const QString &fileName )
 {
     return QImageIO::imageFormat(fileName);
 }
 
 
-static bool can_handle_bmp = FALSE;
-static bool did_handle_bmp = FALSE;
-
-bool qt_image_native_bmp()
-{
-    if ( can_handle_bmp ) {
-	did_handle_bmp = TRUE;
-	return TRUE;
-    }
-    return FALSE;
-}
-
-bool qt_image_did_native_bmp()
-{
-    return did_handle_bmp;
-}
-
 /*!
   Loads a pixmap from the file \e fileName.
   Returns TRUE if successful, or FALSE if the pixmap could not be loaded.
@@ -495,28 +663,23 @@ bool qt_image_did_native_bmp()
   \sa loadFromData(), save(), imageFormat(), QImage::load(), QImageIO
 */
 
-bool QPixmap::load( const char *fileName, const char *format,
+bool QPixmap::load( const QString &fileName, const char *format,
 		    int conversion_flags )
 {
     QImageIO io( fileName, format );
-#if defined(_WS_WIN_)
-    can_handle_bmp = TRUE;
-#endif
     bool result = io.read();
     if ( result ) {
-	detach();
+	detach(); // ###hanord: Why detach here, convertFromImage does it
 	result = convertFromImage( io.image(), conversion_flags );
     }
-#if defined(_WS_WIN_)
-    can_handle_bmp = did_handle_bmp = FALSE;
-#endif
     return result;
 }
 
 /*!
   \overload
 */
-bool QPixmap::load( const char *fileName, const char *format,
+
+bool QPixmap::load( const QString &fileName, const char *format,
 		    ColorMode mode )
 {
     int conversion_flags = 0;
@@ -533,10 +696,12 @@ bool QPixmap::load( const char *fileName, const char *format,
     return load( fileName, format, conversion_flags );
 }
 
+
 /*!
   \overload
 */
-bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
+
+bool QPixmap::convertFromImage( const QImage &image, ColorMode mode )
 {
     int conversion_flags = 0;
     switch (mode) {
@@ -549,7 +714,7 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
       default:
 	;// Nothing.
     }
-    return convertFromImage( img, conversion_flags );
+    return convertFromImage( image, conversion_flags );
 }
 
 
@@ -578,9 +743,6 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
     QBuffer b( a );
     b.open( IO_ReadOnly );
     QImageIO io( &b, format );
-#if defined(_WS_WIN_)
-    can_handle_bmp = TRUE;
-#endif
     bool result = io.read();
     b.close();
     a.resetRawData( (char *)buf, len );
@@ -588,15 +750,13 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
 	detach();
 	result = convertFromImage( io.image(), conversion_flags );
     }
-#if defined(_WS_WIN_)
-    can_handle_bmp = did_handle_bmp = FALSE;
-#endif
     return result;
 }
 
 /*!
   \overload
 */
+
 bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
 			    ColorMode mode )
 {
@@ -617,14 +777,15 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
 /*!
   \overload
 */
-bool QPixmap::loadFromData( QByteArray buf,
-			    const char *format,
+
+bool QPixmap::loadFromData( const QByteArray &buf, const char *format,
 			    int conversion_flags )
 {
     return loadFromData( (const uchar *)(buf.data()), buf.size(),
 			 format, conversion_flags );
 }
 
+
 /*!
   Saves the pixmap to the file \e fileName, using the image file format
   \e format.  Returns TRUE if successful, or FALSE if the pixmap could not
@@ -632,7 +793,7 @@ bool QPixmap::loadFromData( QByteArray buf,
   \sa load(), loadFromData(), imageFormat(), QImage::save(), QImageIO
 */
 
-bool QPixmap::save( const char *fileName, const char *format ) const
+bool QPixmap::save( const QString &fileName, const char *format ) const
 {
     if ( isNull() )
 	return FALSE;				// nothing to save
@@ -645,26 +806,160 @@ bool QPixmap::save( const char *fileName, const char *format ) const
 /*!
   \fn int QPixmap::serialNumber() const
 
-  Returns a number that uniquely identifies this QPixmap object. The
-  serial number is very useful for caching.
+  Returns a number that uniquely identifies the contents of this QPixmap object.
+  This means that multiple QPixmaps objects can have the same serial number
+  as long as they refere to the same contents.
+  The serial number is for example very useful for caching.
 
   \sa QPixmapCache
 */
 
 
+/*!
+  Returns the default pixmap optimization setting.
+  \sa setDefaultOptimization(), setOptimization(), optimization()
+*/
+
+QPixmap::Optimization QPixmap::defaultOptimization()
+{
+    return defOptim;
+}
+
+/*!
+  Sets the default pixmap optimization.
+
+  All \e new pixmaps that are created will use this default optimization.
+  You may also set optimization for individual pixmaps using the
+  setOptimization() function.
+
+  The initial default optimization setting is \c QPixmap::Normal.
+
+  \sa defaultOptimization(), setOptimization(), optimization()
+*/
+
+void QPixmap::setDefaultOptimization( Optimization optimization )
+{
+    if ( optimization != DefaultOptim )
+	defOptim = optimization;
+}
+
+
+// helper for next function.
+static QPixmap grabChildWidgets( QWidget * w )
+{
+    QPixmap res( w->width(), w->height() );
+    res.fill( w, QPoint( 0, 0 ) );
+    QPainter::redirect( w, &res ); // ### overwrites earlier redirect
+    QPaintEvent e( w->rect(), FALSE );
+    QApplication::sendEvent( w, &e );
+    QPainter::redirect( w, 0 );
+
+    const QObjectList * children = w->children();
+    if ( children ) {
+	QPainter p( &res );
+	QObjectListIt it( *children );
+	QObject * child;
+	while( (child=it.current()) != 0 ) {
+	    ++it;
+	    if ( child->isWidgetType() &&
+		 !((QWidget *)child)->testWState( Qt::WState_ForceHide ) &&
+		 ((QWidget *)child)->geometry().intersects( w->rect() ) ) {
+		// those conditions aren't quite right, it's possible
+		// to have a grandchild completely outside its
+		// grandparent, but partially inside its parent.  no
+		// point in optimizing for that.
+
+		// make sure to evaluate pos() first - who knows what
+		// the paint event(s) inside grabChildWidgets() will do.
+		QPoint childpos = ((QWidget *)child)->pos();
+		p.drawPixmap( childpos, grabChildWidgets( (QWidget *)child ) );
+	    }
+	}
+    }
+    return res;
+}
+
+
+/*!  Creates a pixmap and paints \a widget in it.
+
+  If \a widget has children, they are painted too, appropriately located.
+
+  If you specify \a x, \a y, \a w or \a h, only the rectangle you
+  specify is painted.  The defaults are 0, 0 (top-left corner) and
+  -1,-1 (which means the entire widget).
+
+  (If \e w is negative, the function copies everything to the right
+  border of the window.  If \e h is negative, the function copies
+  everything to the bottom of the window.)
+
+  If \a widget is 0, or if the rectangle defined by \a x, \a y, the
+  modified \a w and the modified \a h does not overlap the \a
+  widget->rect(), this function returns a null QPixmap.
+
+  This function actually asks \a widget to paint itself (and its
+  children to paint themselves).  QPixmap::grabWindow() grabs pixels
+  off the screen, which is a bit faster and picks up \e exactly what's
+  on-screen.  This function works by calling paintEvent() with painter
+  redirection turned on, which gets the result of paintEvent(),
+  without e.g. overlying windows.
+
+  If there is overlap, it returns a pixmap of the size you want,
+  containing a rendering of \a widget.  If the rectangle you ask for
+  is a superset of \a widget, the area outside \a widget are covered
+  with the widget's background.
+
+  \sa grabWindow() QPainter::redirect() QWidget::paintEvent()
+*/
+
+QPixmap QPixmap::grabWidget( QWidget * widget, int x, int y, int w, int h )
+{
+    QPixmap res;
+    if ( !widget )
+	return res;
+
+    if ( w < 0 )
+	w = widget->width() - x;
+    if ( h < 0 )
+	h = widget->height() - y;
+
+    QRect wr( x, y, w, h );
+    if ( wr == widget->rect() )
+	return grabChildWidgets( widget );
+    if ( !wr.intersects( widget->rect() ) )
+	return res;
+
+    res.resize( w, h );
+    res.fill( widget, QPoint( w,h ) );
+    QPixmap tmp( grabChildWidgets( widget ) );
+    ::bitBlt( &res, 0, 0, &tmp, x, y, w, h );
+    return res;
+}
+
+
+
+
+
 /*****************************************************************************
   QPixmap stream functions
  *****************************************************************************/
 
 /*!
   \relates QPixmap
-  Writes a pixmap to the stream as a BMP image.
+  Writes a pixmap to the stream as a PNG image.
+
   \sa QPixmap::save()
+  \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QPixmap &pixmap )
 {
-    QImageIO io( s.device(), "BMP" );
+    QImageIO io;
+    io.setIODevice( s.device() );
+    if ( s.version() == 1 )
+	io.setFormat( "BMP" );
+    else
+	io.setFormat( "PNG" );
+
     io.setImage( pixmap.convertToImage() );
     io.write();
     return s;
@@ -674,6 +969,7 @@ QDataStream &operator<<( QDataStream &s, const QPixmap &pixmap )
   \relates QPixmap
   Reads a pixmap from the stream.
   \sa QPixmap::load()
+  \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QPixmap &pixmap )
@@ -683,3 +979,4 @@ QDataStream &operator>>( QDataStream &s, QPixmap &pixmap )
 	pixmap.convertFromImage( io.image() );
     return s;
 }
+
