@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qpixmap.h,v 2.22.2.4 1998/10/08 18:21:02 hanord Exp $
+** $Id: qt/src/kernel/qpixmap.h   2.1.1   edited 2000-04-05 $
 **
 ** Definition of QPixmap class
 **
 ** Created : 940501
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -28,22 +30,32 @@
 #include "qpaintdevice.h"
 #include "qcolor.h"
 #include "qstring.h"
+#include "qnamespace.h"
 #endif // QT_H
 
 
-class Q_EXPORT QPixmap : public QPaintDevice		// pixmap class
+#if defined(_WS_WIN_)
+// Internal pixmap memory optimization class for Windows 9x
+class QMultiCellPixmap;
+#endif
+
+
+class Q_EXPORT QPixmap : public QPaintDevice, public Qt
 {
-friend class QPaintDevice;
-friend class QPainter;
 public:
     enum ColorMode { Auto, Color, Mono };
+    enum Optimization { DefaultOptim, NoOptim, MemoryOptim=NoOptim,
+			NormalOptim, BestOptim };
 
     QPixmap();
-    QPixmap( int w, int h,  int depth=-1 );
-    QPixmap( const QSize &, int depth=-1 );
-    QPixmap( const char *fileName, const char *format=0, ColorMode mode=Auto );
-    QPixmap( const char *fileName, const char *format, int conversion_flags );
+    QPixmap( int w, int h,  int depth = -1, Optimization = DefaultOptim );
+    QPixmap( const QSize &, int depth = -1, Optimization = DefaultOptim );
+    QPixmap( const QString& fileName, const char *format=0,
+	     ColorMode mode=Auto );
+    QPixmap( const QString& fileName, const char *format,
+	     int conversion_flags );
     QPixmap( const char *xpm[] );
+    QPixmap( const QByteArray &data );
     QPixmap( const QPixmap & );
    ~QPixmap();
 
@@ -59,7 +71,7 @@ public:
     int		depth()		const { return data->d; }
     static int	defaultDepth();
 
-    void	fill( const QColor &fillColor=white );
+    void	fill( const QColor &fillColor = Qt::white );
     void	fill( const QWidget *, int xofs, int yofs );
     void	fill( const QWidget *, const QPoint &ofs );
     void	resize( int width, int height );
@@ -70,7 +82,9 @@ public:
     bool	selfMask() const;
     QBitmap	createHeuristicMask( bool clipTight = TRUE ) const;
 
-    static  QPixmap  grabWindow( WId, int x=0, int y=0, int w=-1, int h=-1 );
+    static  QPixmap grabWindow( WId, int x=0, int y=0, int w=-1, int h=-1 );
+    static  QPixmap grabWidget( QWidget * widget,
+				int x=0, int y=0, int w=-1, int h=-1 );
 
     QPixmap	    xForm( const QWMatrix & ) const;
     static QWMatrix trueMatrix( const QWMatrix &, int w, int h );
@@ -79,101 +93,104 @@ public:
     bool	convertFromImage( const QImage &, ColorMode mode=Auto );
     bool	convertFromImage( const QImage &, int conversion_flags );
 
-    static const char *imageFormat( const char *fileName );
-    bool	load( const char *fileName, const char *format=0,
+    static const char* imageFormat( const QString &fileName );
+    bool	load( const QString& fileName, const char *format=0,
 		      ColorMode mode=Auto );
-    bool	load( const char *fileName, const char *format,
+    bool	load( const QString& fileName, const char *format,
 		      int conversion_flags );
     bool	loadFromData( const uchar *buf, uint len,
-			      const char *format=0,
+			      const char* format=0,
 			      ColorMode mode=Auto );
     bool	loadFromData( const uchar *buf, uint len,
-			      const char *format,
+			      const char* format,
 			      int conversion_flags );
-    bool	loadFromData( QByteArray data,
-			      const char *format=0,
+    bool	loadFromData( const QByteArray &data,
+			      const char* format=0,
 			      int conversion_flags=0 );
-    bool	save( const char *fileName, const char *format ) const;
+    bool	save( const QString& fileName, const char* format ) const;
 
-#if defined(_WS_WIN_) || defined(_WS_PM_)
-    HANDLE	hbm() const;
+#if defined(_WS_WIN_)
+    HBITMAP	hbm()		const;
 #endif
 
     int		serialNumber()	const;
 
-    enum Optimization { NoOptim, NormalOptim, BestOptim };
-
     Optimization	optimization() const;
     void		setOptimization( Optimization );
     static Optimization defaultOptimization();
     static void		setDefaultOptimization( Optimization );
 
-#if 1
-    bool	isOptimized() const;
-    void	optimize( bool );
-    static bool isGloballyOptimized();
-    static void optimizeGlobally( bool );
-#endif
-
     virtual void detach();
 
     bool	isQBitmap() const;
 
 #if defined(_WS_WIN_)
-    HANDLE	allocMemDC();
-    void	freeMemDC();
+    // These functions are internal and used by Windows 9x only
+    bool	isMultiCellPixmap() const;
+    HDC		multiCellHandle() const;
+    HBITMAP	multiCellBitmap() const;
+    int		multiCellOffset() const;
+    int		allocCell();
+    void	freeCell( bool = FALSE );
 #endif
 
 protected:
     QPixmap( int w, int h, const uchar *data, bool isXbitmap );
-    int		metric( int ) const;
+    int metric( int ) const;
+
+#if defined(_WS_WIN_)
+    struct QMCPI {				// mem optim for win9x
+	QMultiCellPixmap *mcp;
+	int	offset;
+    };
+#endif
 
     struct QPixmapData : public QShared {	// internal pixmap data
 	QCOORD	w, h;
 	short	d;
-	uint	unused	 : 1;
-	uint	optim	 : 1;
 	uint	uninit	 : 1;
 	uint	bitmap	 : 1;
 	uint	selfmask : 1;
+#if defined(_WS_WIN_)
+	uint	mcp	 : 1;
+#endif
 	int	ser_no;
 	QBitmap *mask;
 #if defined(_WS_WIN_)
-	HANDLE	hbm;
 	void   *bits;
 	QPixmap *maskpm;
-#elif defined(_WS_PM_)
-	HANDLE	hdcmem;
-	HANDLE	hbm;
+	union {
+	    HBITMAP hbm;    // if mcp == FALSE
+	    QMCPI  *mcpi;   // if mcp == TRUE
+	} hbm_or_mcpi;
 #elif defined(_WS_X11_)
 	void   *ximage;
 	void   *maskgc;
 #endif
-	Optimization opt;
+	Optimization optim;
     } *data;
 
 private:
-    void	init( int, int, int );
+    QPixmap( int w, int h, int depth, bool, Optimization );
+    void	init( int, int, int, bool, Optimization );
     void	deref();
-    QPixmap	copy() const;
-    static bool optimAll;
-    static Optimization defOpt;
+    QPixmap	copy( bool ignoreMask = FALSE ) const;
+    static Optimization defOptim;
     friend Q_EXPORT void bitBlt( QPaintDevice *, int, int,
 				 const QPaintDevice *,
 				 int, int, int, int, RasterOp, bool );
     friend Q_EXPORT void bitBlt( QPaintDevice *, int, int,
 				 const QImage* src,
 				 int, int, int, int, int conversion_flags );
+    friend class QBitmap;
+    friend class QPaintDevice;
+    friend class QPainter;
 };
 
 
 inline bool QPixmap::isNull() const
 {
-#if defined(_WS_X11_)
-    return hd == 0;
-#else
-    return data->hbm == 0;
-#endif
+    return data->w == 0;
 }
 
 inline void QPixmap::fill( const QWidget *w, const QPoint &ofs )
@@ -196,10 +213,10 @@ inline bool QPixmap::selfMask() const
     return data->selfmask;
 }
 
-#if defined(_WS_WIN_) || defined(_WS_PM_)
-inline HANDLE QPixmap::hbm() const
+#if defined(_WS_WIN_)
+inline HBITMAP QPixmap::hbm() const
 {
-    return data->hbm;
+    return data->mcp ? 0 : data->hbm_or_mcpi.hbm;
 }
 #endif
 
@@ -209,11 +226,6 @@ inline int QPixmap::serialNumber() const
 }
 
 inline QPixmap::Optimization QPixmap::optimization() const
-{
-    return data->opt;
-}
-
-inline bool QPixmap::isOptimized() const
 {
     return data->optim;
 }
@@ -223,6 +235,13 @@ inline bool QPixmap::isQBitmap() const
     return data->bitmap;
 }
 
+#if defined(_WS_WIN_)
+inline bool QPixmap::isMultiCellPixmap() const
+{
+    return data->mcp;
+}
+#endif
+
 
 /*****************************************************************************
   QPixmap stream functions
