@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qpointarray.cpp,v 2.17 1998/07/03 00:09:38 hanord Exp $
+** $Id: qt/src/kernel/qpointarray.cpp   2.1.1   edited 2000-05-22 $
 **
 ** Implementation of QPointArray class
 **
 ** Created : 940213
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,47 +27,13 @@
 #include "qrect.h"
 #include "qbitarray.h"
 #include "qdatastream.h"
+#include "qwmatrix.h"
 #include <stdarg.h>
 
-/*!
-  \class QPointVal qpointarray.h
-  \brief The QPointVal class is an internal class, used with QPointArray.
-
-  The QPointVal is required by the indexing [] operator on point arrays.
-  It is probably a bad idea to use it in any other context.
-*/
-
-/*! \fn QPointVal::QPointVal (QPointData* ptr)
-
-  Constructs a reference to an element in a QPointArray.  This is
-  what QPointArray::operator[] contructs its return value with.
-*/
-
-/*! \fn QPointVal& QPointVal::operator= (const QPoint& point)
-  Assigns a given point to the value referenced by this QPointVal.
-*/
-
-/*! \fn QPointVal& QPointVal::operator+= (const QPoint& point)
-  Vector-adds a given point to the value referenced by this QPointVal.
-*/
-
-/*! \fn QPointVal& QPointVal::operator-= (const QPoint& point)
-  Vector-subtracts a given point to the value referenced by this QPointVal.
-*/
-
-/*! \fn QPointVal::operator QPoint() const
-  Returns the value of the point referenced by this QPointVal.
-*/
-
-/*! \fn int QPointVal::x () const
-  Returns the X coordinate of the point referenced by this QPointVal.
-*/
-
-/*! \fn int QPointVal::y () const
-  Returns the Y coordinate of the point referenced by this QPointVal.
-*/
+const double Q_PI   = 3.14159265358979323846;   // pi
 
 
+// NOT REVISED
 /*!
   \class QPointArray qpointarray.h
   \brief The QPointArray class provides an array of points.
@@ -74,17 +42,22 @@
 
   \inherit QArray
 
+  The QPointArray is an array of QPoint objects. In addition to the functions
+  provided by QArray, QPointArray provides some handy methods:
+
+  For convenient reading and writing of the point data: setPoints(),
+  putPoints(), point(), and setPoint().
+
+  For geometry operations: boundingRect() and translate(). As for the latter,
+  note that QWMatrix provides a map() function for more general transformation
+  of QPointArrays.
+
   QPointArray is used by the QPainter to draw
   \link QPainter::drawLineSegments() line segments\endlink,
   \link QPainter::drawPolyline() polylines\endlink,
   \link QPainter::drawPolygon() polygons\endlink and
   \link QPainter::drawQuadBezier() Bezier curves\endlink.
 
-  The QPointArray is not an array of QPoint, instead it contains a
-  platform dependent point type to make the QPainter functions more
-  efficient (no conversion needed).  On the other hand, QPointArray
-  has member functions that operate on QPoint to make programming
-  easier.
 
   Note that since this class is a QArray, it is
   \link shclass.html explicitly shared\endlink
@@ -159,6 +132,13 @@ QPointArray::QPointArray( int nPoints, const QCOORD *points )
 }
 
 
+/*!
+  \fn QPointArray::~QPointArray()
+
+  Destructs the point array.
+*/
+
+
 /*!
   \fn QPointArray &QPointArray::operator=( const QPointArray &a )
   Assigns a
@@ -177,22 +157,6 @@ QPointArray::QPointArray( int nPoints, const QCOORD *points )
   \link shclass.html deep copy\endlink of the array.
 */
 
-/*!
-  Fills the point array with the point \e p.
-  If \e size is specified as different from -1, then the array will be
-  resized before filled.
-
-  Returns TRUE if successful, or FALSE if the memory cannot be allocated
-  (only when \e size != -1).
-
-  \sa resize()
-*/
-
-bool QPointArray::fill( const QPoint &p, int size )
-{
-    QPointData p2( p.x(), p.y() );
-    return QArrayM(QPointData)::fill( p2, size );
-}
 
 
 /*!
@@ -201,11 +165,11 @@ bool QPointArray::fill( const QPoint &p, int size )
 
 void QPointArray::translate( int dx, int dy )
 {
-    register QPointData *p = data();
+    register QPoint *p = data();
     register int i = size();
+    QPoint pt( dx, dy );
     while ( i-- ) {
-	p->x += (Qpnta_t)dx;
-	p->y += (Qpnta_t)dy;
+	*p += pt;
 	p++;
     }
 }
@@ -217,9 +181,9 @@ void QPointArray::translate( int dx, int dy )
 
 void QPointArray::point( uint index, int *x, int *y ) const
 {
-    QPointData p = QArrayM(QPointData)::at( index );
-    *x = (int)p.x;
-    *y = (int)p.y;
+    QPoint p = QArray<QPoint>::at( index );
+    *x = (int)p.x();
+    *y = (int)p.y();
 }
 
 /*!
@@ -228,8 +192,7 @@ void QPointArray::point( uint index, int *x, int *y ) const
 
 QPoint QPointArray::point( uint index ) const
 {
-    QPointData p = QArrayM(QPointData)::at( index );
-    return QPoint( (QCOORD)p.x, (QCOORD)p.y );
+    return QArray<QPoint>::at( index );
 }
 
 /*!
@@ -238,10 +201,7 @@ QPoint QPointArray::point( uint index ) const
 
 void QPointArray::setPoint( uint index, int x, int y )
 {
-    QPointData p;
-    p.x = (Qpnta_t)x;
-    p.y = (Qpnta_t)y;
-    QArrayM(QPointData)::at( index ) = p;
+    QArray<QPoint>::at( index ) = QPoint( x, y );
 }
 
 /*!
@@ -402,42 +362,6 @@ bool QPointArray::putPoints( int index, int nPoints, int firstx, int firsty,
 }
 
 
-/*!
-  Returns the point at position \e index in the array.
-  \sa operator[]
-*/
-
-QPoint QPointArray::at( uint index ) const
-{
-    QPointData p = QArrayM(QPointData)::at( index );
-    return QPoint( (QCOORD)p.x, (QCOORD)p.y );
-}
-
-/*!
-  \fn QPointVal QPointArray::operator[]( int index )
-
-  Returns a reference to the point at position \e index in the array.
-*/
-
-/*!
-  \fn QPointVal QPointArray::operator[]( uint index )
-
-  Returns a reference to the point at position \e index in the array.
-*/
-
-/*!
-  \fn QPoint QPointArray::operator[] (int i) const
-
-  Returns the point at position \e index in the array.
-*/
-
-/*!
-  \fn QPoint QPointArray::operator[] (uint i) const
-
-  Returns the point at position \e index in the array.
-*/
-
-
 /*!
   Returns the bounding rectangle of the points in the array, or
   QRect(0,0,0,0) if the array is empty.
@@ -447,20 +371,20 @@ QRect QPointArray::boundingRect() const
 {
     if ( isEmpty() )
 	return QRect( 0, 0, 0, 0 );		// null rectangle
-    register QPointData *pd = data();
+    register QPoint *pd = data();
     int minx, maxx, miny, maxy;
-    minx = maxx = pd->x;
-    miny = maxy = pd->y;
+    minx = maxx = pd->x();
+    miny = maxy = pd->y();
     pd++;
     for ( int i=1; i<(int)size(); i++ ) {	// find min+max x and y
-	if ( pd->x < minx )
-	    minx = pd->x;
-	else if ( pd->x > maxx )
-	    maxx = pd->x;
-	if ( pd->y < miny )
-	    miny = pd->y;
-	else if ( pd->y > maxy )
-	    maxy = pd->y;
+	if ( pd->x() < minx )
+	    minx = pd->x();
+	else if ( pd->x() > maxx )
+	    maxx = pd->x();
+	if ( pd->y() < miny )
+	    miny = pd->y();
+	else if ( pd->y() > maxy )
+	    maxy = pd->y();
 	pd++;
     }
     return QRect( QPoint(minx,miny), QPoint(maxx,maxy) );
@@ -479,15 +403,21 @@ static inline int fix_angle( int a )
 /*!
   Sets the points of the array to those describing an arc of an
   ellipse with size \a w by \a h and position (\a x, \a y ), starting
-  from angle \a1, spanning \a a2.
+  from angle \a a1, spanning \a a2.  The resulting array has sufficient
+  resolution for pixel accuracy (see the overloaded function which
+  takes an additional QWMatrix parameter).
+
   Angles are specified in 16ths of a degree,
   i.e. a full circle equals 5760 (16*360). Positive values mean
   counter-clockwise while negative values mean clockwise direction.
-  Zero degrees is at the 3'o clock position.
+  Zero degrees is at the 3 o'clock position.
 */
 
 void QPointArray::makeArc( int x, int y, int w, int h, int a1, int a2 )
 {
+    QWMatrix unit;
+    makeArc(x,y,w,h,a1,a2,unit);
+#if QT_OLD_MAKEELLIPSE // ### WWA says discard this.
     a1 = fix_angle( a1 );
     if ( a1 < 0 )
 	a1 += 16*360;
@@ -502,25 +432,194 @@ void QPointArray::makeArc( int x, int y, int w, int h, int a1, int a2 )
 	while ( npts-- ) {
 	    if ( i >= (int)size() )			// wrap index
 		i = 0;
-	    a.QArrayM(QPointData)::at( j++ ) = QArrayM(QPointData)::at( i++ );
+	    a.QArray<QPoint>::at( j++ ) = QArray<QPoint>::at( i++ );
 	}
     } else {
 	while ( npts-- ) {
 	    if ( i < 0 )				// wrap index
 		i = (int)size()-1;
-	    a.QArrayM(QPointData)::at( j++ ) = QArrayM(QPointData)::at( i-- );
+	    a.QArray<QPoint>::at( j++ ) = QArray<QPoint>::at( i-- );
 	}
     }
     *this = a;
     return;
+#endif
+}
+
+
+// Based upon:
+//   parelarc.c from Graphics Gems III
+//   VanAken / Simar, "A Parametric Elliptical Arc Algorithm"
+//
+static void
+qtr_elips(QPointArray& a, int& offset, double dxP, double dyP, double dxQ, double dyQ, double dxK, double dyK, int m)
+{
+#define PIV2  102944     /* fixed point PI/2 */
+#define TWOPI 411775     /* fixed point 2*PI */
+#define HALF  32768      /* fixed point 1/2 */
+
+    int xP, yP, xQ, yQ, xK, yK;
+    xP = int(dxP * 65536.0); yP = int(dyP * 65536.0);
+    xQ = int(dxQ * 65536.0); yQ = int(dyQ * 65536.0);
+    xK = int(dxK * 65536.0); yK = int(dyK * 65536.0);
+
+    int i;
+    int vx, ux, vy, uy, xJ, yJ;
+
+    vx = xK - xQ;                 /* displacements from center */
+    ux = xK - xP;
+    vy = yK - yQ;
+    uy = yK - yP;
+    xJ = xP - vx + HALF;          /* center of ellipse J */
+    yJ = yP - vy + HALF;
+
+    int r;
+    ux -= (r = ux >> (2*m + 3));  /* cancel 2nd-order error */
+    ux -= (r >>= (2*m + 4));      /* cancel 4th-order error */
+    ux -= r >> (2*m + 3);         /* cancel 6th-order error */
+    ux += vx >> (m + 1);          /* cancel 1st-order error */
+    uy -= (r = uy >> (2*m + 3));  /* cancel 2nd-order error */
+    uy -= (r >>= (2*m + 4));      /* cancel 4th-order error */
+    uy -= r >> (2*m + 3);         /* cancel 6th-order error */
+    uy += vy >> (m + 1);          /* cancel 1st-order error */
+
+    int n = offset;
+    for (i = (PIV2 << m) >> 16; i >= 0; --i) {
+        a[n++] = QPoint((xJ + vx) >> 16, (yJ + vy) >> 16);
+        ux -= vx >> m;
+        vx += ux >> m;
+        uy -= vy >> m;
+        vy += uy >> m;
+    }
+    offset = n;
+
+#undef PIV2
+#undef TWOPI
+#undef HALF
+}
+
+
+/*!
+  Sets the points of the array to those describing an arc of an
+  ellipse with size \a w by \a h and position (\a x, \a y ), starting
+  from angle \a a1, spanning \a a2, transformed by the matrix \a xf.
+  The resulting array has sufficient resolution for pixel accuracy.
+
+  Angles are specified in 16ths of a degree,
+  i.e. a full circle equals 5760 (16*360). Positive values mean
+  counter-clockwise while negative values mean clockwise direction.
+  Zero degrees is at the 3 o'clock position.
+*/
+void QPointArray::makeArc( int x, int y, int w, int h,
+			       int a1, int a2,
+			       const QWMatrix& xf )
+{
+    if ( --w < 0 || --h < 0 ) {
+	resize( 0 );
+	return;
+    }
+
+    bool rev = a2 < 0;
+    if ( rev ) {
+	a1 += a2;
+	a2 = -a2;
+    }
+    a1 = fix_angle( a1 );
+    if ( a1 < 0 )
+	a1 += 16*360;
+    a2 = fix_angle( a2 );
+
+    bool arc = a1 != 0 || a2 != 360*16 || rev;
+
+    double xP, yP, xQ, yQ, xK, yK;
+
+    xf.map(x+w, y+h/2.0, &xP, &yP);
+    xf.map(x+w/2.0, y, &xQ, &yQ);
+    xf.map(x+w, y, &xK, &yK);
+
+    int m = 2;
+    int max;
+    int q = int(QMAX(QABS(xP-xQ),QABS(yP-yQ)));
+    if ( arc )
+	q *= 2;
+    do {
+	m++;
+	max = 4*(1 + int((Q_PI/2)*(1<<m)));
+    } while (max < q && m < 16); // 16 limits memory usage on HUGE arcs
+    resize(max);
+
+    int n = 0;
+
+    double inc = 1.0/(1<<m);
+
+    int nquad[4];
+    nquad[0]=0;
+
+    qtr_elips(*this, n, xP, yP, xQ, yQ, xK, yK, m);
+    nquad[1] = n;
+
+    xP = xQ; yP = yQ;
+    xf.map(x, y+h/2.0, &xQ, &yQ);
+    xf.map(x, y, &xK, &yK);
+    qtr_elips(*this, n, xP, yP, xQ, yQ, xK, yK, m);
+    nquad[2] = n;
+
+    xP = xQ; yP = yQ;
+    xf.map(x+w/2.0, y+h, &xQ, &yQ);
+    xf.map(x, y+h, &xK, &yK);
+    qtr_elips(*this, n, xP, yP, xQ, yQ, xK, yK, m);
+    nquad[3] = n;
+
+    xP = xQ; yP = yQ;
+    xf.map(x+w, y+h/2.0, &xQ, &yQ);
+    xf.map(x+w, y+h, &xK, &yK);
+    qtr_elips(*this, n, xP, yP, xQ, yQ, xK, yK, m);
+
+    if ( arc ) {
+	// We could merge the sub-ellipse extraction into the above so
+	// that we didn't generate points we don't need, but this is
+	// clearer, and optimizes for the common case.
+	double da1 = double(a1)*Q_PI / (360*8);
+	double da2 = double(a2)*Q_PI / (360*8);
+	int t = 0;
+	while ( da1 > Q_PI/2 ) {
+	    da1 -= Q_PI/2;
+	    t++;
+	}
+
+	int i = nquad[t]+int(da1/inc+0.5);
+	int k = int(da2/inc+0.5);
+	QPointArray r(k);
+	int j = 0;
+
+	if ( rev ) {
+	    while ( k-- )
+		r[j++] = at((i+k)%n);
+	} else {
+	    while ( j < k )
+		r[j++] = at((i+j)%n);
+	}
+	*this = r;
+    } else {
+	resize(n);
+    }
 }
 
 /*!
   Sets the points of the array to those describing an ellipse with
   size \a w by \a h and position (\a x, \a y ).
+
+  The returned array has sufficient
+  resolution for use as pixels (see the overloaded function which
+  takes an additional QWMatrix parameter).
 */
 void QPointArray::makeEllipse( int xx, int yy, int w, int h )
 {						// midpoint, 1/4 ellipse
+    QWMatrix unit;
+    makeArc(xx,yy,w,h,0,360*16,unit);
+    return;
+
+#if QT_OLD_MAKEELLIPSE // ### WWA says discard this.
     if ( w <= 0 || h <= 0 ) {
 	if ( w == 0 || h == 0 ) {
 	    resize( 0 );
@@ -588,6 +687,7 @@ void QPointArray::makeEllipse( int xx, int yy, int w, int h )
     }
     delete[] px;
     delete[] py;
+#endif
 }
 
 
@@ -626,7 +726,7 @@ void split(const double *p, double *l, double *r)
 //   by Alan Paeth
 //   from "Graphics Gems", Academic Press, 1990
 static
-int pnt_on_line( const double* p, const double* q, const double* t )
+int pnt_on_line( const int* p, const int* q, const int* t )
 {
 /*
  * given a line through P:(px,py) Q:(qx,qy) and T:(tx,ty)
@@ -660,6 +760,11 @@ int pnt_on_line( const double* p, const double* q, const double* t )
  * into a larger, spanning vectors within the Lemming editor.
  */
 
+	// if all points are coincident, return condition 2 (on line)
+	if(q[0]==p[0] && q[1]==p[1] && q[0]==t[0] && q[1]==t[1]) {
+		return 2;
+	}
+
     if ( QABS((q[1]-p[1])*(t[0]-p[0])-(t[1]-p[1])*(q[0]-p[0])) >=
         (QMAX(QABS(q[0]-p[0]), QABS(q[1]-p[1])))) return 0;
 
@@ -697,16 +802,34 @@ void polygonizeQBezier( double* acc, int& accsize, const double ctrl[],
     double r[8];
     split( ctrl, l, r);
 
-    if ( pnt_on_line( &ctrl[0], &ctrl[6], &ctrl[2] ) == 2
-      && pnt_on_line( &ctrl[0], &ctrl[6], &ctrl[4] ) == 2 )
+    // convert to integers for line condition check
+    int c0[2]; c0[0] = int(ctrl[0]); c0[1] = int(ctrl[1]);
+    int c1[2]; c1[0] = int(ctrl[2]); c1[1] = int(ctrl[3]);
+    int c2[2]; c2[0] = int(ctrl[4]); c2[1] = int(ctrl[5]);
+    int c3[2]; c3[0] = int(ctrl[6]); c3[1] = int(ctrl[7]);
+
+    if ( QABS(c1[0]-c0[0]) <= 1 && QABS(c1[1]-c0[1]) <= 1
+      && QABS(c2[0]-c0[0]) <= 1 && QABS(c2[1]-c0[1]) <= 1
+      && QABS(c3[0]-c1[0]) <= 1 && QABS(c3[1]-c0[1]) <= 1 )
     {
-	// Approximate by 2 lines.
+	// Approximate by one line.
+	// Dont need to write last pt as it is the same as first pt
+	// on the next segment
+	acc[accsize++] = l[0];
+	acc[accsize++] = l[1];
+	return;
+    }
+
+    if ( ( pnt_on_line( c0, c3, c1 ) == 2 && pnt_on_line( c0, c3, c2 ) == 2 )
+      || ( QABS(c1[0]-c0[0]) <= 1 && QABS(c1[1]-c0[1]) <= 1
+        && QABS(c2[0]-c0[0]) <= 1 && QABS(c2[1]-c0[1]) <= 1
+        && QABS(c3[0]-c1[0]) <= 1 && QABS(c3[1]-c0[1]) <= 1 ) )
+    {
+	// Approximate by one line.
+	// Dont need to write last pt as it is the same as first pt
+	// on the next segment
 	acc[accsize++] = l[0];
 	acc[accsize++] = l[1];
-	acc[accsize++] = l[6];
-	acc[accsize++] = l[7];
-	acc[accsize++] = r[6];
-	acc[accsize++] = r[7];
 	return;
     }
 
@@ -724,17 +847,16 @@ QPointArray QPointArray::quadBezier() const
 #ifdef USE_SIMPLE_QBEZIER_CODE
     if ( size() != 4 ) {
 #if defined(CHECK_RANGE)
-	warning( "QPointArray::bezier: The array must have 4 control points" );
+	qWarning( "QPointArray::bezier: The array must have 4 control points" );
 #endif
 	QPointArray p;
 	return p;
     }
 
     int v;
-    const int n = 3;				// n + 1 control points
     float xvec[4];
     float yvec[4];
-    for ( v=0; v<=n; v++ ) {			// store all x,y in xvec,yvec
+    for ( v=0; v<4; v++ ) {			// store all x,y in xvec,yvec
 	int x, y;
 	point( v, &x, &y );
 	xvec[v] = (float)x;
@@ -759,27 +881,27 @@ QPointArray QPointArray::quadBezier() const
     float ay = yvec[3] - (y0 + cy + by);
     float t = dt;
 
-    pd->x = (Qpnta_t)xvec[0];
-    pd->y = (Qpnta_t)yvec[0];
+    pd->rx() = (QCOORD)xvec[0];
+    pd->ry() = (QCOORD)yvec[0];
     pd++;
     m -= 2;
 
     while ( m-- ) {
-	pd->x = (Qpnta_t)qRound( ((ax * t + bx) * t + cx) * t + x0 );
-	pd->y = (Qpnta_t)qRound( ((ay * t + by) * t + cy) * t + y0 );
+	pd->rx() = (QCOORD)qRound( ((ax * t + bx) * t + cx) * t + x0 );
+	pd->ry() = (QCOORD)qRound( ((ay * t + by) * t + cy) * t + y0 );
 	pd++;
 	t += dt;
     }
 
-    pd->x = (Qpnta_t)xvec[3];
-    pd->y = (Qpnta_t)yvec[3];
+    pd->rx() = (QCOORD)xvec[3];
+    pd->ry() = (QCOORD)yvec[3];
 
     return p;
 #else
 
     if ( size() != 4 ) {
 #if defined(CHECK_RANGE)
-	warning( "QPointArray::bezier: The array must have 4 control points" );
+	qWarning( "QPointArray::bezier: The array must have 4 control points" );
 #endif
 	QPointArray pa;
 	return pa;
@@ -787,7 +909,7 @@ QPointArray QPointArray::quadBezier() const
 	QRect r = boundingRect();
 	int m = 4+2*QMAX(r.width(),r.height());
 	double *p = new double[m];
-	double *ctrl = new double[8];
+	double ctrl[8];
 	int i;
 	for (i=0; i<4; i++) {
 	    ctrl[i*2] = at(i).x();
@@ -795,7 +917,7 @@ QPointArray QPointArray::quadBezier() const
 	}
 	int len=0;
 	polygonizeQBezier( p, len, ctrl, m );
-	QPointArray pa(len/2);
+	QPointArray pa((len/2)+1); // one extra point for last point on line
 	int j=0;
 	for (i=0; j<len; i++) {
 	    // Don't round - it looks terrible
@@ -803,8 +925,9 @@ QPointArray QPointArray::quadBezier() const
 	    int y = int(p[j++]);
 	    pa[i] = QPoint(x,y);
 	}
+	// add last pt on the line, which will be at the last control pt
+	pa[(int)pa.size()-1] = at(3);
 	delete[] p;
-	delete[] ctrl;
 
 	return pa;
     }
@@ -821,11 +944,7 @@ QPointArray QPointArray::quadBezier() const
   \relates QPointArray
   Writes a point array to the stream and returns a reference to the stream.
 
-  The serialization format is:
-  <ol>
-  <li> The array size (UINT32)
-  <li> The array points (QPoint)
-  </ol>
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QPointArray &a )
@@ -841,6 +960,8 @@ QDataStream &operator<<( QDataStream &s, const QPointArray &a )
 /*!
   \relates QPointArray
   Reads a point array from the stream and returns a reference to the stream.
+
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QPointArray &a )
@@ -857,3 +978,61 @@ QDataStream &operator>>( QDataStream &s, QPointArray &a )
     }
     return s;
 }
+
+
+
+
+struct QShortPoint {			// Binary compatible with XPoint
+    short x, y;
+};
+
+uint QPointArray::splen = 0;
+void* QPointArray::sp = 0;		// Really a QShortPoint*
+
+/*!
+  \internal
+
+  Converts the point coords to short (16bit) size, compatible with
+  X11's XPoint structure. The pointer returned points to a static
+  array, so its contents will be overwritten the next time this
+  function is called.
+*/
+
+void* QPointArray::shortPoints( int index, int nPoints ) const
+{
+
+    if ( isNull() || !nPoints )
+	return 0;
+    QPoint* p = data();
+    p += index;
+    uint i = nPoints < 0 ? size() : nPoints;
+    if ( splen < i ) {
+	if ( sp )
+	    delete[] ((QShortPoint*)sp);
+	sp = new QShortPoint[i];
+	splen = i;
+    }
+    QShortPoint* ps = (QShortPoint*)sp;
+    while ( i-- ) {
+	ps->x = (short)p->x();
+	ps->y = (short)p->y();
+	p++;
+	ps++;
+    }
+    return sp;
+}
+
+
+/*!
+  \internal
+
+  Deallocates the internal buffer used by shortPoints().
+*/
+
+void QPointArray::cleanBuffers()
+{
+    if ( sp )
+	delete[] ((QShortPoint*)sp);
+    sp = 0;
+    splen = 0;
+}
