@@ -1,33 +1,35 @@
 /****************************************************************************
-** $Id: qprinter_x11.cpp,v 2.20.2.3 1999/01/12 11:38:34 hanord Exp $
+** $Id: qt/src/kernel/qprinter_x11.cpp   2.1.1   edited 2000-04-06 $
 **
 ** Implementation of QPrinter class for X11
 **
 ** Created : 950810
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qprinter.h"
+#include "qpaintdevicemetrics.h"
+#include "qfile.h"
 #include "qfileinfo.h"
 #include "qdir.h"
-#include "qpaintdevicedefs.h"
-#include "qpsprinter.h"
+#include "qpsprinter_p.h"
 #include "qprintdialog.h"
-#include "qfile.h"
 #include "qapplication.h"
 #include <stdlib.h>
 
@@ -38,8 +40,10 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <signal.h>
 #endif
 
 #if defined(_WS_X11_)
@@ -56,6 +60,7 @@
 #include <process.h>
 #endif
 
+// NOT REVISED
 
 /*****************************************************************************
   QPrinter member functions
@@ -74,26 +79,31 @@
 */
 
 QPrinter::QPrinter()
-    : QPaintDevice( PDT_PRINTER | PDF_EXTDEV )
+    : QPaintDevice( QInternal::Printer | QInternal::ExternalDevice )
 {
     pdrv = 0;
+    pid = 0;
     orient = Portrait;
     page_size = A4;
     ncopies = 1;
     from_pg = to_pg = min_pg = max_pg = 0;
     state = PST_IDLE;
-    printer_name = getenv( "PRINTER" );
     output_file = FALSE;
-    print_prog = "lpr";
+    to_edge	= FALSE;
 }
 
 /*!
-  Destroys the printer paint device and cleans up.
+  Destructs the printer paint device and cleans up.
 */
 
 QPrinter::~QPrinter()
 {
     delete pdrv;
+    if ( pid ) {
+	(void)::kill( pid, 6 );
+	(void)::wait( 0 );
+	pid = 0;
+    }
 }
 
 
@@ -105,7 +115,7 @@ QPrinter::~QPrinter()
 bool QPrinter::newPage()
 {
     if ( state == PST_ACTIVE && pdrv )
-	return ((QPSPrinter*)pdrv)->cmd( PDC_PRT_NEWPAGE, 0, 0 );
+	return ((QPSPrinter*)pdrv)->cmd( QPSPrinter::NewPage, 0, 0 );
     return FALSE;
 }
 
@@ -119,8 +129,13 @@ bool QPrinter::newPage()
 bool QPrinter::abort()
 {
     if ( state == PST_ACTIVE && pdrv ) {
-	((QPSPrinter*)pdrv)->cmd( PDC_PRT_ABORT, 0, 0 );
+	((QPSPrinter*)pdrv)->cmd( QPSPrinter::AbortPrinting, 0, 0 );
 	state = PST_ABORTED;
+	if ( pid ) {
+	    (void)::kill( pid, 6 );
+	    (void)::wait( 0 );
+	    pid = 0;
+	}
     }
     return state == PST_ABORTED;
 }
@@ -140,7 +155,7 @@ bool QPrinter::aborted() const
   Opens a printer setup dialog and asks the user to specify what printer
   to use and miscellaneous printer settings.
 
-  Returns TRUE if the user pressed "Ok" to print, or FALSE if the
+  Returns TRUE if the user pressed "OK" to print, or FALSE if the
   user cancelled the operation.
 */
 
@@ -158,28 +173,30 @@ bool QPrinter::setup( QWidget * parent )
 
 bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 {
-    if ( c ==  PDC_BEGIN ) {
+    if ( c ==  PdcBegin ) {
 	if ( state == PST_IDLE ) {
 	    if ( output_file ) {
+		int fd = 0;
 #if defined(_OS_WIN32_)
-		int fd = open( output_filename.data(),
-				O_CREAT | O_BINARY | O_TRUNC | O_WRONLY );
+		if ( qt_winver & Qt::WV_NT_based )
+		    fd = _topen( qt_winTchar(output_filename,TRUE),
+				_O_CREAT | _O_BINARY | _O_TRUNC | _O_WRONLY );
+		else
+		    fd = _open( output_filename.ascii(),
+				_O_CREAT | _O_BINARY | _O_TRUNC | _O_WRONLY );
 #else
-		int fd = ::open( output_filename.data(),
-				 O_CREAT | O_NOCTTY | O_TRUNC | O_WRONLY,
-				 0666 );
+		fd = ::open( output_filename.local8Bit(),
+			     O_CREAT | O_NOCTTY | O_TRUNC | O_WRONLY,
+			     0666 );
 #endif
 		if ( fd >= 0 ) {
 		    pdrv = new QPSPrinter( this, fd );
 		    state = PST_ACTIVE;
 		}
 	    } else {
-		QString pr = printer_name.copy();
-		if ( pr.isEmpty() )
-		    pr = getenv( "PRINTER" );
-		if ( pr.isEmpty() )
-		    pr = "lp";
-		pr.insert( 0, "-P" );
+		QString pr;
+		if ( printer_name )
+		    pr = printer_name;
 #if defined(_OS_WIN32_)
 		// Not implemented
 		// lpr needs -Sserver argument
@@ -187,11 +204,17 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 		QApplication::flushX();
 		int fds[2];
 		if ( pipe( fds ) != 0 ) {
-		    warning( "QPSPrinter: could not open pipe to print" );
+		    qWarning( "QPSPrinter: could not open pipe to print" );
 		    state = PST_ERROR;
 		    return FALSE;
 		}
 #if 0 && defined(_OS_OS2EMX_)
+		// this code is still not used, and maybe it's not
+		// usable either, any more.  if you want to use it,
+		// you may need to fix it first.
+
+		// old comment:
+
 		// this code is usable but not in use.  spawn() is
 		// preferable to fork()/exec() for very large
 		// programs.  if fork()/exec() is a problem and you
@@ -202,6 +225,7 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 		::close( fds[0] );
 		fcntl(tmp, F_SETFD, FD_CLOEXEC);
 		fcntl(fds[1], F_SETFD, FD_CLOEXEC);
+		pr.prepend( option_string ? option_string : "-P" ); // ###
 		if ( spawnlp(P_NOWAIT,print_prog.data(), print_prog.data(),
 			     pr.data(), output->name(), 0) == -1 ) {
 		    ;			// couldn't exec, ignored
@@ -211,12 +235,18 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 		pdrv = new QPSPrinter( this, fds[1] );
 		state = PST_ACTIVE;
 #else
-		if ( fork() == 0 ) {	// child process
+		pid = fork();
+		if ( pid == 0 ) {	// child process
+		    // if possible, exit quickly, so the actual lp/lpr
+		    // becomes a child of init, and ::waitpid() is
+		    // guaranteed not to wait.
+		    if ( fork() > 0 )
+			exit( 0 );
 		    dup2( fds[0], 0 );
 #if defined(_WS_X11_)
-		    // ###
 		    // hack time... getting the maximum number of open
-		    // files, if possible.  if not we assume it's 256.
+		    // files, if possible.  if not we assume it's the
+		    // larger of 256 and the fd we got
 		    int i;
 #if defined(_OS_OS2EMX_)
 		    LONG req_count = 0;
@@ -231,27 +261,47 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 #elif defined(OPEN_MAX)
 		    i = (int)OPEN_MAX;
 #else
-		    i = 256;
+		    i = QMAX( 256, fds[0] );
 #endif // ways-to-set i
 		    while( --i > 0 )
 			::close( i );
 #endif // _WS_X11_
-		    (void)execlp( print_prog.data(), print_prog.data(),
-				  pr.data(), 0 );
-		    // if execlp returns EACCES it couldn't find the
-		    // program.  if no special print program has been
-		    // set, let's try a little harder...
-		    if ( print_prog == "lpr" && ( errno == EACCES ||
-						  errno == ENOENT ||
-						  errno == ENOEXEC ) ) {
-			(void)execl( "/bin/lpr", "lpr", pr.data(), 0 );
-			(void)execl( "/usr/bin/lpr", "lpr", pr.data(), 0 );
-			pr[1] = 'd';
-			(void)execlp( "lp", "lp", pr.data(), 0 );
-			(void)execl( "/bin/lp", "lp", pr.data(), 0 );
-			(void)execl( "/usr/bin/lp", "lp", pr.data(), 0 );
+		    if ( print_prog ) {
+			pr.prepend( option_string ? option_string :
+				    QString::fromLatin1( "-P" ) );
+			(void)execlp( print_prog.ascii(), print_prog.ascii(),
+				      pr.ascii(), 0 );
+		    } else {
+			// if no print program has been specified, be smart
+			// about the option string too.
+			const char * lprarg = 0;
+			QString lprhack;
+			const char * lparg = 0;
+			QString lphack;
+			if ( pr || option_string ) {
+			    lprhack = pr;
+			    lprhack.prepend( option_string ? option_string :
+					     QString::fromLatin1( "-P" ) );
+			    lprarg = lprhack.ascii();
+			    lphack = pr;
+			    lphack.prepend( option_string ? option_string :
+					    QString::fromLatin1( "-d" ) );
+			    lparg = lphack.ascii();
+			}
+			(void)execlp( "lp", "lp", lparg, 0 );
+			(void)execlp( "lpr", "lpr", lprarg, 0 );
+			(void)execl( "/bin/lp", "lp", lparg, 0 );
+			(void)execl( "/bin/lpr", "lpr", lprarg, 0 );
+			(void)execl( "/usr/bin/lp", "lp", lparg, 0 );
+			(void)execl( "/usr/bin/lpr", "lpr", lprarg, 0 );
 		    }
-		    exit( 0 );
+		    // if we couldn't exec anything, close the fd,
+		    // wait for a second so the parent process (the
+		    // child of the GUI process) has exited.  then
+		    // exit.
+		    ::close( 0 );
+		    (void)::sleep( 1 );
+		    ::exit( 0 );
 		} else {		// parent process
 		    ::close( fds[0] );
 		    pdrv = new QPSPrinter( this, fds[1] );
@@ -260,8 +310,8 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 #endif // else part of _OS_OS2EMX_
 #endif // else part for #if _OS_WIN32_
 	    }
-	    if ( state == PST_ACTIVE && pdrv ) 
-		return  ((QPSPrinter*)pdrv)->cmd( c, paint, p );
+	    if ( state == PST_ACTIVE && pdrv )
+		return ((QPSPrinter*)pdrv)->cmd( c, paint, p );
 	} else {
 	    // ignore it?  I don't know
 	}
@@ -269,10 +319,14 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 	bool r = FALSE;
 	if ( state == PST_ACTIVE && pdrv ) {
 	    r = ((QPSPrinter*)pdrv)->cmd( c, paint, p );
-	    if ( c == PDC_END ) {
+	    if ( c == PdcEnd ) {
 		state = PST_IDLE;
 		delete pdrv;
 		pdrv = 0;
+		if ( pid ) {
+		    (void)::waitpid( pid, 0, 0 );
+		    pid = 0;
+		}
 	    }
 	}
 	return r;
@@ -295,7 +349,7 @@ int QPrinter::metric( int m ) const
     int val;
     PageSize s = pageSize();
 #if defined(CHECK_RANGE)
-    ASSERT( (uint)s <= (uint)Executive );
+    ASSERT( (uint)s < (uint)NPageSize );
 #endif
     static int widths[]	 = { 595, 516, 612, 612, 541,
 			     2384, 1684, 1191, 842, 420, 297, 210, 148, 105,
@@ -307,31 +361,61 @@ int QPrinter::metric( int m ) const
 			     4127, 2920, 127, 2064, 1460, 1032, 729, 516, 363,
 			     258, 181, 648, 684, 624, 935, 792, 1224 };
     switch ( m ) {
-	case PDM_WIDTH:
+	case QPaintDeviceMetrics::PdmWidth:
 	    val = orient == Portrait ? widths[ s ] : heights[ s ];
+	    if ( !fullPage() )
+		val -= 2*margins().width();
 	    break;
-	case PDM_HEIGHT:
+	case QPaintDeviceMetrics::PdmHeight:
 	    val = orient == Portrait ? heights[ s ] : widths[ s ];
+	    if ( !fullPage() )
+		val -= 2*margins().height();
 	    break;
-	case PDM_WIDTHMM:
-	    val = orient == Portrait ? widths[ s ] : heights[ s ];
+	case QPaintDeviceMetrics::PdmDpiX:
+	    val = 72;
+	    break;
+	case QPaintDeviceMetrics::PdmDpiY:
+	    val = 72;
+	    break;
+	case QPaintDeviceMetrics::PdmWidthMM:
+	    val = metric( QPaintDeviceMetrics::PdmWidth );
 	    val = (val * 254 + 360) / 720; // +360 to get the right rounding
 	    break;
-	case PDM_HEIGHTMM:
-	    val = orient == Portrait ? heights[ s ] : widths[ s ];
+	case QPaintDeviceMetrics::PdmHeightMM:
+	    val = metric( QPaintDeviceMetrics::PdmHeight );
 	    val = (val * 254 + 360) / 720;
 	    break;
-	case PDM_NUMCOLORS:
+	case QPaintDeviceMetrics::PdmNumColors:
 	    val = 16777216;
 	    break;
-	case PDM_DEPTH:
+	case QPaintDeviceMetrics::PdmDepth:
 	    val = 24;
 	    break;
 	default:
 	    val = 0;
 #if defined(CHECK_RANGE)
-	    warning( "QPixmap::metric: Invalid metric command" );
+	    qWarning( "QPixmap::metric: Invalid metric command" );
 #endif
     }
     return val;
 }
+
+
+/*!  Returns the width of the left/right and top/bottom margins of the
+printer.  This is a best-effort guess, not based on perfect knowledge.
+
+If you have called setFullPage( TRUE ) (this is recommended for
+high-quality printing), margins().width() may be treated as the
+smallest sane left/right margin you can use, and margins().height() as
+the smallest sane top/bottom margins you can use.
+
+If you have called setFullPage( FALSE ) (this is the default),
+margins() is automatically subtracted from the pageSize() by QPrinter.
+
+\sa setFullPage() QPaintDeviceMetrics PageSize
+*/
+
+QSize QPrinter::margins() const
+{
+    return (orient == Portrait) ? QSize( 36, 22 ) : QSize( 22, 36 );
+}
