@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qsignalmapper.cpp,v 1.5.2.2 1999/01/13 19:11:42 ettrich Exp $
+** $Id: qt/src/kernel/qsignalmapper.cpp   2.1.1   edited 2000-03-24 $
 **
 ** Implementation of QSignalMapper class
 **
 ** Created : 980503
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -28,13 +30,13 @@ struct QSignalMapperRec {
     QSignalMapperRec()
     {
 	has_int = 0;
-	str_id = 0;
+	str_id = QString::null;
     }
 
     uint has_int:1;
 
     int int_id;
-    const char* str_id;
+    QString str_id;
     // extendable to other types of identification
 };
 
@@ -48,6 +50,7 @@ public:
     QPtrDict<QSignalMapperRec> dict;
 };
 
+// NOT REVISED
 /*!
   \class QSignalMapper qsignalmapper.h
   \brief A QSignalMapper object bundles signals from identifiable senders.
@@ -77,7 +80,7 @@ QSignalMapper::~QSignalMapper()
 
 /*!
   Adds a mapping such that when map() is signalled from the given
-  sender, the signal mapper(identifier) is emitted.
+  sender, the signal mapped(identifier) is emitted.
 
   There may be at most one integer identifier for each object.
 */
@@ -90,12 +93,12 @@ void QSignalMapper::setMapping( const QObject* sender, int identifier )
 
 /*!
   Adds a mapping such that when map() is signalled from the given
-  sender, the signal mapped(identifier) is emitted.
+  sender, the signal mapper(identifier) is emitted.
 
   There may be at most one string identifier for each object, and
   it may not be null.
 */
-void QSignalMapper::setMapping( const QObject* sender, const char* identifier )
+void QSignalMapper::setMapping( const QObject* sender, const QString &identifier )
 {
     QSignalMapperRec* rec = getRec(sender);
     rec->str_id = identifier;
@@ -126,7 +129,7 @@ void QSignalMapper::map()
     if ( rec ) {
 	if ( rec->has_int )
 	    emit mapped( rec->int_id );
-	if ( rec->str_id )
+	if ( !rec->str_id.isEmpty() )
 	    emit mapped( rec->str_id );
     }
 }
@@ -152,11 +155,11 @@ QSignalMapperRec* QSignalMapper::getRec( const QObject* sender )
 */
 
 /*!
-  \fn void QSignalMapper::mapped(const char*)
+  \fn void QSignalMapper::mapped(const QString&)
 
 
   This signal is emitted when map() is signalled from an object which
   has a string mapping set.
 
-  \sa setMapping(const char*)
+  \sa setMapping(QString)
 */
