@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qbitarray.cpp,v 2.8.2.1 1998/10/19 12:08:19 eiriken Exp $
+** $Id: qt/src/tools/qbitarray.cpp   2.1.1   edited 2000-05-22 $
 **
 ** Implementation of QBitArray class
 **
 ** Created : 940118
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -27,6 +29,7 @@
 #define SHBLOCK	 ((bitarr_data*)(sharedBlock()))
 
 
+// NOT REVISED
 /*!
   \class QBitVal qbitarray.h
   \brief The QBitVal class is an internal class, used with QBitArray.
@@ -36,27 +39,27 @@
 */
 
 /*!
-  \fn QBitVal::QBitVal (QBitArray* a, uint i) 
+  \fn QBitVal::QBitVal (QBitArray* a, uint i)
 
   Constructs a reference to an element in a QBitArray.  This is
   what QBitArray::operator[] contructs its return value with.
 */
 
 /*!
-  \fn QBitVal::operator int() 
+  \fn QBitVal::operator int()
 
   Returns the value referenced by the QBitVal.
 */
 
 /*!
-  \fn QBitVal& QBitVal::operator= (const QBitVal& v) 
+  \fn QBitVal& QBitVal::operator= (const QBitVal& v)
 
   Sets the value referenced by the QBitVal to that referenced by another
   QBitVal.
 */
 
 /*!
-  \fn QBitVal& QBitVal::operator= (int v) 
+  \fn QBitVal& QBitVal::operator= (bool v)
 
   Sets the value referenced by the QBitVal.
 */
@@ -245,7 +248,7 @@ bool QBitArray::testBit( uint index ) const
 {
 #if defined(CHECK_RANGE)
     if ( index >= size() ) {
-	warning( "QBitArray::testBit: Index %d out of range", index );
+	qWarning( "QBitArray::testBit: Index %d out of range", index );
 	return FALSE;
     }
 #endif
@@ -261,7 +264,7 @@ void QBitArray::setBit( uint index )
 {
 #if defined(CHECK_RANGE)
     if ( index >= size() ) {
-	warning( "QBitArray::setBit: Index %d out of range", index );
+	qWarning( "QBitArray::setBit: Index %d out of range", index );
 	return;
     }
 #endif
@@ -292,7 +295,7 @@ void QBitArray::clearBit( uint index )
 {
 #if defined(CHECK_RANGE)
     if ( index >= size() ) {
-	warning( "QBitArray::clearBit: Index %d out of range", index );
+	qWarning( "QBitArray::clearBit: Index %d out of range", index );
 	return;
     }
 #endif
@@ -312,7 +315,7 @@ bool QBitArray::toggleBit( uint index )
 {
 #if defined(CHECK_RANGE)
     if ( index >= size() ) {
-	warning( "QBitArray::toggleBit: Index %d out of range", index );
+	qWarning( "QBitArray::toggleBit: Index %d out of range", index );
 	return FALSE;
     }
 #endif
@@ -327,7 +330,7 @@ bool QBitArray::toggleBit( uint index )
 /*!
   \fn bool QBitArray::at( uint index ) const
   Returns the value (0 or 1) of the bit at position \e index.
-  \sa operator[]
+  \sa operator[]()
 */
 
 /*!
@@ -381,7 +384,7 @@ QBitArray &QBitArray::operator&=( const QBitArray &a )
     }
 #if defined(CHECK_RANGE)
     else
-	warning( "QBitArray::operator&=: Bit arrays have different size" );
+	qWarning( "QBitArray::operator&=: Bit arrays have different size" );
 #endif
     return *this;
 }
@@ -416,7 +419,7 @@ QBitArray &QBitArray::operator|=( const QBitArray &a )
     }
 #if defined(CHECK_RANGE)
     else
-	warning( "QBitArray::operator|=: Bit arrays have different size" );
+	qWarning( "QBitArray::operator|=: Bit arrays have different size" );
 #endif
     return *this;
 }
@@ -451,7 +454,7 @@ QBitArray &QBitArray::operator^=( const QBitArray &a )
     }
 #if defined(CHECK_RANGE)
     else
-	warning( "QBitArray::operator^=: Bit arrays have different size" );
+	qWarning( "QBitArray::operator^=: Bit arrays have different size" );
 #endif
     return *this;
 }
@@ -520,6 +523,33 @@ QBitArray operator^( const QBitArray &a1, const QBitArray &a2 )
 }
 
 
+/*! \enum QBitArray::array_data
+
+  \warning This will be renamed in the next major release of Qt.  Until
+  then it is undocumented and we recommend against its use.
+
+  \internal
+
+  ### 3.0 rename ###
+*/
+
+
+/*! \fn QBitArray::array_data * QBitArray::newData()
+
+  Returns data specific to QBitArray that extends what QGArray provides.
+  \internal
+  QCollection mechanism for allowing extra/different data.
+*/
+
+
+/*! \fn void  QBitArray::deleteData ( array_data * d)
+
+  Deletes data specific to QBitArray that extended what QGArray provided.
+  \internal
+  QCollection mechanism for allowing extra/different data.
+*/
+
+
 /*****************************************************************************
   QBitArray stream functions
  *****************************************************************************/
@@ -528,16 +558,12 @@ QBitArray operator^( const QBitArray &a1, const QBitArray &a2 )
   \relates QBitArray
   Writes a bit array to a stream.
 
-  Serialization format:
-  <ol>
-  <li> The array size (UINT32)
-  <li> The array bits, (size+7)/8 bytes
-  </ol>
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QBitArray &a )
 {
-    UINT32 len = a.size();
+    Q_UINT32 len = a.size();
     s << len;					// write size of array
     if ( len > 0 )				// write data
 	s.writeRawBytes( a.data(), a.QByteArray::size() );
@@ -547,15 +573,17 @@ QDataStream &operator<<( QDataStream &s, const QBitArray &a )
 /*!
   \relates QBitArray
   Reads a bit array from a stream.
+
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QBitArray &a )
 {
-    UINT32 len;
+    Q_UINT32 len;
     s >> len;					// read size of array
     if ( !a.resize( (uint)len ) ) {		// resize array
 #if defined(CHECK_NULL)
-	warning( "QDataStream: Not enough memory to read QBitArray" );
+	qWarning( "QDataStream: Not enough memory to read QBitArray" );
 #endif
 	len = 0;
     }
