@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qdatastream.cpp,v 2.15 1998/07/03 00:09:43 hanord Exp $
+** $Id: qt/src/tools/qdatastream.cpp   2.1.1   edited 2000-05-22 $
 **
 ** Implementation of QDataStream class
 **
 ** Created : 930831
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -27,17 +29,18 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+// REVISED: warwick
 /*!
   \class QDataStream qdatastream.h
 
-  \brief The QDataStream class provides basic functions for serialization of
+  \brief The QDataStream class provides serialization of
   binary data to a QIODevice.
 
   \ingroup io
 
   A data stream is a binary stream of encoded information which is 100%
   independent of the host computer operation system, CPU or byte order.	 A
-  stream that is written by a PC under DOS/Windows can easily be read by a
+  stream that is written by a PC under DOS/Windows can be read by a
   Sun SPARC running Solaris.
 
   The QDataStream class implements serialization of primitive types, like
@@ -46,8 +49,9 @@
 
   The programmer can select which byte order to use when serializing data.
   The default setting is big endian (MSB first). Changing it to little
-  endian breaks the portability.  We therefore recommend keeping this
-  setting unless you have special needs or requirements.
+  endian breaks the portability (unless the reader also changes to little
+  endian).  We recommend keeping this setting unless you have
+  special requirements.
 
   A data stream cooperates closely with a QIODevice. A QIODevice
   represents an input/output medium one can read data from and write data
@@ -60,7 +64,6 @@
     QDataStream s( &f );			// serialize using f
     s << "the answer is";			// serialize string
     s << (Q_INT32)42;				// serialize integer
-    f.close();					// done
   \endcode
 
   Example (read data from a stream):
@@ -71,14 +74,80 @@
     char   *str;
     Q_INT32 a;
     s >> str >> a;				// "the answer is" and 42
-    f.close();					// done
     delete str;					// delete string
   \endcode
 
   In the last example, if you read into a QString instead of a \c char*
   you do not have to delete it.
 
-  \sa QTextStream
+  Normally, each item written to the stream is written in a fixed binary
+  format.
+  For example, a \c char* is written as a 32-bit integer equal to the
+  length of the string including the NUL byte, followed by all the
+  characters of the string including the NUL byte. Similarly when
+  reading a string, 4 bytes are read to create the 32-bit length value,
+  then that many characters for the string including the NUL. For a complete
+  description of all Qt types supporting data streaming see \link
+  datastreamformat.html Format of the QDataStream operators \endlink .
+
+  If you want a "parsing" input stream, see QTextStream. If you just want the
+  data to be human-readable to aid in debugging, you can set the data
+  stream into printable data mode with setPrintableData(). The data is
+  then written slower, in a human readable bloated form that is sufficient
+  for debugging.
+
+  If you are producing a new binary data format, such as a file format
+  for documents created by your application, you could use a QDataStream
+  to write the data in a portable format. Typically, you would write
+  a brief header containing a magic string and a version number to give
+  yourself room for future expansion. For example:
+
+  \code
+    // Open the file.
+    QFile f( "file.xxx" );
+    f.open( IO_WriteOnly );
+    QDataStream s( &f );
+
+    // Write a header with a "magic number" and a version
+    s << 0xa0b0c0d0;
+    s << 123;
+
+    // Write the data
+    s << [lots of interesting data]
+  \endcode
+
+  Then read it in with:
+
+  \code
+    // Open the file.
+    QFile f( "file.xxx" );
+    f.open( IO_ReadOnly );
+    QDataStream s( &f );
+
+    // Read and check the header
+    Q_UINT32 magic;
+    s >> magic;
+    if ( magic != 0xa0b0c0d0 )
+	return XXX_BAD_FILE_FORMAT;
+
+    // Read the version
+    Q_INT32 version;
+    s >> version;
+    if ( version < 100 )
+	return XXX_BAD_FILE_TOO_OLD;
+    if ( version > 123 )
+	return XXX_BAD_FILE_TOO_NEW;
+    if ( version <= 110 )
+	s.setVersion(1);
+
+    // Read the data
+    s >> [lots of interesting data];
+    if ( version > 120 )
+	s >> [data new in XXX version 1.2];
+    s >> [other interesting data];
+  \endcode
+
+  \sa QTextStream QVariant
 */
 
 
@@ -89,7 +158,7 @@
 #if defined(CHECK_STATE)
 #undef  CHECK_STREAM_PRECOND
 #define CHECK_STREAM_PRECOND  if ( !dev ) {				\
-				warning( "QDataStream: No device" );	\
+				qWarning( "QDataStream: No device" );	\
 				return *this; }
 #else
 #define CHECK_STREAM_PRECOND
@@ -98,9 +167,15 @@
 static int  systemWordSize = 0;
 static bool systemBigEndian;
 
+static const int DefaultStreamVersion = 3;
+// 3 is default in Qt 2.1
+// 2 is the Qt 2.0.x format
+// 1 is the Qt 1.x format
 
 /*!
   Constructs a data stream that has no IO device.
+
+  \sa setDevice()
 */
 
 QDataStream::QDataStream()
@@ -111,11 +186,14 @@ QDataStream::QDataStream()
     owndev    = FALSE;
     byteorder = BigEndian;			// default byte order
     printable = FALSE;
+    ver	      = DefaultStreamVersion;
     noswap    = systemBigEndian;
 }
 
 /*!
-  Constructs a data stream that uses the IO device \e d.
+  Constructs a data stream that uses the IO device \a d.
+
+  \sa setDevice(), device()
 */
 
 QDataStream::QDataStream( QIODevice *d )
@@ -126,11 +204,12 @@ QDataStream::QDataStream( QIODevice *d )
     owndev    = FALSE;
     byteorder = BigEndian;			// default byte order
     printable = FALSE;
+    ver	      = DefaultStreamVersion;
     noswap    = systemBigEndian;
 }
 
 /*!
-  Constructs a data stream that operates on a byte array throught an
+  Constructs a data stream that operates on a byte array through an
   internal QBuffer device.
 
   Example:
@@ -139,7 +218,7 @@ QDataStream::QDataStream( QIODevice *d )
     QByteArray	a;
     a.setRawData( bindata, sizeof(bindata) );	// a points to bindata
     QDataStream s( a, IO_ReadOnly );		// open on a's data
-    s >> <something>;				// read raw bindata
+    s >> [something];				// read raw bindata
     a.resetRawData( bindata, sizeof(bindata) ); // finished
   \endcode
 
@@ -155,11 +234,12 @@ QDataStream::QDataStream( QByteArray a, int mode )
     owndev    = TRUE;
     byteorder = BigEndian;			// default byte order
     printable = FALSE;
+    ver	      = DefaultStreamVersion;
     noswap    = systemBigEndian;
 }
 
 /*!
-  Destroys the data stream.
+  Destructs the data stream.
 
   The destructor will not affect the current IO device, unless it
   is an internal IO device processing a QByteArray passed in the constructor.
@@ -180,7 +260,7 @@ QDataStream::~QDataStream()
 
 /*!
   void QDataStream::setDevice(QIODevice *d )
-  Sets the IO device to \e d.
+  Sets the IO device to \a d.
   \sa device(), unsetDevice()
 */
 
@@ -205,7 +285,20 @@ void QDataStream::unsetDevice()
 
 
 /*!
-  \fn bool QDataStream::eof() const
+  \fn bool QDataStream::atEnd() const
+  Returns TRUE if the IO device has reached the end position (end of
+  stream or file) or if there is no IO device set.
+
+  Returns FALSE if the current position of the read/write head of the IO
+  device is somewhere before the end position.
+
+  \sa QIODevice::atEnd()
+*/
+
+/*!\fn bool QDataStream::eof() const
+
+  \obsolete
+
   Returns TRUE if the IO device has reached the end position (end of
   stream or file) or if there is no IO device set.
 
@@ -217,14 +310,16 @@ void QDataStream::unsetDevice()
 
 /*!
   \fn int QDataStream::byteOrder() const
-  Returns the current byte order setting.
+  Returns the current byte order setting - either \c BigEndian or
+  \c LittleEndian.
+
   \sa setByteOrder()
 */
 
 /*!
-  Sets the serialization byte order to \e bo.
+  Sets the serialization byte order to \a bo.
 
-  The \e bo parameter can be \c QDataStream::BigEndian or
+  The \a bo parameter can be \c QDataStream::BigEndian or
   \c QDataStream::LittleEndian.
 
   The default setting is big endian.  We recommend leaving this setting unless
@@ -257,10 +352,36 @@ void QDataStream::setByteOrder( int bo )
   consists of printable characters (7 bit ASCII).
 
   We recommend enabling printable data only for debugging purposes
-  (it is slower and creates bigger output).
+  (it is slower and creates larger output).
 */
 
 
+/*!
+  \fn int QDataStream::version() const
+  Returns the version number of the data serialization format.
+  In Qt 2.1, this number is by default 3.
+  \sa setVersion()
+*/
+
+/*!
+  \fn void QDataStream::setVersion( int v )
+  Sets the version number of the data serialization format.
+
+  In order to accomodate for new functionality, the datastream
+  serialization format of some Qt classes has changed in some versions of
+  Qt. If you want to read data that was created by an earlier version of
+  Qt, or write data that can be read by a program that was compiled with
+  an earlier version of Qt, use this function to modify the serialization
+  format of QDataStream.
+
+  For Qt 1.x compatibility, use \a v == 1.
+
+  For Qt 2.0.x compatibility, use \a v == 2 (Not required for reading in
+  Qt 2.1).
+
+  \sa version()
+*/
+
 /*****************************************************************************
   QDataStream read functions
  *****************************************************************************/
@@ -438,11 +559,11 @@ QDataStream &QDataStream::operator>>( double &f )
 
 
 /*!
-  Reads the '\0'-terminated string \e s from the stream and returns
+  Reads the '\0'-terminated string \a s from the stream and returns
   a reference to the stream.
 
-  The string is read using readBytes(), which allocates space using \c
-  new.
+  Space for the string is allocated using \c new - the caller must
+  eventually call delete[] on the value.
 */
 
 QDataStream &QDataStream::operator>>( char *&s )
@@ -453,17 +574,17 @@ QDataStream &QDataStream::operator>>( char *&s )
 
 
 /*!
-  Reads the buffer \e s from the stream and returns a reference to the
+  Reads the buffer \a s from the stream and returns a reference to the
   stream.
 
-  The buffer \e s is allocated using \c new. Destroy it with the \c delete
-  operator.  If the length is zero or \e s cannot be allocated, \e s is
+  The buffer \a s is allocated using \c new. Destroy it with the \c delete[]
+  operator.  If the length is zero or \a s cannot be allocated, \a s is
   set to 0.
 
-  The \e l parameter will be set to the length of the buffer.
+  The \a l parameter will be set to the length of the buffer.
 
   The serialization format is an Q_UINT32 length specifier first, then the
-  data (\e length bytes).
+  data (\a l bytes).
 
   \sa readRawBytes(), writeBytes()
 */
@@ -488,10 +609,10 @@ QDataStream &QDataStream::readBytes( char *&s, uint &l )
 
 
 /*!
-  Reads \e len bytes from the stream into \e e s and returns a reference to
+  Reads \a len bytes from the stream into \a s and returns a reference to
   the stream.
 
-  The buffer \e s must be preallocated.
+  The buffer \a s must be preallocated.
 
   \sa readBytes(), QIODevice::readBlock(), writeRawBytes()
 */
@@ -500,7 +621,7 @@ QDataStream &QDataStream::readRawBytes( char *s, uint len )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
-	register char *p = s;
+	register Q_INT8 *p = (Q_INT8*)s;
 	while ( len-- )
 	    *this >> *p++;
     } else {					// read data char array
@@ -683,7 +804,7 @@ QDataStream &QDataStream::operator<<( double f )
 
 
 /*!
-  Writes the '\0'-terminated string \e s to the stream and returns
+  Writes the '\0'-terminated string \a s to the stream and returns
   a reference to the stream.
 
   The string is serialized using writeBytes().
@@ -702,11 +823,11 @@ QDataStream &QDataStream::operator<<( const char *s )
 
 
 /*!
-  Writes the length specifier \e len and the buffer \e s to the stream and
+  Writes the length specifier \a len and the buffer \a s to the stream and
   returns a reference to the stream.
 
-  The \e len is serialized as an Q_UINT32, followed by \e len bytes from
-  \e s.
+  The \a len is serialized as an Q_UINT32, followed by \a len bytes from
+  \a s.
 
   \sa writeRawBytes(), readBytes()
 */
@@ -722,7 +843,7 @@ QDataStream &QDataStream::writeBytes(const char *s, uint len)
 
 
 /*!
-  Writes \e len bytes from \e s to the stream and returns a reference to the
+  Writes \a len bytes from \a s to the stream and returns a reference to the
   stream.
 
   \sa writeBytes(), QIODevice::writeBlock(), readRawBytes()
