@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qdatetime.cpp,v 2.16.2.2 1999/02/15 10:56:45 hanord Exp $
+** $Id: qt/src/tools/qdatetime.cpp   2.1.1   edited 2000-05-22 $
 **
 ** Implementation of date and time classes
 **
 ** Created : 940124
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -45,7 +47,7 @@
 extern "C" int gettimeofday( struct timeval *, struct timezone * );
 #endif
 
-static const uint FIRST_DAY	= 2361222;	// Julian day for 17520914
+static const uint FIRST_DAY	= 2361222;	// Julian day for 1752/09/14
 static const int  FIRST_YEAR	= 1752;		// ### wrong for many countries
 static const uint SECS_PER_DAY	= 86400;
 static const uint MSECS_PER_DAY = 86400000;
@@ -56,6 +58,8 @@ static const uint MSECS_PER_MIN = 60000;
 
 static short monthDays[] ={0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
+// ##### Localize.
+
 const char *QDate::monthNames[] = {
     "Jan", "Feb", "Mar", "Apr", "May", "Jun",
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
@@ -68,46 +72,93 @@ const char *QDate::weekdayNames[] ={
   QDate member functions
  *****************************************************************************/
 
+// REVISED: aavit
+
 /*!
   \class QDate qdatetime.h
   \brief The QDate class provides date functions.
 
   \ingroup time
 
-  The QDate is based on the Gregorian (modern western) calendar. England
-  adopted the Gregorian calendar on September 14th 1752, which is the
-  earliest date that is supported by QDate.  Using earlier dates will give
-  undefined results. Some countries adopted the Gregorian calendar later
-  than England, thus the week day of early dates might be incorrect for
-  these countries (but correct for England).  The end of time is reached
-  around 8000AD, by which time we expect Qt to be obsolete.
+  A QDate object contains a calendar date, i.e. year, month, and day
+  numbers in the modern western (Gregorian) calendar. It can read the
+  current date from the system clock. It provides functions for
+  comparing dates and for manipulating a date by adding a number of
+  days.
+
+  A QDate object is typically created either by giving the year, month
+  and day numbers explicitly, or by using the static function
+  currentDate(), which makes a QDate object which contains the
+  system's clock date. An explicit date can also be set using
+  setYMD().
+
+  The year(), month(), and day() functions provide access to the year,
+  month, and day numbers. Also, dayOfWeek() and dayOfYear() functions
+  are provided. The same information is provided in textual format by
+  the toString(), dayName(), and monthName() functions.
+
+  QDate provides a full set of operators to compare two QDate
+  objects. A date is considered smaller than another if it is earlier
+  than the other.
+
+  The date a given number of days later than a given date can be found
+  using the addDays() function. Correspondingly, the number of days
+  between two dates can be found using the daysTo() function.
+
+  The daysInMonth() and daysInYear() functions tell how many days
+  there are in this date's month and year, respectively. The
+  isLeapYear() function tells whether this date is in a leap year.
+
+  Note that QDate may not be used for date calculations for dates in
+  the remote past, i.e. prior to the introduction of the Gregorian
+  calendar. This calendar was adopted by England Sep. 14. 1752 (hence
+  this is the earliest valid QDate), and subsequently by most other
+  western countries, until 1923.
+
+  The end of time is reached around 8000AD, by which time we expect Qt
+  to be obsolete.
 
   \sa QTime, QDateTime
 */
 
-/*!
-  \fn QDate::QDate()
-  Constructs a null date.  Null dates are invalid.
-*/
 
 /*!
-  Constructs a date with the year \e y, month \e m and day \e d.
+  \fn QDate::QDate()
+  Constructs a null date. Null dates are invalid.
+
+  \sa isNull(), isValid()
+*/
+
+
+/*!
+  Constructs a date with the year \a y, month \a m and day \a d.
+
+  \a y must be in the range 1752-ca. 8000, \a m must be in the range
+  1-12, and \a d must be in the range 1-31. Exception: if \a y is in
+  the range 0-99, it is interpreted as 1900-1999.
+
+  \sa isValid()
 */
 
 QDate::QDate( int y, int m, int d )
 {
+    jd = 0;
     setYMD( y, m, d );
 }
 
 
 /*!
   \fn bool QDate::isNull() const
+
   Returns TRUE if the date is null.  A null date is invalid.
+
   \sa isValid()
 */
 
+
 /*!
-  Returns TRUE if the date is valid.
+  Returns TRUE if this date is valid.
+
   \sa isNull()
 */
 
@@ -118,7 +169,8 @@ bool QDate::isValid() const
 
 
 /*!
-  Returns the year (>= 1752) for this date.
+  Returns the year (>= 1752) of this date.
+
   \sa month(), day()
 */
 
@@ -130,7 +182,8 @@ int QDate::year() const
 }
 
 /*!
-  Returns the month (January=1 .. December=12) for this date.
+  Returns the month (January=1 .. December=12) of this date.
+
   \sa year(), day()
 */
 
@@ -142,7 +195,8 @@ int QDate::month() const
 }
 
 /*!
-  Returns the day of the month (1..31) for this date.
+  Returns the day of the month (1..31) of this date.
+
   \sa year(), month(), dayOfWeek()
 */
 
@@ -155,7 +209,8 @@ int QDate::day() const
 
 /*!
   Returns the weekday (Monday=1 .. Sunday=7) for this date.
-  \sa day()
+
+  \sa day(), dayOfYear()
 */
 
 int QDate::dayOfWeek() const
@@ -165,6 +220,8 @@ int QDate::dayOfWeek() const
 
 /*!
   Returns the day of the year (1..365) for this date.
+
+  \sa day(), dayOfWeek()
 */
 
 int QDate::dayOfYear() const
@@ -174,6 +231,8 @@ int QDate::dayOfYear() const
 
 /*!
   Returns the number of days in the month (28..31) for this date.
+
+  \sa day(), daysInYear()
 */
 
 int QDate::daysInMonth() const
@@ -188,6 +247,8 @@ int QDate::daysInMonth() const
 
 /*!
   Returns the number of days in the year (365 or 366) for this date.
+
+  \sa day(), daysInMonth()
 */
 
 int QDate::daysInYear() const
@@ -199,52 +260,76 @@ int QDate::daysInYear() const
 
 
 /*!
-  Returns the name of the \e month.
+  Returns the name of the \a month.
 
   Month 1 == "Jan", month 2 == "Feb" etc.
+
+  \sa toString(), dayName()
 */
 
-const char *QDate::monthName( int month) const
+QString QDate::monthName( int month ) const
 {
-#if defined(DEBUG)
-    ASSERT( month > 0 && month <= 12 );
+#if defined(CHECK_RANGE)
+    if ( month < 1 || month > 12 ) {
+	qWarning( "QDate::monthName: Parameter out ouf range." );
+	month = 1;
+    }
 #endif
-    return monthNames[month-1];
+    // ### Remove the fromLatin1 during localization
+    return QString::fromLatin1(monthNames[month-1]);
 }
 
 /*!
-  Returns the name of the \e weekday.
+  Returns the name of the \a weekday.
 
   Weekday 1 == "Mon", day 2 == "Tue" etc.
+
+  \sa toString(), monthName()
 */
 
-const char *QDate::dayName( int weekday) const
+QString QDate::dayName( int weekday ) const
 {
-#if defined(DEBUG)
-    ASSERT( weekday > 0 && weekday <= 7 );
+#if defined(CHECK_RANGE)
+    if ( weekday < 1 || weekday > 7 ) {
+	qWarning( "QDate::dayName: Parameter out of range." );
+	weekday = 1;
+    }
 #endif
-    return weekdayNames[weekday-1];
+    // ### Remove the fromLatin1 during localization
+    return QString::fromLatin1(weekdayNames[weekday-1]);
 }
 
 
 /*!
   Returns the date as a string.
 
-  The string format is "Sat May 20 1995".
+  The string format is "Sat May 20 1995". This function uses the
+  dayName() and monthName() functions to generate the string.
+
+  \sa dayName(), monthName()
 */
 
 QString QDate::toString() const
 {
-    QString buf;
     int y, m, d;
     jul2greg( jd, y, m, d );
-    buf.sprintf( "%s %s %d %d", dayName(dayOfWeek()), monthName(m), d, y);
+    QString buf = dayName(dayOfWeek());
+    buf += ' ';
+    buf += monthName(m);
+    QString t;
+    t.sprintf( " %d %d", d, y);
+    buf += t;
     return buf;
 }
 
 
 /*!
-  Sets the year \e y, month \e m and day \e d.
+  Sets the year \a y, month \a m and day \a d.
+
+  \a y must be in the range 1752-ca. 8000, \a m must be in the range
+  1-12, and \a d must be in the range 1-31. Exception: if \a y is in
+  the range 0-99, it is interpreted as 1900-1999.
+
   Returns TRUE if the date is valid, otherwise FALSE.
 */
 
@@ -252,7 +337,7 @@ bool QDate::setYMD( int y, int m, int d )
 {
     if ( !isValid(y,m,d) ) {
 #if defined(CHECK_RANGE)
-	 warning( "QDate::setYMD: Invalid date %04d/%02d/%02d", y, m, d );
+	 qWarning( "QDate::setYMD: Invalid date %04d/%02d/%02d", y, m, d );
 #endif
 	 return FALSE;
     }
@@ -264,7 +349,10 @@ bool QDate::setYMD( int y, int m, int d )
 }
 
 /*!
-  Returns this date plus \e ndays days.
+  Returns a QDate object containing a date \a ndays later than the
+  date of this object (or earlier if \a ndays is negative).
+
+  \sa daysTo()
 */
 
 QDate QDate::addDays( int ndays ) const
@@ -275,8 +363,8 @@ QDate QDate::addDays( int ndays ) const
 }
 
 /*!
-  Returns the number of days from this date to \a d, which is negative
-  if \a d is in the past.
+  Returns the number of days from this date to \a d (which is negative
+  if \a d is earlier than this date).
 
   Example:
   \code
@@ -285,6 +373,7 @@ QDate QDate::addDays( int ndays ) const
     d1.daysTo( d2 );			// returns 3
     d2.daysTo( d1 );			// returns -3
   \endcode
+
   \sa addDays()
 */
 
@@ -296,39 +385,40 @@ int QDate::daysTo( const QDate &d ) const
 
 /*!
   \fn bool QDate::operator==( const QDate &d ) const
-  Returns TRUE if this date is equal to \e d, or FALSE if
+  Returns TRUE if this date is equal to \a d, or FALSE if
   they are different.
 */
 
 /*!
   \fn bool QDate::operator!=( const QDate &d ) const
-  Returns TRUE if this date is different from \e d, or FALSE if
+  Returns TRUE if this date is different from \a d, or FALSE if
   they are equal.
 */
 
 /*!
   \fn bool QDate::operator<( const QDate &d ) const
-  Returns TRUE if this date is before \e d, otherwise FALSE.
+  Returns TRUE if this date is earlier than \a d, otherwise FALSE.
 */
 
 /*!
   \fn bool QDate::operator<=( const QDate &d ) const
-  Returns TRUE if this date is before or equal to \e d, otherwise FALSE.
+  Returns TRUE if this date is earlier than or equal to \a d, otherwise FALSE.
 */
 
 /*!
   \fn bool QDate::operator>( const QDate &d ) const
-  Returns TRUE if this date is after \e d, otherwise FALSE.
+  Returns TRUE if this date is later than \a d, otherwise FALSE.
 */
 
 /*!
   \fn bool QDate::operator>=( const QDate &d ) const
-  Returns TRUE if this date is equal to or after \e d, otherwise FALSE.
+  Returns TRUE if this date is later than or equal to \a d, otherwise FALSE.
 */
 
 
 /*!
-  Returns the current date.
+  Returns the current date, as reported by the system clock.
+
   \sa QTime::currentTime(), QDateTime::currentDateTime()
 */
 
@@ -355,11 +445,21 @@ QDate QDate::currentDate()
 }
 
 /*!
-  Returns TRUE if the specified date is valid.
+  Returns TRUE if the specified date (year \a y, month \a m and day \a
+  d) is valid.
 
-  Note that years 00-99 are treated as 1900-1999.
+  Example:
+  \code
+    QDate::isValid( 2002, 5, 17 );	// TRUE;  May 17th 2002 is OK.
+    QDate::isValid( 2002, 2, 30 );	// FALSE; Feb 30th does not exist
+    QDate::isValid( 2004, 2, 29 );	// TRUE; 2004 is a leap year
+    QDate::isValid( 1202, 6, 6 );	// FALSE; 1202 is pre-Gregorian
+  \endcode
 
-  \sa isNull()
+  Note that a \a y value in the range 00-99 is interpreted as
+  1900-1999.
+
+  \sa isNull(), setYMD()
 */
 
 bool QDate::isValid( int y, int m, int d )
@@ -374,7 +474,7 @@ bool QDate::isValid( int y, int m, int d )
 }
 
 /*!
-  Returns TRUE if the specified year \e y is a leap year.
+  Returns TRUE if the specified year \a y is a leap year.
 */
 
 bool QDate::leapYear( int y )
@@ -443,28 +543,64 @@ void QDate::jul2greg( uint jd, int &y, int &m, int &d )
 /*!
   \class QTime qdatetime.h
 
-  \brief The QTime class provides time functions 24 hours a day.
-
-  \capt Time functions
+  \brief The QTime class provides clock time functions.
 
   \ingroup time
 
-  The time resolution of QTime is a millisecond, although the accuracy
-  depends on the underlying operating system.  Some operating systems
-  (e.g. Linux and Window NT) support a one-millisecond resolution, while
-  others (MS-DOS and Windows 3.1) support only a 55 millisecond resolution.
+  A QTime object contains a clock time, i.e. a number of hours,
+  minutes, seconds and milliseconds since midnight. It can read the
+  current time from the system clock, and measure a span of elapsed
+  time. It provides functions for comparing times and for manipulating
+  a time by adding a number of (milli)seconds.
+
+  QTime operates with 24-hour clock format; it has no concept of
+  AM/PM. It operates with local time; it does not know anything about
+  time zones or daylight savings time.
+
+  A QTime object is typically created either by giving the number of
+  hours, minutes, seconds, and milliseconds explicitly, or by using
+  the static function currentTime(), which makes a QTime object which
+  contains the system's clock time. Note that the accuracy depends on
+  the accuracy of the underlying operating system; not all systems
+  provide 1-millisecond accuracy.
+
+  The hour(), minute(), second(), and msec() functions provide access
+  to the number of hours, minutes, seconds, and milliseconds of the
+  time. The same information is provided in textual format by the
+  toString() function.
+
+  QTime provides a full set of operators to compare two QTime
+  objects. A time is considered smaller than another if it is earlier
+  than the other.
+
+  The time a given number of seconds or milliseconds later than a
+  given time can be found using the addSecs() or addMSecs()
+  functions. Correspondingly, the number of (milli)seconds between two
+  times can be found using the secsTo() or msecsTo() functions.
+
+  QTime can be used to measure a span of elapsed time using the
+  start(), restart(), and elapsed() functions.
 
   \sa QDate, QDateTime
 */
 
 /*!
   \fn QTime::QTime()
-  Constructs a time 00:00:00.000, which is valid.
+
+  Constructs the time 0 hours, minutes, seconds and milliseconds,
+  i.e. 00:00:00.000 (midnight). This is a valid time.
+
+  \sa isValid()
 */
 
 /*!
-  Constructs a time with hour \e h, minute \e m, seconds \e s and milliseconds
-  \e ms.
+  Constructs a time with hour \a h, minute \a m, seconds \a s and
+  milliseconds \a ms.
+
+  \a h must be in the range 0-23, \a m and \a s must be in the range
+  0-59, and \a ms must be in the range 0-999.
+
+  \sa isValid()
 */
 
 QTime::QTime( int h, int m, int s, int ms )
@@ -474,7 +610,7 @@ QTime::QTime( int h, int m, int s, int ms )
 
 
 /*!
-  \fn bool  QTime::isNull() const
+  \fn bool QTime::isNull() const
   Returns TRUE if the time is equal to 00:00:00.000. A null time is valid.
 
   \sa isValid()
@@ -531,8 +667,9 @@ int QTime::msec() const
 
 
 /*!
-  Converts the date to a string, which is returned.  Milliseconds are
-  not included. The string format is "03:40:13".
+  Returns the time of this object in a textual format. Milliseconds
+  are not included. The string format is HH:MM:SS, e.g. 1 second
+  before midnight would be "23:59:59".
 */
 
 QString QTime::toString() const
@@ -544,18 +681,24 @@ QString QTime::toString() const
 
 
 /*!
-  Sets the hour \e h, minute \e m, seconds \e s and milliseconds
-  \e ms.
-  Returns TRUE if the time is valid, otherwise FALSE.
+  Sets the time to hour \a h, minute \a m, seconds \a s and
+  milliseconds \a ms.
+
+  \a h must be in the range 0-23, \a m and \a s must be in the range
+  0-59, and \a ms must be in the range 0-999. Returns TRUE if the set
+  time is valid, otherwise FALSE.
+
+  \sa isValid()
 */
 
 bool QTime::setHMS( int h, int m, int s, int ms )
 {
     if ( !isValid(h,m,s,ms) ) {
 #if defined(CHECK_RANGE)
-	warning( "QTime::setHMS Invalid time %02d:%02d:%02d.%03d", h, m, s,
+	qWarning( "QTime::setHMS Invalid time %02d:%02d:%02d.%03d", h, m, s,
 		 ms );
 #endif
+	ds = MSECS_PER_DAY;		// make this invalid
 	return FALSE;
     }
     ds = (h*SECS_PER_HOUR + m*SECS_PER_MIN + s)*1000 + ms;
@@ -563,20 +706,32 @@ bool QTime::setHMS( int h, int m, int s, int ms )
 }
 
 /*!
-  Returns the time plus \e nsecs seconds.
-  \sa secsTo()
+  Returns a QTime object containg a time \a nsecs seconds later than
+  the time of this object (or earlier if \a ms is negative).
+
+  Note that the time will wrap if it passes midnight.
+
+  Example:
+  \code
+    QTime n( 14, 0, 0 );                // n == 14:00:00
+    QTime t;
+    t = n.addSecs( 70 );                // t == 14:01:10
+    t = n.addSecs( -70 );               // t == 13:58:50
+    t = n.addSecs( 10*60*60 + 5 );      // t == 00:00:05
+    t = n.addSecs( -15*60*60 );         // t == 23:00:00
+  \endcode
+
+  \sa addMSecs(), secsTo(), QDateTime::addSecs()
 */
 
 QTime QTime::addSecs( int nsecs ) const
 {
-    QTime t;
-    t.ds = ((int)ds + nsecs*1000) % MSECS_PER_DAY;
-    return t;
+    return addMSecs(nsecs*1000);
 }
 
 /*!
   Returns the number of seconds from this time to \a t (which is
-  negative if \a t is in the past).
+  negative if \a t is earlier than this time).
 
   Since QTime measures time within a day and there are 86400 seconds
   in a day, the result is between -86400 and 86400.
@@ -590,18 +745,37 @@ int QTime::secsTo( const QTime &t ) const
 }
 
 /*!
-  Returns the time plus \e ms milliseconds.
+  Returns a QTime object containing a time \a ms milliseconds later than
+  the time of this object (or earlier if \a ms is negative).
+
+  Note that the time will wrap if it passes midnight. See addSecs()
+  for an example.
+
+  \sa addSecs(), msecsTo()
 */
 
 QTime QTime::addMSecs( int ms ) const
 {
     QTime t;
-    t.ds = (ds + ms) % MSECS_PER_DAY;
+    if ( ms < 0 ) {
+	// % not well-defined for -ve, but / is.
+	int negdays = (MSECS_PER_DAY-ms) / MSECS_PER_DAY;
+	t.ds = ((int)ds + ms + negdays*MSECS_PER_DAY)
+		% MSECS_PER_DAY;
+    } else {
+	t.ds = ((int)ds + ms) % MSECS_PER_DAY;
+    }
     return t;
 }
 
 /*!
-  Returns the number of milliseconds between this time and \e t.
+  Returns the number of milliseconds from this time to \a t (which is
+  negative if \a t is earlier than this time).
+
+  Since QTime measures time within a day and there are 86400000
+  milliseconds in a day, the result is between -86400000 and 86400000.
+
+  \sa secsTo()
 */
 
 int QTime::msecsTo( const QTime &t ) const
@@ -612,40 +786,51 @@ int QTime::msecsTo( const QTime &t ) const
 
 /*!
   \fn bool QTime::operator==( const QTime &t ) const
-  Returns TRUE if this time is equal to \e t, or FALSE if
-  they are different.
+
+  Returns TRUE if this time is equal to \a t, or FALSE if they are
+  different.
 */
 
 /*!
   \fn bool QTime::operator!=( const QTime &t ) const
-  Returns TRUE if this time is different from \e t, or FALSE if
-  they are equal.
+
+  Returns TRUE if this time is different from \a t, or FALSE if they
+  are equal.
 */
 
 /*!
   \fn bool QTime::operator<( const QTime &t ) const
-  Returns TRUE if this time is before \e t, otherwise FALSE.
+
+  Returns TRUE if this time is earlier than \a t, otherwise FALSE.
 */
 
 /*!
   \fn bool QTime::operator<=( const QTime &t ) const
-  Returns TRUE if this time is before or equal to \e t, otherwise FALSE.
+
+  Returns TRUE if this time is earlier than or equal to \a t,
+  otherwise FALSE.
 */
 
 /*!
   \fn bool QTime::operator>( const QTime &t ) const
-  Returns TRUE if this time is after \e t, otherwise FALSE.
+
+  Returns TRUE if this time is later than \a t, otherwise FALSE.
 */
 
 /*!
   \fn bool QTime::operator>=( const QTime &t ) const
-  Returns TRUE if this time is equal to or after \e t, otherwise FALSE.
+
+  Returns TRUE if this time is later than or equal to \a t, otherwise
+  FALSE.
 */
 
 
 
 /*!
-  Returns the current time.
+  Returns the current time, as reported by the system clock.
+
+  Note that the accuracy depends on the accuracy of the underlying
+  operating system; not all systems provide 1-millisecond accuracy.
 */
 
 QTime QTime::currentTime()
@@ -656,6 +841,8 @@ QTime QTime::currentTime()
 }
 
 /*!
+  \internal
+
   Fetches the current time and returns TRUE if the time is within one
   minute after midnight, otherwise FALSE. The return value is used by
   QDateTime::currentDateTime() to ensure that the date there is correct.
@@ -665,7 +852,7 @@ bool QTime::currentTime( QTime *ct )
 {
     if ( !ct ) {
 #if defined(CHECK_NULL)
-	warning( "QTime::currentTime(QTime *): Null pointer not allowed" );
+	qWarning( "QTime::currentTime(QTime *): Null pointer not allowed" );
 #endif
 	return FALSE;
     }
@@ -718,6 +905,9 @@ bool QTime::currentTime( QTime *ct )
 /*!
   Returns TRUE if the specified time is valid, otherwise FALSE.
 
+  The time is valid if \a h is in the range 0-23, \a m and \a s are in
+  the range 0-59, and \a ms is in the range 0-999.
+
   Example:
   \code
     QTime::isValid(21, 10, 30);		// returns TRUE
@@ -732,15 +922,16 @@ bool QTime::isValid( int h, int m, int s, int ms )
 
 
 /*!
-  Sets the time to the current time, e.g. for timing:
+  Sets this time to the current time. This is practical for timing:
+
   \code
     QTime t;
     t.start();				// start clock
     ... // some lengthy task
-    debug( "%d\n", t.elapsed() );	// prints # msecs elapsed
+    qDebug( "%d\n", t.elapsed() );	// prints # msecs elapsed
   \endcode
 
-  \sa restart(), elapsed()
+  \sa restart(), elapsed(), currentTime()
 */
 
 void QTime::start()
@@ -749,20 +940,22 @@ void QTime::start()
 }
 
 /*!
-  Sets *this to the current time, and returns the number of
-  milliseconds that have elapsed since the last start() or restart().
+  Sets this time to the current time, and returns the number of
+  milliseconds that have elapsed since the last time start() or
+  restart() was called.
 
-  restart is guaranteed to be atomic, and so is very handy for
-  repeated measurements; call start() to start the first measurement,
+  This function is guaranteed to be atomic, and is thus very handy for
+  repeated measurements: call start() to start the first measurement,
   then restart() for each later measurement.
 
   Note that the counter wraps to zero 24 hours after the last call to
   start() or restart().
 
-  \warning If the system's local time changes, the result is undefined.
+  \warning If the system's clock setting has been changed since the
+  last time start() or restart() was called, the result is undefined.
   This can happen e.g. when daylight saving is turned on or off.
 
-  \sa start(), elapsed()
+  \sa start(), elapsed(), currentTime()
 */
 
 int QTime::restart()
@@ -776,13 +969,17 @@ int QTime::restart()
 }
 
 /*!
-  Returns the number of milliseconds that have elapsed since start() or
-  restart() were called.
+  Returns the number of milliseconds that have elapsed since the last
+  time start() or restart() was called.
 
   Note that the counter wraps to zero 24 hours after the last call to
   start() or restart.
 
-  \warning If the system's local time changes, the result is undefined.
+  Note that the accuracy depends on the accuracy of the underlying
+  operating system; not all systems provide 1-millisecond accuracy.
+
+  \warning If the system's clock setting has been changed since the
+  last time start() or restart() was called, the result is undefined.
   This can happen e.g. when daylight saving is turned on or off.
 
   \sa start(), restart()
@@ -803,27 +1000,57 @@ int QTime::elapsed()
 
 /*!
   \class QDateTime qdatetime.h
-  \brief The QDateTime class combines QDate and QTime into a single class.
+  \brief The QDateTime class provides date and time functions.
 
   \ingroup time
 
-  QDateTime provides high precision date and time functions since it can work
-  with Gregorian dates up to about year 8000.
+  A QDateTime object contains a calendar date and a clock time (a
+  "datetime"). It is a combination of the QDate and QTime classes. It
+  can read the current datetime from the system clock. It provides
+  functions for comparing datetimes and for manipulating a datetime by
+  adding a number of seconds or days.
 
-  Most countries that use the Gregorian calendar switched to it between 1550
-  and 1920.
+  A QDateTime object is typically created either by giving a date and
+  time explicitly, or by using the static function currentTime(),
+  which makes a QDateTime object which contains the system's clock
+  time.
+
+  The date() and time() functions provide access to the date and time
+  parts of the datetime. The same information is provided in textual
+  format by the toString() function.
+
+  QDateTime provides a full set of operators to compare two QDateTime
+  objects. A datetime is considered smaller than another if it is
+  earlier than the other.
+
+  The datetime a given number of days or seconds later than a given
+  datetime can be found using the addDays() and addSecs()
+  functions. Correspondingly, the number of days or seconds between
+  two times can be found using the daysTo() or secsTo() functions.
+
+  A datetime can also be set using the setTime_t() function, which
+  takes a POSIX-standard "number of seconds since 00:00:00 on January
+  1, 1970" value.
+
+  The limitations regarding range and resolution mentioned in the
+  QDate and QTime documentation apply for QDateTime also.
 
   \sa QDate, QTime
 */
 
-/*!
-  \fn QDateTime::QDateTime()
-  Constructs a null datetime (i.e. null date and null time).  A null
-  datetime is invalid, since the date is invalid.
-*/
 
 /*!
-  Constructs a datetime with date \e date and null time (00:00:00.000).
+  \fn QDateTime::QDateTime()
+
+  Constructs a null datetime (i.e. null date and null time).  A null
+  datetime is invalid, since the date is invalid.
+
+  \sa isValid()
+*/
+
+
+/*!
+  Constructs a datetime with date \a date and null time (00:00:00.000).
 */
 
 QDateTime::QDateTime( const QDate &date )
@@ -832,7 +1059,7 @@ QDateTime::QDateTime( const QDate &date )
 }
 
 /*!
-  Constructs a datetime with date \e date and time \e time.
+  Constructs a datetime with date \a date and time \a time.
 */
 
 QDateTime::QDateTime( const QDate &date, const QTime &time )
@@ -843,38 +1070,50 @@ QDateTime::QDateTime( const QDate &date, const QTime &time )
 
 /*!
   \fn bool QDateTime::isNull() const
+
   Returns TRUE if both the date and the time are null.	A null date is invalid.
+
   \sa QDate::isNull(), QTime::isNull()
 */
 
 /*!
   \fn bool QDateTime::isValid() const
-  Returns TRUE if both the date and the time are value.
+
+  Returns TRUE if both the date and the time are valid.
+
   \sa QDate::isValid(), QTime::isValid()
 */
 
 /*!
   \fn QDate QDateTime::date() const
+
   Returns the date part of this datetime.
-  \sa time()
+
+  \sa setDate(), time()
 */
 
 /*!
   \fn QTime QDateTime::time() const
+
   Returns the time part of this datetime.
-  \sa date()
+
+  \sa setTime(), date()
 */
 
 /*!
   \fn void QDateTime::setDate( const QDate &date )
+
   Sets the date part of this datetime.
-  \sa setTime()
+
+  \sa date(), setTime()
 */
 
 /*!
   \fn void QDateTime::setTime( const QTime &time )
+
   Sets the time part of this datetime.
-  \sa setDate()
+
+  \sa time(), setDate()
 */
 
 
@@ -883,6 +1122,9 @@ QDateTime::QDateTime( const QDate &date, const QTime &time )
   since 00:00:00 on January 1, 1970, Coordinated Universal Time (UTC).
   On systems that do not support timezones this function will behave as if
   local time were UTC.
+
+  Note that Microsoft Windows supports only a limited range of values for
+  \a secsSince1Jan1970UTC.
 */
 
 void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
@@ -890,11 +1132,10 @@ void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
     time_t tmp = (time_t) secsSince1Jan1970UTC;
     tm *tM = localtime( &tmp );
     if ( !tM ) {
-	tM = localtime( 0 );
+	tM = gmtime( &tmp );
 	if ( !tM ) {
-#if defined(CHECK_NULL)
-	    warning( "QDateTime::setTime_t: Cannot get localtime" );
-#endif
+	    d.jd = QDate::greg2jul( 1970, 1, 1 );
+	    t.ds = 0;
 	    return;
 	}
     }
@@ -907,21 +1148,33 @@ void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
 /*!
   Returns the datetime as a string.
 
-  The string format is "Sat May 20 1995 03:40:13".
+  The string format is "Sat May 20 03:40:13 1998".
+
+  This function uses QDate::dayName(), QDate::monthName(), and
+  QTime::toString() to generate the string.
+
 */
 
 QString QDateTime::toString() const
 {
-    QString buf;
-    QString time = t.toString();
-    buf.sprintf( "%s %s %d %s %d", d.dayName(d.dayOfWeek()),
-		 d.monthName(d.month()), d.day(), (const char*)time, d.year());
+    QString buf = d.dayName(d.dayOfWeek());
+    buf += ' ';
+    buf += d.monthName(d.month());
+    buf += ' ';
+    buf += QString().setNum(d.day());
+    buf += ' ';
+    buf += t.toString();
+    buf += ' ';
+    buf += QString().setNum(d.year());
     return buf;
 }
 
 /*!
-  Returns the datetime plus \e ndays days.
-  \sa daysTo()
+  Returns a QDateTime object containing a datetime \a ndays days later
+  than the datetime of this object (or earlier if \a ndays is
+  negative).
+
+  \sa daysTo(), addSecs()
 */
 
 QDateTime QDateTime::addDays( int ndays ) const
@@ -930,8 +1183,11 @@ QDateTime QDateTime::addDays( int ndays ) const
 }
 
 /*!
-  Returns the datetime plus \e nsecs seconds.
-  \sa secsTo()
+  Returns a QDateTime object containing a datetime \a nsecs seconds
+  later than the datetime of this object (or earlier if \a nsecs is
+  negative).
+
+  \sa secsTo(), addDays()
 */
 
 QDateTime QDateTime::addSecs( int nsecs ) const
@@ -964,10 +1220,10 @@ QDateTime QDateTime::addSecs( int nsecs ) const
 }
 
 /*!
-  Returns the number of days from this datetime to \a dt, which is
-  negative if \a dt is in the past.
+  Returns the number of days from this datetime to \a dt (which is
+  negative if \a dt is earlier than this datetime).
 
-  \sa addDays() secsTo()
+  \sa addDays(), secsTo()
 */
 
 int QDateTime::daysTo( const QDateTime &dt ) const
@@ -976,17 +1232,17 @@ int QDateTime::daysTo( const QDateTime &dt ) const
 }
 
 /*!
-  Returns the number of seconds from this datetime to \a dt, which is
-  negative if \a t is in the past.
+  Returns the number of seconds from this datetime to \a dt (which is
+  negative if \a dt is earlier than this datetime).
 
   Example:
   \code
     QDateTime dt = QDateTime::currentDateTime();
     QDateTime x( QDate(dt.year(),12,24), QTime(17,00) );
-    debug( "There are %d seconds to Christmas", dt.secsTo(x) );
+    qDebug( "There are %d seconds to Christmas", dt.secsTo(x) );
   \endcode
 
-  \sa addSecs() daysTo() QTime::secsTo()
+  \sa addSecs(), daysTo(), QTime::secsTo()
 */
 
 int QDateTime::secsTo( const QDateTime &dt ) const
@@ -996,7 +1252,7 @@ int QDateTime::secsTo( const QDateTime &dt ) const
 
 
 /*!
-  Returns TRUE if this datetime is equal to \e dt, or FALSE if
+  Returns TRUE if this datetime is equal to \a dt, or FALSE if
   they are different.
   \sa operator!=()
 */
@@ -1007,7 +1263,7 @@ bool QDateTime::operator==( const QDateTime &dt ) const
 }
 
 /*!
-  Returns TRUE if this datetime is different from \e dt, or FALSE if
+  Returns TRUE if this datetime is different from \a dt, or FALSE if
   they are equal.
   \sa operator==()
 */
@@ -1018,7 +1274,7 @@ bool QDateTime::operator!=( const QDateTime &dt ) const
 }
 
 /*!
-  Returns TRUE if this datetime is before \e dt, otherwise FALSE.
+  Returns TRUE if this datetime is earlier than \a dt, otherwise FALSE.
 */
 
 bool QDateTime::operator<( const QDateTime &dt ) const
@@ -1029,8 +1285,8 @@ bool QDateTime::operator<( const QDateTime &dt ) const
 }
 
 /*!
-  Returns TRUE if this datetime is before or equal to \e dt, otherwise
-  FALSE.
+  Returns TRUE if this datetime is earlier than or equal to \a dt,
+  otherwise FALSE.
 */
 
 bool QDateTime::operator<=( const QDateTime &dt ) const
@@ -1041,7 +1297,7 @@ bool QDateTime::operator<=( const QDateTime &dt ) const
 }
 
 /*!
-  Returns TRUE if this datetime is after \e dt, otherwise FALSE.
+  Returns TRUE if this datetime is later than \a dt, otherwise FALSE.
 */
 
 bool QDateTime::operator>( const QDateTime &dt ) const
@@ -1052,8 +1308,8 @@ bool QDateTime::operator>( const QDateTime &dt ) const
 }
 
 /*!
-  Returns TRUE if this datetime is equal to or after \e dt, otherwise
-  FALSE.
+  Returns TRUE if this datetime is later than or equal to \a dt,
+  otherwise FALSE.
 */
 
 bool QDateTime::operator>=( const QDateTime &dt ) const
@@ -1064,7 +1320,8 @@ bool QDateTime::operator>=( const QDateTime &dt ) const
 }
 
 /*!
-  Returns the current datetime.
+  Returns the current datetime, as reported by the system clock.
+
   \sa QDate::currentDate(), QTime::currentTime()
 */
 
@@ -1087,7 +1344,7 @@ QDateTime QDateTime::currentDateTime()
   \relates QDate
   Writes the date to the stream.
 
-  Serialization format: [Q_UINT32], Julian day.
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QDate &d )
@@ -1098,6 +1355,8 @@ QDataStream &operator<<( QDataStream &s, const QDate &d )
 /*!
   \relates QDate
   Reads a date from the stream.
+
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QDate &d )
@@ -1112,7 +1371,7 @@ QDataStream &operator>>( QDataStream &s, QDate &d )
   \relates QTime
   Writes a time to the stream.
 
-  Serialization format: [Q_UINT32], milliseconds since midnight.
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QTime &t )
@@ -1123,6 +1382,8 @@ QDataStream &operator<<( QDataStream &s, const QTime &t )
 /*!
   \relates QTime
   Reads a time from the stream.
+
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QTime &t )
@@ -1137,7 +1398,7 @@ QDataStream &operator>>( QDataStream &s, QTime &t )
   \relates QDateTime
   Writes a datetime to the stream.
 
-  Serialization format: [QDate QTime].
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QDateTime &dt )
@@ -1148,6 +1409,8 @@ QDataStream &operator<<( QDataStream &s, const QDateTime &dt )
 /*!
   \relates QDateTime
   Reads a datetime from the stream.
+
+  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QDateTime &dt )
