@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qglist.cpp,v 2.6 1998/07/03 00:09:45 hanord Exp $
+** $Id: qt/src/tools/qglist.cpp   2.1.1   edited 2000-04-06 $
 **
 ** Implementation of QGList and QGListIterator classes
 **
 ** Created : 920624
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,6 +27,8 @@
 #include "qgvector.h"
 #include "qdatastream.h"
 
+
+// NOT REVISED
 /*!
   \class QLNode qglist.h
   \brief The QLNode class is an internal class for the QList template collection.
@@ -46,7 +50,7 @@
 */
 
 /*!
-  \fn GCI QLNode::getData()
+  \fn QCollection::Item QLNode::getData()
   Returns a pointer (\c void*) to the actual data in the list node.
 */
 
@@ -106,28 +110,28 @@
   \endcode
 */
 
-int QGList::compareItems( GCI item1, GCI item2 )
+int QGList::compareItems( QCollection::Item item1, QCollection::Item item2 )
 {
     return item1 != item2;			// compare pointers
 }
 
 /*!
-  Reads a collection/list item from the stream \e s and returns a reference
+  Reads a collection/list item from the stream \a s and returns a reference
   to the stream.
 
-  The default implementation sets \e item to 0.
+  The default implementation sets \a item to 0.
 
   \sa write()
 */
 
-QDataStream &QGList::read( QDataStream &s, GCI &item )
+QDataStream &QGList::read( QDataStream &s, QCollection::Item &item )
 {
     item = 0;
     return s;
 }
 
 /*!
-  Writes a collection/list item to the stream \e s and returns a reference
+  Writes a collection/list item to the stream \a s and returns a reference
   to the stream.
 
   The default implementation does nothing.
@@ -135,7 +139,7 @@ QDataStream &QGList::read( QDataStream &s, GCI &item )
   \sa read()
 */
 
-QDataStream &QGList::write( QDataStream &s, GCI ) const
+QDataStream &QGList::write( QDataStream &s, QCollection::Item ) const
 {
     return s;
 }
@@ -170,7 +174,7 @@ QGList::QGList( const QGList & list )
     numNodes  = 0;
     curIndex  = -1;
     iterators = 0;				// initialize iterator list
-    register QLNode *n = list.firstNode;
+    QLNode *n = list.firstNode;
     while ( n ) {				// copy all items from list
 	append( n->data );
 	n = n->next;
@@ -187,9 +191,10 @@ QGList::~QGList()
     clear();
     if ( !iterators )				// no iterators for this list
 	return;
-    register QGListIterator *i = (QGListIterator*)iterators->first();
+    QGListIterator *i = (QGListIterator*)iterators->first();
     while ( i ) {				// notify all iterators that
 	i->list = 0;				// this list is deleted
+	i->curNode = 0;
 	i = (QGListIterator*)iterators->next();
     }
     delete iterators;
@@ -205,7 +210,7 @@ QGList& QGList::operator=( const QGList &list )
 {
     clear();
     if ( list.count() > 0 ) {
-	register QLNode *n = list.firstNode;
+	QLNode *n = list.firstNode;
 	while ( n ) {				// copy all items from list
 	    append( n->data );
 	    n = n->next;
@@ -216,6 +221,31 @@ QGList& QGList::operator=( const QGList &list )
     return *this;
 }
 
+/*!
+  Compares this list with \a list. Retruns TRUE if the lists
+  contain the same data, else FALSE.
+*/
+
+bool QGList::operator==( const QGList &list ) const
+{
+    if ( count() != list.count() )
+	return FALSE;
+    
+    if ( count() == 0 )
+	return TRUE;
+    
+    QLNode *n1 = firstNode;
+    QLNode *n2 = list.firstNode;
+    while ( n1 && n2 ) {
+	// should be mutable
+	if ( ( (QGList*)this )->compareItems( n1->data, n2->data ) != 0 )
+	    return FALSE;
+	n1 = n1->next;
+	n2 = n2->next;
+    }
+    
+    return TRUE;
+}
 
 /*!
   \fn uint QGList::count() const
@@ -243,7 +273,7 @@ QLNode *QGList::locate( uint index )
 
     if ( index >= numNodes ) {
 #if defined(CHECK_RANGE)
-	warning( "QGList::locate: Index %d out of range", index );
+	qWarning( "QGList::locate: Index %d out of range", index );
 #endif
 	return 0;
     }
@@ -281,7 +311,7 @@ QLNode *QGList::locate( uint index )
   Inserts an item at its sorted position in the list.
 */
 
-void QGList::inSort( GCI d )
+void QGList::inSort( QCollection::Item d )
 {
     int index = 0;
     register QLNode *n = firstNode;
@@ -298,7 +328,7 @@ void QGList::inSort( GCI d )
   Inserts an item at the start of the list.
 */
 
-void QGList::prepend( GCI d )
+void QGList::prepend( QCollection::Item d )
 {
     register QLNode *n = new QLNode( newItem(d) );
     CHECK_PTR( n );
@@ -318,7 +348,7 @@ void QGList::prepend( GCI d )
   Inserts an item at the end of the list.
 */
 
-void QGList::append( GCI d )
+void QGList::append( QCollection::Item d )
 {
     register QLNode *n = new QLNode( newItem(d) );
     CHECK_PTR( n );
@@ -338,7 +368,7 @@ void QGList::append( GCI d )
   Inserts an item at position \e index in the list.
 */
 
-bool QGList::insertAt( uint index, GCI d )
+bool QGList::insertAt( uint index, QCollection::Item d )
 {
     if ( index == 0 ) {				// insert at head of list
 	prepend( d );
@@ -417,8 +447,8 @@ QLNode *QGList::unlink()
 	curNode = n->prev;
 	curIndex--;
     }
-    if ( iterators ) {				// update iterators
-	register QGListIterator *i = (QGListIterator*)iterators->first();
+    if ( iterators && iterators->count() ) {	// update iterators
+	QGListIterator *i = (QGListIterator*)iterators->first();
 	while ( i ) {				// fix all iterators that
 	    if ( i->curNode == n )		// refers to pending node
 		i->curNode = curNode;
@@ -440,7 +470,7 @@ bool QGList::removeNode( QLNode *n )
 #if defined(CHECK_NULL)
     if ( n == 0 || (n->prev && n->prev->next != n) ||
 	 (n->next && n->next->prev != n) ) {
-	warning( "QGList::removeNode: Corrupted node" );
+	qWarning( "QGList::removeNode: Corrupted node" );
 	return FALSE;
     }
 #endif
@@ -458,7 +488,7 @@ bool QGList::removeNode( QLNode *n )
   Removes the item \e d from the list.	Uses compareItems() to find the item.
 */
 
-bool QGList::remove( GCI d )
+bool QGList::remove( QCollection::Item d )
 {
     if ( d ) {					// find the item
 	if ( find(d) == -1 )
@@ -477,7 +507,7 @@ bool QGList::remove( GCI d )
   Removes the item \e d from the list.
 */
 
-bool QGList::removeRef( GCI d )
+bool QGList::removeRef( QCollection::Item d )
 {
     if ( d ) {					// find the item
 	if ( findRef(d) == -1 )
@@ -526,18 +556,18 @@ bool QGList::removeAt( uint index )
   Takes the node \e n out of the list.
 */
 
-GCI QGList::takeNode( QLNode *n )
+QCollection::Item QGList::takeNode( QLNode *n )
 {
 #if defined(CHECK_NULL)
     if ( n == 0 || (n->prev && n->prev->next != n) ||
 	 (n->next && n->next->prev != n) ) {
-	warning( "QGList::takeNode: Corrupted node" );
+	qWarning( "QGList::takeNode: Corrupted node" );
 	return 0;
     }
 #endif
     curNode = n;
     unlink();					// unlink node
-    GCI d = n->data;
+    Item d = n->data;
     delete n;					// delete the node, not data
     curNode  = firstNode;
     curIndex = curNode ? 0 : -1;
@@ -549,10 +579,10 @@ GCI QGList::takeNode( QLNode *n )
   Takes the current item out of the list.
 */
 
-GCI QGList::take()
+QCollection::Item QGList::take()
 {
     QLNode *n = unlink();			// unlink node
-    GCI d = n ? n->data : 0;
+    Item d = n ? n->data : 0;
     delete n;					// delete node, keep contents
     return d;
 }
@@ -562,12 +592,12 @@ GCI QGList::take()
   Takes the item at position \e index out of the list.
 */
 
-GCI QGList::takeAt( uint index )
+QCollection::Item QGList::takeAt( uint index )
 {
     if ( !locate(index) )
 	return 0;
     QLNode *n = unlink();			// unlink node
-    GCI d = n ? n->data : 0;
+    Item d = n ? n->data : 0;
     delete n;					// delete node, keep contents
     return d;
 }
@@ -577,11 +607,11 @@ GCI QGList::takeAt( uint index )
   Takes the first item out of the list.
 */
 
-GCI QGList::takeFirst()
+QCollection::Item QGList::takeFirst()
 {
     first();
     QLNode *n = unlink();			// unlink node
-    GCI d = n ? n->data : 0;
+    Item d = n ? n->data : 0;
     delete n;
     return d;
 }
@@ -591,11 +621,11 @@ GCI QGList::takeFirst()
   Takes the last item out of the list.
 */
 
-GCI QGList::takeLast()
+QCollection::Item QGList::takeLast()
 {
     last();
     QLNode *n = unlink();			// unlink node
-    GCI d = n ? n->data : 0;
+    Item d = n ? n->data : 0;
     delete n;
     return d;
 }
@@ -609,6 +639,19 @@ GCI QGList::takeLast()
 void QGList::clear()
 {
     register QLNode *n = firstNode;
+
+    firstNode = lastNode = curNode = 0;		// initialize list
+    numNodes = 0;
+    curIndex = -1;
+
+    if ( iterators && iterators->count() ) {
+	QGListIterator *i = (QGListIterator*)iterators->first();
+	while ( i ) {				// notify all iterators that
+	    i->curNode = 0;			// this list is empty
+	    i = (QGListIterator*)iterators->next();
+	}
+    }
+
     QLNode *prevNode;
     while ( n ) {				// for all nodes ...
 	deleteItem( n->data );			// deallocate data
@@ -616,16 +659,6 @@ void QGList::clear()
 	n = n->next;
 	delete prevNode;			// deallocate node
     }
-    firstNode = lastNode = curNode = 0;		// initialize list
-    numNodes = 0;
-    curIndex = -1;
-    if ( !iterators )				// no iterators for this list
-	return;
-    register QGListIterator *i = (QGListIterator*)iterators->first();
-    while ( i ) {				// notify all iterators that
-	i->curNode = 0;				// this list is empty
-	i = (QGListIterator*)iterators->next();
-    }
 }
 
 
@@ -634,7 +667,7 @@ void QGList::clear()
   Finds an item in the list.
 */
 
-int QGList::findRef( GCI d, bool fromStart )
+int QGList::findRef( QCollection::Item d, bool fromStart )
 {
     register QLNode *n;
     int	     index;
@@ -659,7 +692,7 @@ int QGList::findRef( GCI d, bool fromStart )
   Finds an item in the list.  Uses compareItems().
 */
 
-int QGList::find( GCI d, bool fromStart )
+int QGList::find( QCollection::Item d, bool fromStart )
 {
     register QLNode *n;
     int	     index;
@@ -685,7 +718,7 @@ int QGList::find( GCI d, bool fromStart )
   Counts the number an item occurs in the list.
 */
 
-uint QGList::containsRef( GCI d ) const
+uint QGList::containsRef( QCollection::Item d ) const
 {
     register QLNode *n = firstNode;
     uint     count = 0;
@@ -702,7 +735,7 @@ uint QGList::containsRef( GCI d ) const
   Counts the number an item occurs in the list.	 Uses compareItems().
 */
 
-uint QGList::contains( GCI d ) const
+uint QGList::contains( QCollection::Item d ) const
 {
     register QLNode *n = firstNode;
     uint     count = 0;
@@ -717,7 +750,7 @@ uint QGList::contains( GCI d ) const
 
 
 /*!
-  \fn GCI QGList::at( uint index )
+  \fn QCollection::Item QGList::at( uint index )
   \internal
   Sets the item at position \e index to the current item.
 */
@@ -735,19 +768,19 @@ uint QGList::contains( GCI d ) const
 */
 
 /*!
-  \fn GCI QGList::get() const
+  \fn QCollection::Item QGList::get() const
   \internal
   Returns the current item.
 */
 
 /*!
-  \fn GCI QGList::cfirst() const
+  \fn QCollection::Item QGList::cfirst() const
   \internal
   Returns the first item in the list.
 */
 
 /*!
-  \fn GCI QGList::clast() const
+  \fn QCollection::Item QGList::clast() const
   \internal
   Returns the last item in the list.
 */
@@ -758,7 +791,7 @@ uint QGList::contains( GCI d ) const
   Returns the first list item.	Sets this to current.
 */
 
-GCI QGList::first()
+QCollection::Item QGList::first()
 {
     if ( firstNode ) {
 	curIndex = 0;
@@ -772,7 +805,7 @@ GCI QGList::first()
   Returns the last list item.  Sets this to current.
 */
 
-GCI QGList::last()
+QCollection::Item QGList::last()
 {
     if ( lastNode ) {
 	curIndex = numNodes-1;
@@ -786,7 +819,7 @@ GCI QGList::last()
   Returns the next list item (after current).  Sets this to current.
 */
 
-GCI QGList::next()
+QCollection::Item QGList::next()
 {
     if ( curNode ) {
 	if ( curNode->next ) {
@@ -805,7 +838,7 @@ GCI QGList::next()
   Returns the previous list item (before current).  Sets this to current.
 */
 
-GCI QGList::prev()
+QCollection::Item QGList::prev()
 {
     if ( curNode ) {
 	if ( curNode->prev ) {
@@ -839,6 +872,89 @@ void QGList::toVector( QGVector *vector ) const
     }
 }
 
+void QGList::heapSortPushDown( QCollection::Item* heap, int first, int last )
+{
+    int r = first;
+    while( r <= last/2 ) {
+	// Node r has only one child ?
+	if ( last == 2*r ) {
+	    // Need for swapping ?
+	    if ( compareItems( heap[r], heap[ 2*r ] ) > 0 ) {
+		QCollection::Item tmp = heap[r];
+		heap[ r ] = heap[ 2*r ];
+		heap[ 2*r ] = tmp;
+	    }
+	    // That's it ...
+	    r = last;
+	} else {
+	    // Node has two children
+	    if ( compareItems( heap[r], heap[ 2*r ] ) > 0 &&
+		 compareItems( heap[ 2*r ], heap[ 2*r+1 ] ) <= 0 ) {
+		// Swap with left child
+		QCollection::Item tmp = heap[r];
+		heap[ r ] = heap[ 2*r ];
+		heap[ 2*r ] = tmp;
+		r *= 2;
+	    } else if ( compareItems( heap[r], heap[ 2*r+1 ] ) > 0 &&
+			compareItems( heap[ 2*r+1 ], heap[ 2*r ] ) < 0 ) {
+		// Swap with right child
+		QCollection::Item tmp = heap[r];
+		heap[ r ] = heap[ 2*r+1 ];
+		heap[ 2*r+1 ] = tmp;
+		r = 2*r+1;
+	    } else {
+		// We are done
+		r = last;
+	    }
+	}
+    }
+}
+
+
+/*! Sorts the list by the result of the virtual compareItems() function.
+
+  The Heap-Sort algorithm is used for sorting.  It sorts n items with
+  O(n*log n) compares.  This is the asymptotic optimal solution of the
+  sorting problem.
+*/
+
+void QGList::sort()
+{
+    uint n = count();
+    if ( n < 2 )
+	return;
+
+    // Create the heap
+    QCollection::Item* realheap = new QCollection::Item[ n ];
+    // Wow, what a fake. But I want the heap to be indexed as 1...n
+    QCollection::Item* heap = realheap - 1;
+    int size = 0;
+    QLNode* insert = firstNode;
+    for( ; insert != 0; insert = insert->next ) {
+	heap[++size] = insert->data;
+	int i = size;
+	while( i > 1 && compareItems( heap[i], heap[ i / 2 ] ) < 0 ) {
+	    QCollection::Item tmp = heap[ i ];
+	    heap[ i ] = heap[ i/2 ];
+	    heap[ i/2 ] = tmp;
+	    i /= 2;
+	}
+    }
+
+    insert = firstNode;
+    // Now do the sorting
+    for ( int i = n; i > 0; i-- ) {
+	insert->data = heap[1];
+	insert = insert->next;
+	if ( i > 1 ) {
+	    heap[1] = heap[i];
+	    heapSortPushDown( heap, 1, i - 1 );
+	}
+    }
+
+    delete [] realheap;
+}
+
 
 /*****************************************************************************
   QGList stream functions
@@ -865,7 +981,7 @@ QDataStream &QGList::read( QDataStream &s )
     s >> num;					// read number of items
     clear();					// clear list
     while ( num-- ) {				// read all items
-	GCI d;
+	Item d;
 	read( s, d );
 	CHECK_PTR( d );
 	if ( !d )				// no memory
@@ -954,14 +1070,8 @@ QGListIterator::QGListIterator( const QGListIterator &it )
 
 QGListIterator &QGListIterator::operator=( const QGListIterator &it )
 {
-    if ( list ) {				// detach from old list
-	if ( list->iterators->removeRef(this) ){
-	    if ( list->iterators->count() == 0 ) {
-		delete list->iterators;		// this was the last iterator
-		list->iterators = 0;
-	    }
-	}
-    }
+    if ( list )					// detach from old list
+	list->iterators->removeRef( this );
     list = it.list;
     curNode = it.curNode;
     if ( list )
@@ -976,17 +1086,8 @@ QGListIterator &QGListIterator::operator=( const QGListIterator &it )
 
 QGListIterator::~QGListIterator()
 {
-    if ( list ) {				// detach iterator from list
-#if defined(DEBUG)
-	ASSERT( list->iterators );
-#endif
-	if ( list->iterators->removeRef(this) ) {
-	    if ( list->iterators->count() == 0 ) {
-		delete list->iterators;		// this was the last iterator
-		list->iterators = 0;
-	    }
-	}
-    }
+    if ( list )					// detach iterator from list
+	list->iterators->removeRef(this);
 }
 
 
@@ -1008,11 +1109,11 @@ QGListIterator::~QGListIterator()
   Sets the list iterator to point to the first item in the list.
 */
 
-GCI QGListIterator::toFirst()
+QCollection::Item QGListIterator::toFirst()
 {
     if ( !list ) {
 #if defined(CHECK_NULL)
-	warning( "QGListIterator::toFirst: List has been deleted" );
+	qWarning( "QGListIterator::toFirst: List has been deleted" );
 #endif
 	return 0;
     }
@@ -1024,11 +1125,11 @@ GCI QGListIterator::toFirst()
   Sets the list iterator to point to the last item in the list.
 */
 
-GCI QGListIterator::toLast()
+QCollection::Item QGListIterator::toLast()
 {
     if ( !list ) {
 #if defined(CHECK_NULL)
-	warning( "QGListIterator::toLast: List has been deleted" );
+	qWarning( "QGListIterator::toLast: List has been deleted" );
 #endif
 	return 0;
     }
@@ -1037,7 +1138,7 @@ GCI QGListIterator::toLast()
 
 
 /*!
-  \fn GCI QGListIterator::get() const
+  \fn QCollection::Item QGListIterator::get() const
   \internal
   Returns the iterator item.
 */
@@ -1048,11 +1149,11 @@ GCI QGListIterator::toLast()
   Moves to the next item (postfix).
 */
 
-GCI QGListIterator::operator()()
+QCollection::Item QGListIterator::operator()()
 {
     if ( !curNode )
 	return 0;
-    GCI d = curNode->getData();
+    QCollection::Item d = curNode->getData();
     curNode = curNode->next;
     return  d;
 }
@@ -1062,7 +1163,7 @@ GCI QGListIterator::operator()()
   Moves to the next item (prefix).
 */
 
-GCI QGListIterator::operator++()
+QCollection::Item QGListIterator::operator++()
 {
     if ( !curNode )
 	return 0;
@@ -1075,7 +1176,7 @@ GCI QGListIterator::operator++()
   Moves \e jumps positions forward.
 */
 
-GCI QGListIterator::operator+=( uint jumps )
+QCollection::Item QGListIterator::operator+=( uint jumps )
 {
     while ( curNode && jumps-- )
 	curNode = curNode->next;
@@ -1087,7 +1188,7 @@ GCI QGListIterator::operator+=( uint jumps )
   Moves to the previous item (prefix).
 */
 
-GCI QGListIterator::operator--()
+QCollection::Item QGListIterator::operator--()
 {
     if ( !curNode )
 	return 0;
@@ -1100,7 +1201,7 @@ GCI QGListIterator::operator--()
   Moves \e jumps positions backward.
 */
 
-GCI QGListIterator::operator-=( uint jumps )
+QCollection::Item QGListIterator::operator-=( uint jumps )
 {
     while ( curNode && jumps-- )
 	curNode = curNode->prev;
