@@ -1,28 +1,31 @@
 /****************************************************************************
-** $Id: qiodevice.cpp,v 2.12.2.1 1998/09/28 11:51:00 warwick Exp $
+** $Id: qt/src/tools/qiodevice.cpp   2.1.1   edited 2000-03-24 $
 **
 ** Implementation of QIODevice class
 **
 ** Created : 940913
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qiodevice.h"
 
+// NOT REVISED
 /*!
   \class QIODevice qiodevice.h
 
@@ -30,34 +33,99 @@
 
   \ingroup io
 
-  An I/O device represents a medium that one can read bytes from and write
-  bytes to.  The QIODevice class itself is not capable of reading or
-  writing any data; it has virtual functions for doing so. These functions
-  are implemented by the subclasses QFile and QBuffer.
+  An I/O device represents a medium that one can read bytes from
+  and/or write bytes to.  The QIODevice class is the abstract
+  superclass of all such devices; classes like QFile, QBuffer and
+  QSocket inherit QIODevice and implement virtual functions like
+  write() appropriately.
 
-  There are two types of I/O devices;
-  \link isDirectAccess() <em>direct access</em>\endlink and
-  \link isSequentialAccess() <em>sequential access </em>\endlink devices.
-  Files can normally be accessed
-  directly, except \c stdin etc., which must be processed sequentially.
-  Buffers are always direct access devices.
+  While applications sometimes use QIODevice directly, mostly it is
+  better to go through QTextStream and QDataStream, which provide
+  stream operations on any QIODevice subclass.  QTextStream provides
+  text-oriented stream functionality (for human-readable ASCII files,
+  for example), while QDataStream deals with binary data in a totally
+  platform-independent manner.
 
-  The access mode of an I/O device can be either \e raw or \e buffered.
-  QFile objects can be created using one of these.  Raw access mode is more
-  low level, while buffered access use smart buffering techniques.
-  The raw access mode is best when I/O is block-operated using 4kB block size
-  or greater.  Buffered access works better when reading small portions of
-  data at a time.
+  The public member functions in QIODevice roughly fall into two
+  groups: The action functions and the state access functions.  The
+  most important action functions are: <ul>
 
-  An I/O device operation can be executed in either \e synchronous or
-  \e asynchronous mode.	 The I/O devices currently supported by Qt only
-  execute synchronously.
+  <li> open() opens a device for reading and/or writing, depending on
+  the argument to open().
 
-  The QDataStream and QTextStream provide binary and text operations
-  on QIODevice objects.
+  <li> close() closes the device and tidies up.
 
-  QIODevice provides numerous pure virtual functions you need to implement
-  when subclassing it. Here is a skeleton subclass:
+  <li> readBlock() reads a block of data from the device.
+
+  <li> writeBlock() writes a block of data to the device.
+
+  <li> readLine() reads a line (of text, usually) from the device.
+
+  <li> flush() ensures that all buffered data are written to the real device.
+
+  </ul>There are also some other, less used, action functions: <ul>
+
+  <li> getch() reads a single character.
+
+  <li> ungetch() forgets the last call to getch(), if possible.
+
+  <li> putch() writes a single character.
+
+  <li> size() returns the size of the device, if there is one.
+
+  <li> at() returns the current read/write pointer, if there is one
+  for this device, or it moves the pointer.
+
+  <li> atEnd() says whether there is more to read, if that is a
+  meaningful question for this device.
+
+  <li> reset() moves the read/write pointer to the start of the
+  device, if that is possible for this device.
+
+  </ul>The state access are all "get" functions.  The QIODevice subclass
+  calls setState() to update the state, and simple access functions
+  tell the user of the device what the device's state is.  Here are
+  the settings, and their associated access functions: <ul>
+
+  <li> Access type.  Some devices are direct access (it is possible to
+  read/write anywhere) while others are sequential.  QIODevice
+  provides the access functions isDirectAccess(), isSequentialAccess()
+  and isCombinedAccess() to tell users what a given I/O device
+  supports.
+
+  <li> Buffering.  Some devices are accessed in raw mode while others
+  are buffered.  Buffering usually provides greater efficiency,
+  particularly for small read/write operations.  isBuffered() tells
+  the user whether a given device is buffered.  (This can often be set
+  by the application in the call to open().)
+
+  <li> Synchronicity.  Synchronous devices work there and then, for
+  example files.  When you read from a file, the file delivers its
+  data right away.  Others, such as a socket connected to a HTTP
+  server, may not deliver the data until seconds after you ask to read
+  it.  isSynchronous() and isAsynchronous() tells the user how this
+  device operates.
+
+  <li> CR/LF translation.  For simplicity, applications often like to
+  see jus a single CR/LF style, and QIODevice subclasses can provide
+  that.  isTranslated() returns TRUE if this object translates CR/LF
+  to just LF.  (This can often be set by the application in the call
+  to open().)
+
+  <li> Accessibility.  Some files cannot be written, for example.
+  isReadable(), isWritable and isReadWrite() tells the application
+  whether it can read from and write to a given device.  (This can
+  often be set by the application in the call to open().)
+
+  <li> Finally, isOpen() returns TRUE if the device is open.  This can
+  quite obviously be set using open() :)
+
+  </ul>
+
+  QIODevice provides numerous pure virtual functions you need to
+  implement when subclassing it.  Here is a skeleton subclass with all
+  the members you are certain to need, and some it's likely that you
+  will need:
 
   \code
     class YourDevice : public QIODevice
@@ -65,20 +133,20 @@
     public:
 	YourDevice();
        ~YourDevice();
-  
+
 	bool open( int mode );
 	void close();
 	void flush();
-  
+
 	uint size() const;
 	int  at() const;	// not a pure virtual function
 	bool at( int );		// not a pure virtual function
 	bool atEnd() const;	// not a pure virtual function
-  
-	int readBlock( char *data, uint len );
+
+	int readBlock( char *data, uint maxlen );
 	int writeBlock( const char *data, uint len );
 	int readLine( char *data, uint maxlen );
-  
+
 	int getch();
 	int putch( int );
 	int ungetch( int );
@@ -86,7 +154,7 @@
   \endcode
 
   The three non-pure virtual functions can be ignored if your device
-  is sequential (e.g. a tape device).
+  is sequential (e.g. an RS-232 port).
 
   \sa QDataStream, QTextStream
 */
@@ -100,11 +168,11 @@ QIODevice::QIODevice()
 {
     ioMode = 0;					// initial mode
     ioSt = IO_Ok;
-    index = 0;
+    ioIndex = 0;
 }
 
 /*!
-  Destroys the I/O device.
+  Destructs the I/O device.
 */
 
 QIODevice::~QIODevice()
@@ -254,6 +322,7 @@ QIODevice::~QIODevice()
   <li>\c IO_ConnectError Could not connect to the device.
   <li>\c IO_AbortError The operation was unexpectedly aborted.
   <li>\c IO_TimeOutError The operation timed out.
+  <li>\c IO_OnCloseError An unspecified error happend on close.
   </ul>
 
   \sa resetStatus()
@@ -283,7 +352,7 @@ void QIODevice::setType( int t )
 {
 #if defined(CHECK_RANGE)
     if ( (t & IO_TypeMask) != t )
-	warning( "QIODevice::setType: Specified type out of range" );
+	qWarning( "QIODevice::setType: Specified type out of range" );
 #endif
     ioMode &= ~IO_TypeMask;			// reset type bits
     ioMode |= t;
@@ -298,7 +367,7 @@ void QIODevice::setMode( int m )
 {
 #if defined(CHECK_RANGE)
     if ( (m & IO_ModeMask) != m )
-	warning( "QIODevice::setMode: Specified mode out of range" );
+	qWarning( "QIODevice::setMode: Specified mode out of range" );
 #endif
     ioMode &= ~IO_ModeMask;			// reset mode bits
     ioMode |= m;
@@ -313,7 +382,7 @@ void QIODevice::setState( int s )
 {
 #if defined(CHECK_RANGE)
     if ( ((uint)s & IO_StateMask) != (uint)s )
-	warning( "QIODevice::setState: Specified state out of range" );
+	qWarning( "QIODevice::setState: Specified state out of range" );
 #endif
     ioMode &= ~IO_StateMask;			// reset state bits
     ioMode |= (uint)s;
@@ -387,7 +456,7 @@ void QIODevice::setStatus( int s )
 
 int QIODevice::at() const
 {
-    return index;
+    return ioIndex;
 }
 
 /*!
@@ -399,11 +468,11 @@ bool QIODevice::at( int pos )
 {
 #if defined(CHECK_RANGE)
     if ( (uint)pos > size() ) {
-	warning( "QIODevice::at: Index %d out of range", pos );
+	qWarning( "QIODevice::at: Index %d out of range", pos );
 	return FALSE;
     }
 #endif
-    index = pos;
+    ioIndex = pos;
     return TRUE;
 }
 
@@ -414,14 +483,14 @@ bool QIODevice::at( int pos )
 
 bool QIODevice::atEnd() const
 {
-    if ( isSequentialAccess() ) {
+    if ( isSequentialAccess() || isTranslated() ) {
 	QIODevice* that = (QIODevice*)this;
 	int c = that->getch();
 	bool result = c < 0;
 	that->ungetch(c);
 	return result;
     } else {
-        return at() == (int)size();
+	return at() == (int)size();
     }
 }
 
@@ -433,8 +502,8 @@ bool QIODevice::atEnd() const
 
 
 /*!
-  \fn int QIODevice::readBlock( char *data, uint len )
-  Reads at most \e len bytes from the I/O device into \e data and
+  \fn int QIODevice::readBlock( char *data, uint maxlen )
+  Reads at most \e maxlen bytes from the I/O device into \e data and
   returns the number of bytes actually read.
 
   This virtual function must be reimplemented by all subclasses.
@@ -442,6 +511,28 @@ bool QIODevice::atEnd() const
   \sa writeBlock()
 */
 
+/*!
+  This convenience function returns all of the remaining data in the
+  device.  Note that this only works for direct access devices, such
+  as QFile.
+  
+  \sa isDirectAccess() 
+*/
+QByteArray QIODevice::readAll()
+{
+    int n = size()-at();
+    QByteArray ba(size()-at());
+    char* c = ba.data();
+    while ( n ) {
+	int r = readBlock( c, n );
+	if ( r < 0 )
+	    return QByteArray();
+	n -= r;
+	c += r;
+    }
+    return ba;
+}
+
 /*!
   \fn int QIODevice::writeBlock( const char *data, uint len )
   Writes \e len bytes from \e p to the I/O device and returns the number of
@@ -452,6 +543,15 @@ bool QIODevice::atEnd() const
   \sa readBlock()
 */
 
+/*!
+  This convenience function is the same as calling
+  writeBlock( data.data(), data.size() ).
+*/
+int QIODevice::writeBlock( const QByteArray& data )
+{
+    return writeBlock( data.data(), data.size() );
+}
+
 /*!
   Reads a line of text, up to \e maxlen bytes including a terminating
   \0.  If there is a newline at the end if the line, it is not stripped.
