@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qstrlist.h,v 2.12.2.3 1998/08/25 09:20:55 hanord Exp $
+** $Id: qt/src/tools/qstrlist.h   2.1.1   edited 2000-03-24 $
 **
 ** Definition of QStrList, QStrIList and QStrListIterator classes
 **
 ** Created : 920730
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -31,36 +33,31 @@
 #endif // QT_H
 
 
-#if defined(DEFAULT_TEMPLATECLASS)
 #if defined(Q_TEMPLATEDLL)
 template class Q_EXPORT QList<char>;
 template class Q_EXPORT QListIterator<char>;
 #endif
-typedef QList<char>			QStrListBase;
-typedef QListIterator<char>		QStrListIterator;
-#else
-typedef Q_DECLARE(QListM,char)		QStrListBase;
-typedef Q_DECLARE(QListIteratorM,char)	QStrListIterator;
-#endif
+
+typedef QList<char>		QStrListBase;
+typedef QListIterator<char>	QStrListIterator;
 
 
 class Q_EXPORT QStrList : public QStrListBase
 {
 public:
-    QStrList( bool deepCopies=TRUE ) { dc = deepCopies; }
+    QStrList( bool deepCopies=TRUE ) { dc = deepCopies; del_item = deepCopies; }
     QStrList( const QStrList & );
    ~QStrList()			{ clear(); }
     QStrList& operator=( const QStrList & );
 
 private:
-    GCI	  newItem( GCI d )	{ return dc ? qstrdup( (const char*)d ) : d; }
-    void  deleteItem( GCI d )	{ if ( dc ) delete[] (char*)d; }
-    int	  compareItems( GCI s1, GCI s2 )
-				{ return strcmp((const char*)s1,
-						(const char*)s2); }
-    QDataStream &read( QDataStream &s, GCI &d )
+    Item newItem( Item d ) { return dc ? qstrdup( (const char*)d ) : d; }
+    void deleteItem( Item d ) { if ( del_item ) delete[] (char*)d; }
+    int compareItems( Item s1, Item s2 ) { return strcmp((const char*)s1,
+							 (const char*)s2); }
+    QDataStream &read( QDataStream &s, Item &d )
 				{ s >> (char *&)d; return s; }
-    QDataStream &write( QDataStream &s, GCI d ) const
+    QDataStream &write( QDataStream &s, Item d ) const
 				{ return s << (const char *)d; }
     bool  dc;
 };
@@ -72,7 +69,7 @@ public:
     QStrIList( bool deepCopies=TRUE ) : QStrList( deepCopies ) {}
    ~QStrIList()			{ clear(); }
 private:
-    int	  compareItems( GCI s1, GCI s2 )
+    int	  compareItems( Item s1, Item s2 )
 				{ return stricmp((const char*)s1,
 						 (const char*)s2); }
 };
@@ -82,6 +79,7 @@ inline QStrList & QStrList::operator=( const QStrList &strList )
 {
     clear();
     dc = strList.dc;
+    del_item = dc;
     QStrListBase::operator=(strList);
     return *this;
 }
