@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qbutton.cpp,v 2.64.2.2 1998/11/01 20:37:01 agulbra Exp $
+** $Id: qt/src/widgets/qbutton.cpp   2.1.1   edited 2000-05-02 $
 **
 ** Implementation of QButton widget class
 **
 ** Created : 940206
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,10 +27,12 @@
 #include "qbuttongroup.h"
 #include "qbitmap.h"
 #include "qpainter.h"
-#include "qkeycode.h"
 #include "qtimer.h"
 #include "qaccel.h"
 #include "qpixmapcache.h"
+#include "qfocusdata.h"
+#include "qapplication.h"
+#include "qpushbutton.h"
 #include <ctype.h>
 
 static const int autoRepeatDelay  = 300;
@@ -44,18 +48,22 @@ static const int drawingPixHeight = 100;
  */
 
 static QPixmap *drawpm = 0;
+
+
 static void cleanupButtonPm()
 {
     delete drawpm;
+    drawpm = 0;
 }
-static QPixmap *getDrawingPixmap()
+
+
+static inline void makeDrawingPixmap()
 {
     if ( !drawpm ) {
 	qAddPostRoutine( cleanupButtonPm );
 	drawpm = new QPixmap( drawingPixWidth, drawingPixHeight );
 	CHECK_PTR( drawpm );
     }
-    return drawpm;
 }
 
 
@@ -78,6 +86,14 @@ void QButton::ensureData()
 }
 
 
+/*! Returns a pointer to the group of which this button is a member.
+
+  If the button is not a member of any QButtonGroup, this function
+  returns 0.
+
+  \sa setGroup() QButtonGroup
+*/
+
 QButtonGroup *QButton::group() const
 {
     return d ? d->group : 0;
@@ -98,22 +114,7 @@ QTimer *QButton::timer()
 }
 
 
-/*
-  Internal function that returns the shortcut character in a string.
-  Returns zero if no shortcut character was found.
-  Example:
-    shortcutChar("E&xit") returns 'x'.
-*/
-
-static int shortcutChar( const char *str )
-{
-    const char *p = str ? strchr(str, '&') : 0;
-    while ( p && *p && p[1] == '&' )
-	p = strchr( p+2, '&' );
-    return (p && *p && p[1] && p[1] != '&') ? p[1] : 0;
-}
-
-
+// NOT REVISED
 /*!
   \class QButton qbutton.h
 
@@ -132,8 +133,7 @@ static int shortcutChar( const char *str )
   Any button can have either a text or pixmap label.  setText() sets
   the button to be a text button and setPixmap() sets it to be a
   pixmap button.  The text/pixmap is manipulated as necessary to
-  create "disabled" appearance when the button is \link
-  QWidget::setEnabled() disabled\endlink.
+  create "disabled" appearance when the button is disabled.
 
   QButton provides most of the states used for buttons:
   <ul>
@@ -166,20 +166,22 @@ static int shortcutChar( const char *str )
   <li>clicked() is emitted when the button is first pressed then
   released, or when the accelerator key is typed, or when animateClick()
   is called.
-  <li>toggled() is emitted when the state of a toggle button changes.
+  <li>toggled(bool) is emitted when the state of a toggle button changes.
+  <li>stateChanged(int) is emitted when the state of a tristate
+			toggle button changes.
   </ul>
 
-  If the button is a text button with "&" in its text, QButton creates an
-  automatic accelerator key.  This code creates a push button labelled "Rock
-  & Roll" (where the c is underscored).  The button gets an automatic
-  accelerator key, Alt-C:
+  If the button is a text button with "&" in its text, QButton creates
+  an automatic accelerator key.  This code creates a push button
+  labelled "Rock & Roll" (where the c is underscored).  The button
+  gets an automatic accelerator key, Alt-C:
 
   \code
     QPushButton *p = new QPushButton( "Ro&ck && Roll", this );
   \endcode
 
   In this example, when the user presses Alt-C the button will
-  \link animateClick() animate a click\endlink.
+  call animateClick().
 
   You can also set a custom accelerator using the setAccel() function.
   This is useful mostly for pixmap buttons since they have no
@@ -187,7 +189,7 @@ static int shortcutChar( const char *str )
 
   \code
     QPushButton *p;
-    p->setPixmap( QPixmap("print.gif") );
+    p->setPixmap( QPixmap("print.png") );
     p->setAccel( ALT+Key_F7 );
   \endcode
 
@@ -205,7 +207,36 @@ static int shortcutChar( const char *str )
   for a subclass of QButton unless you want to take over all drawing.
 
   \sa QButtonGroup
-  <a href="http://www.microsoft.com/win32dev/uiguide/uigui161.htm">Microsoft Style Guide</a>
+*/
+
+
+/*! \enum QButton::ToggleType
+
+  This enum type defines what a button can do in response to a
+  mouse/keyboard press: <ul>
+
+  <li> \c SingleShot - pressing the button causes an action, then the
+  button returns to the unpressed state.
+
+  <li> \c Toggle - pressing the button toggles it between an \c On and
+  and \c Off state.
+
+  <li> \c Tristate - pressing the button cycles between the three
+  states \c On, \c Off and \c NoChange
+
+  </ul>
+
+*/
+
+/*! \enum QButton::ToggleState
+
+  This enum defines the state of a toggle button at any moment.  The possible values are: <ul>
+
+  <li> \c Off - the button is in the "off" state
+  <li> \c NoChange - the button is in the default/unchanged state
+  <li> \c On - the button is in the "on" state
+
+  </ul>
 */
 
 
@@ -216,13 +247,13 @@ static int shortcutChar( const char *str )
   QButtonGroup::insert().
 */
 
-QButton::QButton( QWidget *parent, const char *name )
-    : QWidget( parent, name )
+QButton::QButton( QWidget *parent, const char *name, WFlags f )
+    : QWidget( parent, name, f )
 {
     bpixmap    = 0;
-    toggleBt   = FALSE;				// button is not on/off
+    toggleTyp  = SingleShot;			// button is simple
     buttonDown = FALSE;				// button is up
-    buttonOn   = FALSE;				// button is off
+    stat       = Off;				// button is off
     mlbDown    = FALSE;				// mouse left button up
     autoresize = FALSE;				// not auto resizing
     animation  = FALSE;				// no pending animateClick
@@ -236,12 +267,12 @@ QButton::QButton( QWidget *parent, const char *name )
 }
 
 /*!
-  Destroys the button and all its child widgets.
+  Destructs the button, deleting all its child widgets.
 */
 
 QButton::~QButton()
 {
-    if ( group() )				// remove from button group
+    if ( group() )
 	group()->remove( this );
     delete bpixmap;
     delete d;
@@ -263,10 +294,10 @@ QButton::~QButton()
 */
 
 /*!
-  \fn void QButton::clicked()
-  This signal is emitted when the button is activated (i.e. first
-  pressed down and then released when the mouse cursor is inside the
-  button).
+  \fn void QButton::clicked() This signal is emitted when the button
+  is activated, i.e. first pressed down and then released when the
+  mouse cursor is inside the button, or when the accelerator key is
+  typed, or when animateClick() is called.
 
   \sa pressed(), released(), toggled()
 */
@@ -282,42 +313,57 @@ QButton::~QButton()
   \sa clicked()
 */
 
+/*!
+  \fn void QButton::stateChanged( int state )
+  This signal is emitted whenever a toggle button changes status.
+  \e state is 2 if the button is on, 1 if it is in the
+  \link QCheckBox::setTristate() "no change" state\endlink
+  or 0 if the button is off.
+
+  This may be the result of a user action, toggle() slot activation,
+  setState(), or because setOn() was called.
+
+  \sa clicked()
+*/
+
 
 /*!
-  \fn const char *QButton::text() const
-  Returns the button text, or 0 if the button has no text.
+  \fn QString QButton::text() const
+  Returns the button text, or
+  \link QString::operator!() null string\endlink
+  if the button has no text.
   \sa setText()
 */
 
 /*!
-  Sets the button to display \e text and repaints
+  Sets the button to display \e text.
 
-  The button resizes itself if auto-resizing is enabled, changes its
-  minimum size if autoMinimumSize() is enabled, and sets the
-  appropriate accelerator.
+  If the text contains an ampersand, QButton creates an automatic
+  acceleror for it, such as  Alt-c for "&Cancel".
 
-  \sa text(), setPixmap(), setAutoMinimumSize(), setAutoResize(),
-  setAccel(), QPixmap::mask()
+  \sa text(), setPixmap(), setAccel(), QPixmap::mask()
 */
 
-void QButton::setText( const char *text )
+void QButton::setText( const QString &text )
 {
     if ( btext == text )
 	return;
-    int oldAccelChar = shortcutChar(btext);
-    int newAccelChar = shortcutChar(text);
     btext = text;
+
     if ( bpixmap ) {
 	delete bpixmap;
 	bpixmap = 0;
     }
+
     if ( autoresize )
 	adjustSize();
-    if ( oldAccelChar && !newAccelChar && !accel() )
-	setAccel( 0 );
-    if ( newAccelChar )
-	setAccel( ALT+toupper(newAccelChar) );
-    repaint( FALSE );
+
+    setAccel( QAccel::shortcutKey( btext ) );
+
+    if ( isVisible() ) {
+	update();
+	updateGeometry();
+    }
 }
 
 
@@ -327,7 +373,7 @@ void QButton::setText( const char *text )
 */
 
 /*!
-  Sets the button to display \a pixmap and repaints at once.
+  Sets the button to display \a pixmap
 
   If \a pixmap is monochrome (i.e. it is a QBitmap or its \link
   QPixmap::depth() depth\endlink is 1) and it does not have a mask,
@@ -335,21 +381,19 @@ void QButton::setText( const char *text )
   this is to draw transparent bitmaps, which is important for
   e.g. toggle buttons.
 
-  The button resizes itself if auto-resizing is enabled, changes its
-  minimum size if autoMinimumSize() is enabled, and always disables
-  any accelerator.
-
-  \sa pixmap(), setText(), setAutoMinimumSize(), setAutoResize(),
-  setAccel(), QPixmap::mask()
+  \sa pixmap(), setText(), setAccel(), QPixmap::mask()
 */
 
 void QButton::setPixmap( const QPixmap &pixmap )
 {
+    if ( bpixmap && bpixmap->serialNumber() == pixmap.serialNumber() )
+	return;
+
     bool newSize;
     if ( bpixmap ) {
 	newSize = pixmap.width() != bpixmap->width() ||
 		  pixmap.height() != bpixmap->height();
-        *bpixmap = pixmap;
+	*bpixmap = pixmap;
     } else {
 	newSize = TRUE;
 	bpixmap = new QPixmap( pixmap );
@@ -357,14 +401,17 @@ void QButton::setPixmap( const QPixmap &pixmap )
     }
     if ( bpixmap->depth() == 1 && !bpixmap->mask() )
 	bpixmap->setMask( *((QBitmap *)bpixmap) );
-    int oldAccelChar = shortcutChar(btext);
     if ( !btext.isNull() )
-	btext.resize( 0 );
+	btext = QString::null;
     if ( autoresize && newSize )
 	adjustSize();
-    if ( oldAccelChar )
-	setAccel( 0 );
-    repaint( FALSE );
+    setAccel( 0 );
+    if ( autoMask() )
+	updateMask();
+    if ( isVisible() ) {
+	update();
+	updateGeometry();
+    }
 }
 
 
@@ -376,7 +423,7 @@ void QButton::setPixmap( const QPixmap &pixmap )
 
 int QButton::accel() const
 {
-    return d && d->a ? d->a->key(0) : 0;
+    return d && d->a ? d->a->key( 0 ) : 0;
 }
 
 /*!
@@ -404,28 +451,25 @@ int QButton::accel() const
 
 void QButton::setAccel( int key )
 {
-    if ( key == -1 ) {				// guess from the text
-	int c = shortcutChar( text() );
-	key = c ? ALT+toupper(c) : 0;
-    }
-
+    if ( d && d->a )
+	d->a->clear();
+    if ( !key )
+	return;
     ensureData();
-    if ( key == 0 ) {				// delete accel
-	delete d->a;
-	d->a = 0;
-    } else {
-	if ( d->a )
-	    d->a->clear();
-	else
-	    d->a = new QAccel( this, "buttonAccel" );
-	d->a->connectItem( d->a->insertItem(key,0),
-			   this, SLOT(animateClick()) );
-    }
+    if ( !d->a )
+	d->a = new QAccel( this, "buttonAccel" );
+    d->a->connectItem( d->a->insertItem( key, 0 ),
+		       this, SLOT(animateClick()) );
 }
 
 
 /*!
   \fn bool QButton::autoResize() const
+
+  \obsolete
+
+  Strange pre-layout stuff.
+
   Returns TRUE if auto-resizing is enabled, or FALSE if auto-resizing is
   disabled.
 
@@ -435,7 +479,10 @@ void QButton::setAccel( int key )
 */
 
 
-/*!
+/*!\obsolete
+
+  Strange pre-layout stuff.
+
   Enables auto-resizing if \e enable is TRUE, or disables it if \e enable is
   FALSE.
 
@@ -491,8 +538,8 @@ void QButton::setAutoRepeat( bool enable )
   Performs an animated click: The button is pressed and a short while
   later released.
 
-  pressed(), released(), clicked() and toggled() signals are emitted as
-  appropriate.
+  pressed(), released(), clicked(), toggled(), and stateChanged()
+  signals are emitted as appropriate.
 
   This function does nothing if the button is \link setEnabled()
   disabled. \endlink
@@ -526,7 +573,7 @@ void QButton::animateClick()
   toggle() as well if you need to do that.  The pressed() and
   released() signals are not emitted by this function.
 
-  This method is provided in case you need to override the mouse event
+  This method is provided in case you need to reimplement the mouse event
   handlers.
 
   \sa isDown(), setOn(), toggle(), toggled()
@@ -534,6 +581,8 @@ void QButton::animateClick()
 
 void QButton::setDown( bool enable )
 {
+    if ( d )
+	timer()->stop();
     mlbDown = FALSE;				// the safe setting
     if ( (bool)buttonDown != enable ) {
 	buttonDown = enable;
@@ -541,34 +590,64 @@ void QButton::setDown( bool enable )
     }
 }
 
+/*!
+  \fn QButton::ToggleType QButton::toggleType() const
+
+  Returns the current toggle type.
+
+  \sa setToggleType()
+*/
+
+/*!
+  \fn void QButton::setState( ToggleState t)
+
+  This protected function sets the button state into state t but does
+  \e not cause repainting.
+
+  \sa setToggleType()
+*/
+
+/*!
+  \fn QButton::ToggleState QButton::state() const
+  Returns the state of the button.
+
+  \sa ToggleState ToggleType setState()
+*/
 
 /*!
   \fn bool QButton::isOn() const
   Returns TRUE if this toggle button is switched on, or FALSE if it is
   switched off.
-  \sa setOn(), toggleButton()
+  \sa setOn(), isToggleButton()
 */
 
 /*!
+  \fn void QButton::setOn( bool enable )
+
   Switches a toggle button on if \e enable is TRUE or off if \e enable is
   FALSE.  This function should be called only for toggle buttons.
-  \sa isOn(), toggleButton()
+  \sa isOn(), isToggleButton()
 */
 
-void QButton::setOn( bool enable )
+void QButton::setState( ToggleState s )
 {
-    if ( !toggleBt ) {
+    if ( !toggleTyp ) {
 #if defined(CHECK_STATE)
-	warning( "QButton::setOn(): (%s) Only toggle buttons may be switched",
-		 name( "unnamed" ) );
+	qWarning( "QButton::setState() / setOn: (%s) Only toggle buttons "
+		 "may be switched", name( "unnamed" ) );
 #endif
 	return;
     }
 
-    if ( (bool)buttonOn != enable ) {		// changed state
-	buttonOn = enable;
+    if ( (ToggleState)stat != s ) {		// changed state
+	bool was = stat != Off;
+	stat = s;
+    if ( autoMask() )
+        updateMask();
 	repaint( FALSE );
-	emit toggled( buttonOn );
+	if ( was != (stat != Off) )
+	    emit toggled( stat != Off );
+	emit stateChanged( s );
     }
 }
 
@@ -580,6 +659,8 @@ void QButton::setOn( bool enable )
 */
 
 /*!
+  \fn void QButton::setToggleButton( bool enable )
+
   Makes the button a toggle button if \e enable is TRUE, or a normal button
   if \e enable is FALSE.
 
@@ -591,19 +672,13 @@ void QButton::setOn( bool enable )
   \sa isToggleButton()
 */
 
-void QButton::setToggleButton( bool enable )
-{
-    toggleBt = enable;
-}
-
-
 /*!
-  Returns TRUE if \e pos is inside the widget rectangle, or FALSE if it
-  is outside.
+  Returns TRUE if \e pos is inside the clickable button rectangle, or
+  FALSE if it is outside.
 
-  This virtual function is reimplemented by subclasses.
+  Per default, the clickable area is the entire widget. Subclasses may
+  reimplement it, though.
 */
-
 bool QButton::hitButton( const QPoint &pos ) const
 {
     return rect().contains( pos );
@@ -612,7 +687,11 @@ bool QButton::hitButton( const QPoint &pos ) const
 /*!
   Draws the button.  The default implementation does nothing.
 
-  This virtual function is reimplemented by subclasses to draw real buttons.
+  This virtual function is reimplemented by subclasses to draw real
+  buttons. At some point in time, these reimplementations are supposed
+  to call drawButtonLabel().
+
+  \sa drawButtonLabel(), paintEvent()
 */
 
 void QButton::drawButton( QPainter * )
@@ -621,9 +700,12 @@ void QButton::drawButton( QPainter * )
 }
 
 /*!
-  Draws the button text or pixmap.  The default implementation does nothing.
+  Draws the button text or pixmap.
 
-  This virtual function is reimplemented by subclasses to draw real buttons.
+  This virtual function is reimplemented by subclasses to draw real
+  buttons. It's invoked by drawButton().
+
+  \sa drawButton(), paintEvent()
 */
 
 void QButton::drawButtonLabel( QPainter * )
@@ -632,28 +714,88 @@ void QButton::drawButtonLabel( QPainter * )
 }
 
 
-/*!
-  Handles keyboard events for the button.
-
-  Space calls animateClick(), the arrow keys cause focus changes.
+static bool got_a_release = FALSE; // ### binary compatibility trick, keyReleaseEvent is new
+/*!\reimp
 */
 
 void QButton::keyPressEvent( QKeyEvent *e )
 {
-    if ( e->key() == Key_Space )
-	animateClick();
-    else if ( e->key() == Key_Up || e->key() == Key_Left )
-	focusNextPrevChild( FALSE );
-    else if ( e->key() == Key_Down || e->key() == Key_Right )
-	focusNextPrevChild( TRUE );
-    else
+    switch ( e->key() ) {
+    case Key_Enter:
+    case Key_Return:
+	if ( inherits("QPushButton") )
+	    emit clicked();
+	else
+	    e->ignore();
+	break;
+    case Key_Space:
+	if ( !e->isAutoRepeat() ) {
+	    if ( got_a_release )
+		setDown( TRUE );
+	    else {
+		buttonDown = TRUE;
+		repaint( FALSE );
+	    }
+	}
+	break;
+    case Key_Up:
+    case Key_Left:
+	if ( group() )
+	    group()->moveFocus( e->key() );
+	else
+	    focusNextPrevChild( FALSE );
+	break;
+    case Key_Right:
+    case Key_Down:
+	if ( group() )
+	    group()->moveFocus( e->key() );
+	else
+	    focusNextPrevChild( TRUE );
+	break;
+    case Key_Escape:
+	if ( buttonDown ) {
+	    buttonDown = FALSE;
+	    update();
+	    break;
+	}
+	// fall through
+    default:
 	e->ignore();
+    }
+}
+
+/*!
+  \reimp
+ */
+void QButton::keyReleaseEvent( QKeyEvent * e)
+{
+    got_a_release = TRUE;
+    switch ( e->key() ) {
+    case Key_Space:
+	if ( buttonDown && !e->isAutoRepeat() ) {
+	    buttonDown = FALSE;
+	    nextState();
+	    emit released();
+	    emit clicked();
+	}
+	break;
+    default:
+	e->ignore();
+    }
 }
 
 
-/*!
-  Handles mouse press events for the button.
-  \sa mouseReleaseEvent()
+
+/*! \reimp */
+
+bool QButton::focusNextPrevChild( bool next )
+{
+    // we do not want this any more
+    return QWidget::focusNextPrevChild( next );
+}
+
+
+/*!\reimp
 */
 
 void QButton::mousePressEvent( QMouseEvent *e )
@@ -664,6 +806,9 @@ void QButton::mousePressEvent( QMouseEvent *e )
     if ( hit ) {				// mouse press on button
 	mlbDown = TRUE;				// left mouse button down
 	buttonDown = TRUE;
+    if ( autoMask() )
+        updateMask();
+
 	repaint( FALSE );
 	emit pressed();
 	if ( repeat )
@@ -671,9 +816,7 @@ void QButton::mousePressEvent( QMouseEvent *e )
     }
 }
 
-/*!
-  Handles mouse release events for the button.
-  \sa mousePressEvent()
+/*!\reimp
 */
 
 void QButton::mouseReleaseEvent( QMouseEvent *e)
@@ -681,27 +824,20 @@ void QButton::mouseReleaseEvent( QMouseEvent *e)
     if ( e->button() != LeftButton || !mlbDown )
 	return;
     if ( d )
-        timer()->stop();
+	timer()->stop();
     mlbDown = FALSE;				// left mouse button up
     buttonDown = FALSE;
     if ( hitButton( e->pos() ) ) {		// mouse release on button
-	int t = toggleBt && ( !isOn() || !group() || !group()->isExclusive() );
-	if ( t )
-	    buttonOn = !buttonOn;
-	repaint( FALSE );
-	if ( t )
-	    emit toggled( buttonOn );
-	emit released();
-	emit clicked();
+    nextState();
+        emit released();
+    emit clicked();
     } else {
-	repaint( FALSE );
-	emit released();
+	    repaint( FALSE );
+	    emit released();
     }
 }
 
-/*!
-  Handles mouse move events for the button.
-  \sa mousePressEvent(), mouseReleaseEvent()
+/*!\reimp
 */
 
 void QButton::mouseMoveEvent( QMouseEvent *e )
@@ -725,65 +861,53 @@ void QButton::mouseMoveEvent( QMouseEvent *e )
 
 
 /*!
-  Handles paint events for the button.
+  Handles paint events for buttons.  Small and typically complex
+  buttons (less than 300x100 pixels) are painted double-buffered to
+  reduce flicker. The actually drawing is done in the virtual functions
+  drawButton() and drawButtonLabel().
 
-  Opens the painter on the button and calls drawButton().
+  \sa drawButton(), drawButtonLabel()
 */
-
 void QButton::paintEvent( QPaintEvent *event )
 {
-    QPainter paint;
-
     if ( event &&
 	 width() <= drawingPixWidth &&
 	 height() <= drawingPixHeight ) {
-	QPixmap *pm = getDrawingPixmap();
-	ASSERT( pm );
-	
-	pm->fill( this, 0, 0 );
-	paint.begin( pm, this );
-	drawButton( &paint );
-	paint.end();
-	
-	bitBlt( this, event->rect().topLeft(), pm, event->rect() );
-    } else {
-	paint.begin( this );
-
-	// This optimization is worth it, since we often call repaint()
-	// to draw exactly the whole button.
-	if ( event && !event->rect().contains(rect()) )
-	    paint.setClipRect( event->rect() );
-
-	if ( event )
-	    erase( event->rect() );
+	makeDrawingPixmap(); // makes file-static drawpm variable
+	if ( backgroundOrigin() == ParentOrigin && !isTopLevel() )
+	    drawpm->fill( this, x(), y() );
 	else
-	    erase();
-
+	    drawpm->fill( this, 0, 0 );
+	QPainter paint;
+	paint.begin( drawpm, this );
 	drawButton( &paint );
 	paint.end();
+
+	paint.begin( this );
+	paint.drawPixmap( 0, 0, *drawpm );
+	paint.end();
+    } else {
+	erase( event->region() );
+	QPainter paint( this );
+	drawButton( &paint );
     }
-
-
 }
 
-/*!
-  Handles focus in events for the button.
-  \sa focusOutEvent()
+/*!\reimp
 */
 
-void QButton::focusInEvent( QFocusEvent * )
+void QButton::focusInEvent( QFocusEvent * e)
 {
-    repaint( FALSE );
+    QWidget::focusInEvent( e );
 }
 
-/*!
-  Handles focus out events for the button.
-  \sa focusInEvent()
+/*!\reimp
 */
 
-void QButton::focusOutEvent( QFocusEvent * )
+void QButton::focusOutEvent( QFocusEvent * e )
 {
-    repaint( FALSE );
+    buttonDown = FALSE;
+    QWidget::focusOutEvent( e );
 }
 
 
@@ -814,18 +938,33 @@ void QButton::animateTimeout()
 	return;
     animation  = FALSE;
     buttonDown = FALSE;
-    bool t = isToggleButton() &&
-	     (!isOn() || !group() || !group()->isExclusive());
-    if ( t )
-	buttonOn = !buttonOn;
-    repaint( FALSE );
-    if ( t )
-	emit toggled( buttonOn );
+    nextState();
     emit released();
     emit clicked();
 }
 
 
+void QButton::nextState()
+{
+    bool t = isToggleButton() && !( isOn() && isExclusiveToggle() );
+    bool was = stat != Off;
+    if ( t ) {
+	if ( toggleTyp == Tristate )
+	    stat = ( stat + 1 ) % 3;
+	else
+	    stat = stat ? Off : On;
+    }
+    if ( autoMask() )
+        updateMask();
+    repaint( FALSE );
+    if ( t ) {
+	if ( was != (stat != Off) )
+	    emit toggled( stat != Off );
+	emit stateChanged( stat );
+    }
+}
+
+
 /*! \reimp */
 
 void QButton::enabledChange( bool e )
@@ -843,3 +982,30 @@ void QButton::toggle()
     if ( isToggleButton() )
 	 setOn( !isOn() );
 }
+
+/*!
+  Sets the type of toggling behavior.  The default is \a SingleShot.
+
+  Subclasses use this, and present it with a more comfortable interface.
+*/
+void QButton::setToggleType( ToggleType type )
+{
+    toggleTyp = type;
+    if ( type != Tristate && stat == NoChange )
+	setState( On );
+}
+
+
+
+/*!
+  Returns TRUE if this button behaves exclusively inside a QButtonGroup.
+  In that case, this button can only be toggled off by another button
+  being toggled on.
+*/
+
+bool QButton::isExclusiveToggle() const
+{
+    return group() && ( group()->isExclusive() ||
+			group()->isRadioButtonExclusive() &&
+			inherits( "QRadioButton" ) );
+}
