@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qlcdnumber.cpp,v 2.14.2.1 1998/08/19 15:53:28 agulbra Exp $
+** $Id: qt/src/widgets/qlcdnumber.cpp   2.1.1   edited 2000-05-05 $
 **
 ** Implementation of QLCDNumber class
 **
 ** Created : 940518
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qlcdnumber.h"
 #include "qbitarray.h"
 #include "qpainter.h"
-#include <stdio.h>
 
+
+// NOT REVISED
 /*!
   \class QLCDNumber qlcdnumber.h
 
   \brief The QLCDNumber widget displays a number with LCD-like digits.
 
-  \ingroup realwidgets
+  \ingroup basic
 
   It can display a number in just about any size, in decimal,
   hexadecimal, octal or binary notation, and is easy to connect to
@@ -51,23 +54,46 @@
   in the string) and space.  QLCDNumber substitutes spaces for illegal
   characters.
 
-  It is impossible to retrieve the contents of a QLCDNumber object.
+  It is not possible to retrieve the contents of a QLCDNumber object.
   If you need to, we recommend that you connect the signals which feed
   the display() slot to another slot as well and store the value
   there.
 
-  \warning The current version of QLCDNumber does not work well
-  with background pixmaps.
-
   Incidentally, QLCDNumber is the very oldest part of Qt, tracing back
   to a BASIC program on the <a
-  href=http://www.nvg.unit.no/spectrum/>Sinclair Spectrum</a>.
+  href="http://www.nvg.ntnu.no/sinclair/spectrum.htm">Sinclair
+  Spectrum</a>.
 
-  <img src=qlcdnum-m.gif> <img src=qlcdnum-w.gif>
+  <img src=qlcdnum-m.png> <img src=qlcdnum-w.png>
 
   \sa QLabel, QFrame
 */
 
+/*! \enum QLCDNumber::Mode
+
+  This type determines how numbers are shown. The possible values are:
+
+  <ul>
+  <li>\c Hex - Hexadecimal
+  <li>\c Dec - Decimal
+  <li>\c Oct - Octal
+  <li>\c Bin - Binary
+  </ul>
+ */
+
+/*! \enum QLCDNumber::SegmentStyle
+
+  This type determines the visual appearance of the QLCDNumber
+  widget. The possible values are:
+
+  <ul>
+  <li>\c Outline gives raised segments filled with the background brush.
+  <li>\c Filled gives raised segments filled with the foreground brush.
+  <li>\c Flat gives flat segments filled with the foreground brush.
+  </ul>
+ */
+
+
 
 /*!
   \fn void QLCDNumber::overflow()
@@ -91,29 +117,30 @@ static QString int2string( int num, int base, int ndigits, bool *oflow )
     }
     switch( base ) {
 	case QLCDNumber::HEX:
-	    s.sprintf( "%*lx", ndigits, num );
+	    s.sprintf( "%*x", ndigits, num );
 	    break;
 	case QLCDNumber::DEC:
-	    s.sprintf( "%*li", ndigits, num );
+	    s.sprintf( "%*i", ndigits, num );
 	    break;
 	case QLCDNumber::OCT:
-	    s.sprintf( "%*lo", ndigits, num );
+	    s.sprintf( "%*o", ndigits, num );
 	    break;
-	case QLCDNumber::BIN: {
-	    char buf[42];
-	    char *p = &buf[41];
-	    uint n = num;
-	    int len = 0;
-	    *p = '\0';
-	    do {
-		*--p = (char)((n&1)+'0');
-		n >>= 1;
-		len++;
-	    } while ( n != 0 );
-	    len = ndigits - len;
-	    if ( len > 0 )
-	    s.fill( ' ', len );
-	    s += p;
+	case QLCDNumber::BIN:
+	    {
+		char buf[42];
+		char *p = &buf[41];
+		uint n = num;
+		int len = 0;
+		*p = '\0';
+		do {
+		    *--p = (char)((n&1)+'0');
+		    n >>= 1;
+		    len++;
+		} while ( n != 0 );
+		len = ndigits - len;
+		if ( len > 0 )
+		s.fill( ' ', len );
+		s += QString::fromLatin1(p);
 	    }
 	    break;
     }
@@ -160,9 +187,9 @@ static QString double2string( double num, int base, int ndigits, bool *oflow )
 }
 
 
-static char *getSegments( char ch )		// gets list of segments for ch
+static const char *getSegments( char ch )		// gets list of segments for ch
 {
-    static char segments[30][8] =
+    static const char segments[30][8] =
        { { 0, 1, 2, 4, 5, 6,99, 0},		// 0	0 / O
 	 { 2, 5,99, 0, 0, 0, 0, 0},		// 1	1
 	 { 0, 2, 3, 4, 6,99, 0, 0},		// 2	2
@@ -294,10 +321,11 @@ void QLCDNumber::init()
     smallPoint = FALSE;
     setNumDigits( ndigits );
     setSegmentStyle( Outline );
+    d = 0;
 }
 
 /*!
-  Destroys the LCD number.
+  Destructs the LCD number.
 */
 
 QLCDNumber::~QLCDNumber()
@@ -328,7 +356,7 @@ void QLCDNumber::setNumDigits( int numDigits )
 {
     if ( numDigits > 99 ) {
 #if defined(CHECK_RANGE)
-	warning( "QLCDNumber::setNumDigits: (%s) Max 99 digits allowed",
+	qWarning( "QLCDNumber::setNumDigits: (%s) Max 99 digits allowed",
 		 name( "unnamed" ) );
 #endif
 	numDigits = 99;
@@ -412,7 +440,7 @@ QLCDNumber::Mode QLCDNumber::mode() const
 /*!
   Returns the last value set by one of the display() slots.
 
-  If display(const char *) was the last called, 0 is returned.
+  If display(QString ) was the last called, 0 is returned.
 
   \sa intValue(), display()
 */
@@ -426,7 +454,7 @@ double QLCDNumber::value() const
   Returns the last value set by one of the display() slots rounded to the
   nearest integer.
 
-  If display(const char *) was the last called, 0 is returned.
+  If display(QString ) was the last called, 0 is returned.
 
   \sa value(), display()
 */
@@ -450,7 +478,7 @@ void QLCDNumber::display( int num )
     if ( of )
 	emit overflow();
     else
-	internalDisplay( s );
+	internalSetString( s );
 }
 
 
@@ -466,12 +494,12 @@ void QLCDNumber::display( double num )
     if ( of )
 	emit overflow();
     else
-	internalDisplay( s );
+	internalSetString( s );
 }
 
 
 /*!
-  \overload void QLCDNumber::display( const char *s )
+  \overload void QLCDNumber::display( const QString& s )
 
   This version of the function disregards mode() and smallDecimalPoint().
 
@@ -482,10 +510,10 @@ void QLCDNumber::display( double num )
   characters.
 */
 
-void QLCDNumber::display( const char *s )
+void QLCDNumber::display( const QString &s )
 {
     val = 0;
-    internalDisplay( s );
+    internalSetString( s );
 }
 
 /*!
@@ -545,8 +573,6 @@ void QLCDNumber::setBinMode()
   and \c HEX.  All four modes can display both integers, floating-point
   numbers and strings (subject to character set limitations).
 
-  The display is set to "0".
-
   Example:
   \code
     myLcd.setMode( QLCDNumber::HEX );
@@ -559,7 +585,8 @@ void QLCDNumber::setBinMode()
 void QLCDNumber::setMode( Mode m )
 {
     base = m;
-    display( 0 );
+
+    repaint( contentsRect() );
 }
 
 
@@ -580,28 +607,12 @@ void QLCDNumber::setMode( Mode m )
   The inter-digit space is made slightly wider when the decimal point
   is drawn between the digits.
 
-  The display is set to "0.0".
-
   \sa smallDecimalPoint(), setMode()
 */
 
 void QLCDNumber::setSmallDecimalPoint( bool b )
 {
     smallPoint = b;
-    display( 0.0 );
-}
-
-
-/*!
-  Handles resize Events for the QLCDNumber.
-
-  \internal
-
-  FIXME: Why is this implemented and empty?
-*/
-
-void QLCDNumber::resizeEvent( QResizeEvent * )
-{
 }
 
 
@@ -610,6 +621,7 @@ void QLCDNumber::resizeEvent( QResizeEvent * )
   QFrame::paintEvent().
 */
 
+
 void QLCDNumber::drawContents( QPainter *p )
 {
     if ( smallPoint )
@@ -618,32 +630,31 @@ void QLCDNumber::drawContents( QPainter *p )
 	drawString( digitStr, *p, 0, FALSE );
 }
 
+
 /*!
   \internal
 */
 
-void QLCDNumber::internalDisplay( const char *s )
+void QLCDNumber::internalDisplay( const QString & )
 {
-    QString buffer(ndigits+1);
+    ; // Not used anymore
+}
+
+void QLCDNumber::internalSetString( const QString& s )
+{
+    QString buffer;
     int i;
-    int len = strlen(s);
-    QPainter p( this );
+    int len = s.length();
+    QBitArray newPoints(ndigits);
 
     if ( !smallPoint ) {
-	if ( len >= ndigits ) {
-	    for( i=0; i<(int)ndigits; i++ )	// trunc too long string
-		buffer[i] = s[len - ndigits + i];
-	} else {
-	    for( i=0; i<ndigits-len; i++ )	// pad with spaces
-		buffer[i] = ' ';
-	    for( i=0; i<len; i++ )
-		buffer[ndigits - len + i] = s[i];
-	}
-	drawString( buffer, p );
-   } else {
+	if ( len == ndigits )
+	    buffer = s;
+	else
+	    buffer = s.right( ndigits ).rightJustify( ndigits, ' ' );
+    } else {
 	int  index = -1;
 	bool lastWasPoint = TRUE;
-	QBitArray newPoints(ndigits);
 	newPoints.clearBit(0);
 	for ( i=0; i<len; i++ ) {
 	    if ( s[i] == '.' ) {
@@ -675,7 +686,24 @@ void QLCDNumber::internalDisplay( const char *s )
 		newPoints.clearBit(i);
 	    }
 	}
-	drawString(buffer,p,&newPoints);
+    }
+
+    if ( buffer == digitStr )
+	return;
+
+    if ( backgroundMode() == FixedPixmap
+	 || colorGroup().brush( QColorGroup::Background ).pixmap() ) {
+	digitStr = buffer;
+	if ( smallPoint )
+	    points = newPoints;
+	repaint( contentsRect() );
+    }
+    else {
+	QPainter p( this );
+	if ( !smallPoint )
+	    drawString( buffer, p );
+	else
+	    drawString( buffer, p, &newPoints );
     }
 }
 
@@ -683,7 +711,7 @@ void QLCDNumber::internalDisplay( const char *s )
   \internal
 */
 
-void QLCDNumber::drawString( const char *s, QPainter &p,
+void QLCDNumber::drawString( const QString &s, QPainter &p,
 			     QBitArray *newPoints, bool newString )
 {
     QPoint  pos;
@@ -699,7 +727,7 @@ void QLCDNumber::drawString( const char *s, QPainter &p,
     for ( int i=0;  i<ndigits; i++ ) {
 	pos = QPoint( xOffset + xAdvance*i, yOffset );
 	if ( newString )
-	    drawDigit( pos, p, segLen, s[i], digitStr[i] );
+	    drawDigit( pos, p, segLen, s[i], digitStr[i].latin1() );
 	else
 	    drawDigit( pos, p, segLen, s[i]);
 	if ( newPoints ) {
@@ -715,7 +743,7 @@ void QLCDNumber::drawString( const char *s, QPainter &p,
     if ( newString ) {
 	digitStr = s;
 	if ( (int)digitStr.length() > ndigits )
-	    digitStr.resize( ndigits );
+	    digitStr.truncate( ndigits );
 	if ( newPoints )
 	    points = *newPoints;
     }
@@ -736,7 +764,7 @@ void QLCDNumber::drawDigit( const QPoint &pos, QPainter &p, int segLen,
 				// first 9 used if segment table is correct
     int	 nErases;
     int	 nUpdates;
-    char *segs;
+    const char *segs;
     int	 i,j;
 
     const char erase	  = 0;
@@ -786,7 +814,9 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 			      int segLen, bool erase )
 {
     QPoint pt = pos;
-    QColorGroup g = colorGroup();
+    int width = segLen/5;
+
+    const QColorGroup & g = colorGroup();
     QColor lightColor,darkColor,fgColor;
     if ( erase ){
 	lightColor = backgroundColor();
@@ -797,7 +827,6 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	darkColor  = g.dark();
 	fgColor    = g.foreground();
     }
-    int width = segLen/5;
 
 #define LINETO(X,Y) addPoint( a, QPoint(pt.x() + (X),pt.y() + (Y)))
 #define LIGHT
@@ -922,7 +951,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    break;
 #if defined(CHECK_RANGE)
 	default :
-	    warning( "QLCDNumber::drawSegment: (%s) Internal error."
+	    qWarning( "QLCDNumber::drawSegment: (%s) Internal error."
 		     "  Illegal segment id: %d\n",
 		     name( "unnamed" ), segmentNo );
 #endif
@@ -1057,7 +1086,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    break;
 #if defined(CHECK_RANGE)
 	default :
-	    warning( "QLCDNumber::drawSegment: (%s) Internal error."
+	    qWarning( "QLCDNumber::drawSegment: (%s) Internal error."
 		     "  Illegal segment id: %d\n",
 		     name( "unnamed" ), segmentNo );
 #endif
@@ -1069,6 +1098,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 }
 
 
+
 /*!
   Sets the style of the QLCDNumber to \a s.
   <ul>
@@ -1105,16 +1135,17 @@ QLCDNumber::SegmentStyle QLCDNumber::segmentStyle() const
 }
 
 
-/*!  Returns a suitable size for this LCD number.  The size hint is on
-  the small side, so as to be usable as minimum size.
-  
-  The size hint takes numDigits() and smallDecimalPoint() into
-  consideration.
-
-  \sa setNumDigits() setSmallDecimalPoint() setMinimumSize()
+/*!\reimp
 */
-
 QSize QLCDNumber::sizeHint() const
 {
-    return QSize( 18 * (numDigits() + (smallDecimalPoint() ? 0 : 1)), 30 );
+    return QSize( 10 + 9 * (numDigits() + (smallDecimalPoint() ? 0 : 1)), 23 );
+}
+
+
+/*!\reimp
+*/
+QSizePolicy QLCDNumber::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
 }
