@@ -1,21 +1,23 @@
 /****************************************************************************
-** $Id: qtooltip.cpp,v 2.51.2.1 1998/08/31 15:18:11 hanord Exp $
+** $Id: qt/src/widgets/qtooltip.cpp   2.1.1   edited 2000-05-24 $
 **
 ** Tool Tips (or Balloon Help) for any widget or rectangle
 **
-** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.44.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+** information about the Professional Edition licensing, or see
+** http://www.trolltech.com/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -23,6 +25,9 @@
 #include "qlabel.h"
 #include "qptrdict.h"
 #include "qapplication.h"
+#include "qguardedptr.h"
+
+static bool globally_enabled = TRUE;
 
 // Magic value meaning an entire widget - if someone tries to insert a
 // tool tip on this part of a widget it will be interpreted as the
@@ -30,16 +35,37 @@
 
 static inline QRect entireWidget()
 {
-    return QRect( QCOORD_MIN, QCOORD_MIN,
-		  QCOORD_MAX-QCOORD_MIN, QCOORD_MAX-QCOORD_MIN );
+    return QRect( -QWIDGETSIZE_MAX, -QWIDGETSIZE_MAX,
+		  2*QWIDGETSIZE_MAX, 2*QWIDGETSIZE_MAX );
 }
 
+// Internal class - don't touch
+
+class QTipLabel : public QLabel
+{
+    Q_OBJECT
+public:
+    QTipLabel(const QString& text) : QLabel( 0, "toolTipTip",
+			  WStyle_StaysOnTop +
+			  WStyle_Customize + WStyle_NoBorder + WStyle_Tool )
+    {
+	setMargin(1);
+	setIndent(0);
+	setAutoMask( FALSE );
+	setFrameStyle( QFrame::Plain | QFrame::Box );
+	setLineWidth( 1 );
+	setAlignment( AlignLeft | AlignTop );
+	polish();
+	setText(text);
+	adjustSize();
+    }
+};
 
 // Internal class - don't touch
 
 class QTipManager : public QObject
 {
-    Q_OBJECT	/* tmake ignore Q_OBJECT */
+    Q_OBJECT
 public:
     QTipManager();
    ~QTipManager();
@@ -52,101 +78,42 @@ public:
 	QToolTipGroup  *group;
 	QToolTip       *tip;
 	bool	        autoDelete;
+	QRect 		geometry;
 	Tip	       *next;
     };
 
     bool    eventFilter( QObject * o, QEvent * e );
-    void    add( QWidget *, const QRect &, const char *,
-		 QToolTipGroup *, const char *, QToolTip *, bool );
+    void    add( const QRect &gm, QWidget *, const QRect &, const QString& ,
+		 QToolTipGroup *, const QString& , QToolTip *, bool );
+    void    add( QWidget *, const QRect &, const QString& ,
+		 QToolTipGroup *, const QString& , QToolTip *, bool );
     void    remove( QWidget *, const QRect & );
     void    remove( QWidget * );
 
     void    removeFromGroup( QToolTipGroup * );
 
+public slots:
+    void    hideTip();
+
 private slots:
     void    labelDestroyed();
     void    clientWidgetDestroyed();
     void    showTip();
-    void    hideTip();
-
-protected:
-    void    maybeTip( const QPoint & );
 
 private:
     QTimer  wakeUp;
     QTimer  fallAsleep;
-    QTimer  leaveWindow;
 
-    QPtrDict<Tip>    *tips;
-    QLabel	     *label;
-    QPoint	      pos;
-    QWidget	     *widget;
+    QPtrDict<Tip> *tips;
+    QLabel *label;
+    QPoint pos;
+    QGuardedPtr<QWidget> widget;
     Tip *currentTip;
     Tip *previousTip;
     bool isApplicationFilter;
 };
 
 
-// moc stuff - included by hand.
-
-/****************************************************************************
-** QTipManager meta object code from reading C++ file 'qtooltip.cpp'
-**
-** Created: Mon Aug 31 17:16:31 1998
-**      by: The Qt Meta Object Compiler ($Revision: 2.51.2.1 $)
-**
-** WARNING! All changes made in this file will be lost!
-*****************************************************************************/
-
-#if !defined(Q_MOC_OUTPUT_REVISION)
-#define Q_MOC_OUTPUT_REVISION 2
-#elif Q_MOC_OUTPUT_REVISION != 2
-#error "Moc format conflict - please regenerate all moc files"
-#endif
-
-#include <qmetaobject.h>
-
-
-const char *QTipManager::className() const
-{
-    return "QTipManager";
-}
-
-QMetaObject *QTipManager::metaObj = 0;
-
-void QTipManager::initMetaObject()
-{
-    if ( metaObj )
-	return;
-    if ( strcmp(QObject::className(), "QObject") != 0 )
-	badSuperclassWarning("QTipManager","QObject");
-    if ( !QObject::metaObject() )
-	QObject::initMetaObject();
-    typedef void(QTipManager::*m1_t0)();
-    typedef void(QTipManager::*m1_t1)();
-    typedef void(QTipManager::*m1_t2)();
-    typedef void(QTipManager::*m1_t3)();
-    m1_t0 v1_0 = &QTipManager::labelDestroyed;
-    m1_t1 v1_1 = &QTipManager::clientWidgetDestroyed;
-    m1_t2 v1_2 = &QTipManager::showTip;
-    m1_t3 v1_3 = &QTipManager::hideTip;
-    QMetaData *slot_tbl = new QMetaData[4];
-    slot_tbl[0].name = "labelDestroyed()";
-    slot_tbl[1].name = "clientWidgetDestroyed()";
-    slot_tbl[2].name = "showTip()";
-    slot_tbl[3].name = "hideTip()";
-    slot_tbl[0].ptr = *((QMember*)&v1_0);
-    slot_tbl[1].ptr = *((QMember*)&v1_1);
-    slot_tbl[2].ptr = *((QMember*)&v1_2);
-    slot_tbl[3].ptr = *((QMember*)&v1_3);
-    metaObj = new QMetaObject( "QTipManager", "QObject",
-	slot_tbl, 4,
-	0, 0 );
-}
-
-// moc code ends here
-
-
 // We have a global, internal QTipManager object
 
 static QTipManager *tipManager	  = 0;
@@ -182,7 +149,6 @@ QTipManager::QTipManager()
     isApplicationFilter = FALSE;
     connect( &wakeUp, SIGNAL(timeout()), SLOT(showTip()) );
     connect( &fallAsleep, SIGNAL(timeout()), SLOT(hideTip()) );
-    connect( &leaveWindow, SIGNAL(timeout()), SLOT(hideTip()) );
 }
 
 
@@ -213,33 +179,31 @@ QTipManager::~QTipManager()
     delete label;
 }
 
-void QTipManager::add( QWidget *w, const QRect &r, const char *s,
-		       QToolTipGroup *g, const char *gs,
+void QTipManager::add( const QRect &gm, QWidget *w, const QRect &r, const QString &s,
+		       QToolTipGroup *g, const QString& gs,
 		       QToolTip *tt, bool a )
 {
     QTipManager::Tip *h = (*tips)[ w ];
     QTipManager::Tip *t = new QTipManager::Tip;
-    if ( h )
-	tips->take( w );
     t->next = h;
-
     t->tip = tt;
     t->autoDelete = a;
     t->text = s;
     t->rect = r;
     t->groupText = gs;
     t->group = g;
+    t->geometry = gm;
+
+    if ( h )
+	tips->take( w );
+    else
+	connect( w, SIGNAL(destroyed()), this, SLOT(clientWidgetDestroyed()) );
 
     tips->insert( w, t );
-    connect( w, SIGNAL(destroyed()), this, SLOT(clientWidgetDestroyed()) );
-    if ( a ) {
-	if ( t->rect.contains( pos ) )
-	    showTip();
-	tips->take( w );
-	if( t->next )
-	    tips->insert( w, t->next );
-	t->next = 0;
-    }
+
+    if ( a && t->rect.contains( pos ) && (!g || g->enabled()) )
+	showTip();
+
     if ( !isApplicationFilter && qApp ) {
 	isApplicationFilter = TRUE;
 	qApp->installEventFilter( tipManager );
@@ -247,6 +211,14 @@ void QTipManager::add( QWidget *w, const QRect &r, const char *s,
     }
 }
 
+void QTipManager::add( QWidget *w, const QRect &r, const QString &s,
+		       QToolTipGroup *g, const QString& gs,
+		       QToolTip *tt, bool a )
+{
+    add( QRect( -1, -1, -1, -1 ),
+	 w, r, s, g, gs, tt, a );
+}
+
 
 void QTipManager::remove( QWidget *w, const QRect & r )
 {
@@ -257,15 +229,13 @@ void QTipManager::remove( QWidget *w, const QRect & r )
     if ( t == currentTip )
 	hideTip();
 
+    if ( t == previousTip )
+	previousTip = 0;
+
     if ( t->rect == r ) {
 	tips->take( w );
-	if ( t->next ) {
+	if ( t->next )
 	    tips->insert( w, t->next );
-	} else {
-	    // ### w->setMouseTracking( FALSE );
-	    // ### need to disable sometimes
-	    // w->removeEventFilter( tipManager );
-	}
 	delete t;
     } else {
 	while( t->next && t->next->rect != r )
@@ -350,40 +320,51 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 {
     // avoid dumping core in case of application madness, and return
     // quickly for some common but irrelevant events
-    if ( !tips || !e || !obj || !obj->isWidgetType() ||
-	 e->type() == Event_Timer || e->type() == Event_Paint ||
-	 e->type() == Event_SockAct )
+    if ( !qApp || !qApp->focusWidget() ||
+	 !obj || !obj->isWidgetType() || // isWidgetType() catches most stuff
+	 !e ||
+	 e->type() == QEvent::Paint ||
+	 e->type() == QEvent::Timer ||
+	 e->type() == QEvent::SockAct ||
+	 !tips )
 	return FALSE;
     QWidget *w = (QWidget *)obj;
 
+    if ( e->type() == QEvent::FocusOut || e->type() == QEvent::FocusIn ) {
+	// user moved focus somewhere - hide the tip
+	hideTip();
+	fallAsleep.stop();
+	return FALSE;
+    }
+
     QTipManager::Tip *t = 0;
     while( w && !t ) {
 	t = (*tips)[ w ];
 	if ( !t )
-	    w = w->testWFlags(WType_TopLevel) ? 0 : w->parentWidget();
+	    w = w->isTopLevel() ? 0 : w->parentWidget();
     }
 
     if ( !t ) {
-	if ( e->type() >= Event_MouseButtonPress &&
-	     e->type() <= Event_Leave )
-	    wakeUp.stop();
+	if ( e->type() >= QEvent::MouseButtonPress &&
+	     e->type() <= QEvent::Leave ) {
+	    hideTip();
+	}
 	return FALSE;
     }
 
     // with that out of the way, let's get down to action
 
     switch( e->type() ) {
-    case Event_MouseButtonPress:
-    case Event_MouseButtonRelease:
-    case Event_MouseButtonDblClick:
-    case Event_KeyPress:
-    case Event_KeyRelease:
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseButtonRelease:
+    case QEvent::MouseButtonDblClick:
+    case QEvent::KeyPress:
+    case QEvent::KeyRelease:
 	// input - turn off tool tip mode
 	hideTip();
 	fallAsleep.stop();
-	leaveWindow.stop();
 	break;
-    case Event_MouseMove:
+    case QEvent::MouseMove:
 	{ // a whole scope just for one variable
 	    QMouseEvent * m = (QMouseEvent *)e;
 
@@ -399,13 +380,18 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 
 	    wakeUp.stop();
 	    if ( m->state() == 0 ) {
-		if ( (label && label->isVisible()) )
-		    return TRUE;
-		else if ( fallAsleep.isActive() )
-		    wakeUp.start( 100, TRUE );
-		else {
-		    previousTip = 0;
-		    wakeUp.start( 700, TRUE );
+		if ( label && label->isVisible() ) {
+		    return FALSE;
+		} else {
+		    if ( fallAsleep.isActive() ) {
+			wakeUp.start( 1, TRUE );
+		    } else {
+			previousTip = 0;
+			wakeUp.start( 700, TRUE );
+		    }
+		    if ( t->group && t->group->ena &&
+			    !t->group->del && !t->groupText.isEmpty() )
+			emit t->group->showTip( t->groupText );
 		}
 		widget = w;
 		pos = mousePos;
@@ -415,20 +401,13 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 	    }
 	}
 	break;
-    case Event_Enter:
-	if ( label && label->isVisible() && w == widget )
-	    leaveWindow.stop();
-	else if ( w ) // test in event_leave below should always hit first
-	    wakeUp.stop();
-	break;
-    case Event_Leave:
-	if ( label && label->isVisible() )
-	    leaveWindow.start( 50, TRUE );
-	else if ( widget != w )
-	    wakeUp.stop();
+    case QEvent::Leave:
+    case QEvent::Hide:
+    case QEvent::Destroy:
+	if ( w == widget )
+	    hideTip();
 	break;
     default:
-	hideTip();
 	break;
     }
     return FALSE;
@@ -438,15 +417,13 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 
 void QTipManager::showTip()
 {
-    if ( widget == 0 ) {
-	widget = 0;
+    if ( !widget || !globally_enabled )
 	return;
-    }
 
     QTipManager::Tip *t = (*tips)[ widget ];
     while ( t && !t->rect.contains( pos ) )
 	t = t->next;
-    if ( t == 0 || t == previousTip )
+    if ( t == 0 )
 	return;
 
     if ( t->tip ) {
@@ -454,38 +431,42 @@ void QTipManager::showTip()
 	return;
     }
 
+    if ( t->group && !t->group->ena )
+	return;
+
     if ( label ) {
 	label->setText( t->text );
+	label->adjustSize();
+	if ( t->geometry != QRect( -1, -1, -1, -1 ) )
+	    label->resize( t->geometry.size() );
     } else {
-	label = new QLabel( 0, "toolTipTip",
-			    WStyle_Customize | WStyle_NoBorder | WStyle_Tool );
+	label = new QTipLabel(t->text);
+	if ( t->geometry != QRect( -1, -1, -1, -1 ) )
+	    label->resize( t->geometry.size() );
 	CHECK_PTR( label );
 	connect( label, SIGNAL(destroyed()), SLOT(labelDestroyed()) );
-	label->setFont( QToolTip::font() );
-	label->setPalette( QToolTip::palette() );
-	label->setText( t->text );
-	if ( QApplication::style() == MotifStyle )
-	    label->setFrameStyle( QFrame::Plain | QFrame::Box );
-	else
-	    label->setFrameStyle( QFrame::Raised | QFrame::Panel );
-	label->setLineWidth( 1 );
-	label->setMargin( 3 );
-	label->setAlignment( AlignLeft | AlignTop );
-	label->setAutoResize( TRUE );
     }
-    QPoint p = widget->mapToGlobal( pos ) + QPoint( 2, 16 );
+    QPoint p;
+    if ( t->geometry == QRect( -1, -1, -1, -1 ) ) {
+	p = widget->mapToGlobal( pos ) + QPoint( 2, 16 );
+    } else {
+	p = widget->mapToGlobal( t->geometry.topLeft() );
+	label->setAlignment( WordBreak | AlignCenter );
+	int h = label->heightForWidth( t->geometry.width() - 4 );
+	label->resize( label->width(), h );
+    }
     if ( p.x() + label->width() > QApplication::desktop()->width() )
 	p.setX( QApplication::desktop()->width() - label->width() );
     if ( p.y() + label->height() > QApplication::desktop()->height() )
 	p.setY( p.y() - 20 - label->height() );
-    label->move( p );
-    label->show();
-    label->raise();
+    if ( label->text().length() ) {
+	label->move( p );
+	label->show();
+	label->raise();
+	fallAsleep.start( 10000, TRUE );
+    }
 
-    fallAsleep.start( 5000, TRUE );
-    leaveWindow.stop();
-
-    if ( t->group && !t->groupText.isEmpty() )
+    if ( t->group && t->group->del && !t->groupText.isEmpty() )
 	emit t->group->showTip( t->groupText );
     currentTip = t;
     previousTip = 0;
@@ -496,30 +477,33 @@ void QTipManager::hideTip()
 {
     if ( label && label->isVisible() ) {
 	label->hide();
-	fallAsleep.start( 10000, TRUE );
+	fallAsleep.start( 2000, TRUE );
+	wakeUp.stop();
 	if ( currentTip && currentTip->group )
 	    emit currentTip->group->removeTip();
     } else if ( wakeUp.isActive() ) {
 	wakeUp.stop();
+	if ( currentTip && currentTip->group &&
+	     !currentTip->group->del && !currentTip->groupText.isEmpty() )
+	    emit currentTip->group->removeTip();
     }
 
-    if ( currentTip && currentTip->autoDelete )
-	delete currentTip;
-    else
-	previousTip = currentTip;
-
+    previousTip = currentTip;
     currentTip = 0;
+    if ( previousTip && previousTip->autoDelete )
+	remove( widget, previousTip->rect );
     widget = 0;
 }
 
 
+// NOT REVISED
 /*!
   \class QToolTip qtooltip.h
 
   \brief The QToolTip class provides tool tips (sometimes called
   balloon help) for any widget or rectangular part of a widget.
 
-  \ingroup application
+  \ingroup helpsystem
 
   The tip is a short, one-line text reminding the user of the widget's
   or rectangle's function.  It is drawn immediately below the region,
@@ -545,7 +529,7 @@ void QTipManager::hideTip()
   rectangle within a widget. <li> Adding a tip to a dynamic rectangle
   within a widget. </ol>
 
-  To add a tip to a widget, call the static function QToolTip::add()
+  To add a tip to a widget, call the \e static function QToolTip::add()
   with the widget and tip as arguments:
 
   \code
@@ -567,9 +551,9 @@ void QTipManager::hideTip()
 
   \code
     QToolTip::add( quitButton, "Leave the application", g,
-                   "Leave the application, without asking for confirmation" );
+		   "Leave the application, without asking for confirmation" );
     QToolTip::add( closeButton, "Close this window", g,
-                   "Close this window, without asking for confirmation" );
+		   "Close this window, without asking for confirmation" );
   \endcode
 
   To add a tip to a fixed rectangle within a widget, call the static
@@ -595,6 +579,9 @@ void QTipManager::hideTip()
   can forcibly remove the tip by calling remove() with no arguments.
   This is handy if the widget scrolls.
 
+  Tooltips can be globally disabled using QToolTip::setEnabled(), or
+  disabled in groups with QToolTipGroup::setEnabled().
+
   \sa QStatusBar QWhatsThis QToolTipGroup
   <a href="guibooks.html#fowler">GUI Design Handbook: Tool Tip</a>
 */
@@ -604,31 +591,12 @@ void QTipManager::hideTip()
   Global settings for tool tips.
 */
 
-QFont	    *QToolTip::ttFont    = 0;
-QPalette    *QToolTip::ttPalette = 0;
-
-
-void QToolTip::initialize()
+void QToolTip::initialize() // ## remove 3.0
 {
-    if ( ttFont )				// already initialized
-	return;
-    qAddPostRoutine( cleanup );
-    ttFont = new QFont;
-    CHECK_PTR( ttFont );
-    QColorGroup cg( black, QColor(255,255,220),
-		    QColor(96,96,96), black, black,
-		    black, QColor(255,255,220) );
-    ttPalette = new QPalette( cg, cg, cg );
-    CHECK_PTR( ttPalette );
 }
 
-
-void QToolTip::cleanup()
+void QToolTip::cleanup() // ## remove 3.0
 {
-    delete ttFont;
-    ttFont = 0;
-    delete ttPalette;
-    ttPalette = 0;
 }
 
 
@@ -639,9 +607,8 @@ void QToolTip::cleanup()
 
 QFont QToolTip::font()
 {
-    if ( !ttFont )
-	initialize();
-    return *ttFont;
+    QTipLabel l("");
+    return QApplication::font( &l );
 }
 
 
@@ -652,9 +619,7 @@ QFont QToolTip::font()
 
 void QToolTip::setFont( const QFont &font )
 {
-    if ( !ttFont )
-	initialize();
-    *ttFont = font;
+    QApplication::setFont( font, TRUE, "QTipLabel" );
 }
 
 
@@ -665,9 +630,8 @@ void QToolTip::setFont( const QFont &font )
 
 QPalette QToolTip::palette()
 {
-    if ( !ttPalette )
-	initialize();
-    return *ttPalette;
+    QTipLabel l("");
+    return QApplication::palette( &l );
 }
 
 
@@ -678,18 +642,15 @@ QPalette QToolTip::palette()
 
 void QToolTip::setPalette( const QPalette &palette )
 {
-    if ( !ttPalette )
-	initialize();
-    *ttPalette = palette;
+    QApplication::setPalette( palette, TRUE, "QTipLabel" );
 }
 
-
 /*!
-  Creates a tool tip object.  This is necessary only if you need tool
+  Constructs a tool tip object.  This is necessary only if you need tool
   tips on regions that can move within the widget (most often because
   the widget's contents can scroll).
 
-  \a parent is widget you want to add dynamic tool tips to and \a
+  \a parent is the widget you want to add dynamic tool tips to and \a
   group (optional) is the tool tip group they should belong to.
 
   \sa maybeTip().
@@ -700,7 +661,8 @@ QToolTip::QToolTip( QWidget * parent, QToolTipGroup * group )
     p = parent;
     g = group;
     initTipManager();
-    tipManager->add( p, entireWidget(), 0, g, 0, this, FALSE );
+    tipManager->add( p, entireWidget(),
+		     QString::null, g, QString::null, this, FALSE );
 }
 
 
@@ -713,10 +675,11 @@ QToolTip::QToolTip( QWidget * parent, QToolTipGroup * group )
   and so on.
 */
 
-void QToolTip::add( QWidget *widget, const char *text )
+void QToolTip::add( QWidget *widget, const QString &text )
 {
     initTipManager();
-    tipManager->add( widget, entireWidget(), text, 0, 0, 0, FALSE );
+    tipManager->add( widget, entireWidget(),
+		     text, 0, QString::null, 0, FALSE );
 }
 
 
@@ -730,8 +693,8 @@ void QToolTip::add( QWidget *widget, const char *text )
   Normally, \a longText is shown in a status bar or similar.
 */
 
-void QToolTip::add( QWidget *widget, const char *text,
-		    QToolTipGroup *group, const char *longText )
+void QToolTip::add( QWidget *widget, const QString &text,
+		    QToolTipGroup *group, const QString& longText )
 {
     initTipManager();
     tipManager->add( widget, entireWidget(), text, group, longText, 0, FALSE );
@@ -757,10 +720,10 @@ void QToolTip::remove( QWidget * widget )
   string.
 */
 
-void QToolTip::add( QWidget * widget, const QRect & rect, const char * text )
+void QToolTip::add( QWidget * widget, const QRect & rect, const QString &text )
 {
     initTipManager();
-    tipManager->add( widget, rect, text, 0, 0, 0, FALSE );
+    tipManager->add( widget, rect, text, 0, QString::null, 0, FALSE );
 }
 
 
@@ -776,8 +739,8 @@ void QToolTip::add( QWidget * widget, const QRect & rect, const char * text )
 */
 
 void QToolTip::add( QWidget *widget, const QRect &rect,
-		    const char *text,
-		    QToolTipGroup *group, const char *groupText )
+		    const QString& text,
+		    QToolTipGroup *group, const QString& groupText )
 {
     initTipManager();
     tipManager->add( widget, rect, text, group, groupText, 0, FALSE );
@@ -798,6 +761,19 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
 }
 
 
+/*!
+  Hides any tip that is currently being shown.
+
+  Normally, there is no need to call this function; QToolTip takes
+  care of showing and hiding the tips as the user moves the mouse.
+*/
+
+void QToolTip::hide()
+{
+    if ( tipManager )
+	tipManager->hideTip();
+}
+
 /*!
   \fn virtual void QToolTip::maybeTip( const QPoint & p);
 
@@ -808,12 +784,12 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
   and must decide whether there is a tool tip for the point \a p in
   the widget this QToolTip object relates to.
 
-  \p is given in that widget's local coordinates.  Most maybeTip()
+  \a p is given in that widget's local coordinates.  Most maybeTip()
   implementation will be of the form:
-  
+
   \code
-    if ( \<something\> ) {
-	tip( \<something\>, \<something\> );
+    if ( \\<something\\> ) {
+	tip( \\<something\\>, \\<something\\> );
     }
   \endcode
 
@@ -825,31 +801,36 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
 
 
 /*!
-  Pop up a tip saying \a text right now, and remove that tip once the
-  cursor moves out of rectangle \a rect (which is given in the
+  Pops up a tip saying \a text right now, and removes that tip once
+  the cursor moves out of rectangle \a rect (which is given in the
   coordinate system of the widget this QToolTip relates to).
 
   The tip will not come back if the cursor moves back; your maybeTip()
   has to reinstate it each time.
 */
 
-void QToolTip::tip( const QRect & rect, const char * text )
+void QToolTip::tip( const QRect & rect, const QString &text )
 {
     initTipManager();
-    tipManager->add( parentWidget(), rect, text, 0, 0, 0, TRUE );
+    tipManager->add( parentWidget(), rect, text, 0, QString::null, 0, TRUE );
 }
 
+void QToolTip::tip( const QRect &geometry, const QRect &rect, const QString &text )
+{
+    initTipManager();
+    tipManager->add( geometry, parentWidget(), rect, text, 0, QString::null, 0, TRUE );
+}
 
 /*!
-  Pop up a tip saying \a text right now, and remove that tip once
+  Pops up a tip saying \a text right now, and removes that tip once
   the cursor moves out of rectangle \a rect.
 
   The tip will not come back if the cursor moves back; your maybeTip()
   has to reinstate it each time.
 */
 
-void QToolTip::tip( const QRect & rect, const char * text,
-		    const char * groupText )
+void QToolTip::tip( const QRect & rect, const QString &text,
+		    const QString& groupText )
 {
     initTipManager();
     tipManager->add( parentWidget(), rect, text, group(), groupText, 0, TRUE );
@@ -857,7 +838,7 @@ void QToolTip::tip( const QRect & rect, const char * text,
 
 
 /*!
-  Remove all tool tips for this widget immediately.
+  Removes all tool tips for this tooltip's parent widget immediately.
 */
 
 void QToolTip::clear()
@@ -886,7 +867,7 @@ void QToolTip::clear()
   isn't a member of any group.
 
   The tool tip group is the object responsible for relaying contact
-  betweeen tool tips and a status bar or something else which can show
+  between tool tips and a status bar or something else which can show
   a longer help text.
 
   \sa parentWidget(), QToolTipGroup
@@ -898,6 +879,8 @@ void QToolTip::clear()
 
   \brief The QToolTipGroup class collects tool tips into natural groups.
 
+  \ingroup helpsystem
+
   Tool tips can display \e two texts, the one in the tip and
   optionally another one, typically in a status bar.  QToolTipGroup
   provides a way to link tool tips to this status bar.
@@ -907,14 +890,14 @@ void QToolTip::clear()
 
   \code
     QToolTipGroup * g = new QToolTipGroup( this, "tool tip relay" );
-    connect( g, SIGNAL(showTip(const char *)),
-             myLabel, SLOT(setText(const char *)) );
+    connect( g, SIGNAL(showTip(const QString&)),
+	     myLabel, SLOT(setText(const QString&)) );
     connect( g, SIGNAL(removeTip()),
-             myLabel, SLOT(clear()) );
+	     myLabel, SLOT(clear()) );
     QToolTip::add( giraffeButton, "feed giraffe",
-                   g, "Give the giraffe a meal" );
+		   g, "Give the giraffe a meal" );
     QToolTip::add( gorillaButton, "feed gorilla",
-                   g, "Give the gorilla a meal" );
+		   g, "Give the gorilla a meal" );
   \endcode
 
   This example makes the object myLabel (which you have to supply)
@@ -925,7 +908,7 @@ void QToolTip::clear()
   Deleting a tool tip group removes the tool tips in it.
 */
 
-/*! \fn void QToolTipGroup::showTip (const char* longText)
+/*! \fn void QToolTipGroup::showTip (const QString &longText)
 
   This signal is emitted when one of the tool tips in the group is
   displayed.  \a longText is the supplementary text for the displayed
@@ -950,7 +933,8 @@ void QToolTip::clear()
 QToolTipGroup::QToolTipGroup( QObject *parent, const char *name )
     : QObject( parent, name )
 {
-    // nothing
+    del = TRUE;
+    ena = TRUE;
 }
 
 
@@ -963,3 +947,81 @@ QToolTipGroup::~QToolTipGroup()
     if ( tipManager )
 	tipManager->removeFromGroup( this );
 }
+
+
+/*!  Returns TRUE if the group text is shown delayed (at the same time
+as the tip) and FALSE if it is shown immediately.
+
+\sa setDelay()
+*/
+
+bool QToolTipGroup::delay() const
+{
+    return del;
+}
+
+
+/*!  Sets the group to show its text immediately if \a enable is
+FALSE, and delayed (at the same time as the tip text) if \a enable is
+TRUE.  The default is TRUE.
+
+\sa delay()
+*/
+
+void QToolTipGroup::setDelay( bool enable )
+{
+#if 0
+    if ( enable && !del ) {
+	// maybe we should show the text at once?
+    }
+#endif
+    del = enable;
+}
+
+/*!
+  Sets the group to be enabled (all tool tips in the group show
+  when activated),
+  or disabled (tool tips in the group are never shown).
+
+  \sa QToolTip::setEnabled(), enabled()
+*/
+
+void QToolTipGroup::setEnabled( bool enable )
+{
+    ena = enable;
+}
+
+/*!
+  Returns whether tooltips in the group are enabled.
+  \sa setEnabled()
+*/
+bool QToolTipGroup::enabled() const
+{
+    return (bool)ena;
+}
+
+/*!
+  Sets the all tool tips to be enabled (shown when needed)
+  or disabled (never shown).
+
+  By default, tool tips are enabled. Note that this function
+  effects all tooltips in the entire application.
+
+  \sa QToolTipGroup::setEnabled()
+*/
+
+void QToolTip::setEnabled( bool enable )
+{
+    globally_enabled = enable;
+}
+
+/*!
+  Returns whether tooltips are enabled globally.
+  \sa setEnabled()
+*/
+bool QToolTip::enabled()
+{
+    return globally_enabled;
+}
+
+#include "qtooltip.moc"
